@@ -17,114 +17,92 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: banshee.c,v 1.11 2003/04/16 11:14:22 lordnaga Exp $
+    $Id: banshee.c,v 1.2 2001/08/31 13:32:28 alor Exp $
 */
 
-#ifdef CYGWIN
-   #include <windows.h>
-   #include <winsock2.h>
-#else
-   #include <sys/types.h>
-   #include <sys/time.h>
-   #include <unistd.h>
-#endif
-
 #include <stdio.h>
+#include <unistd.h>
+#include <fcntl.h>
 #include <stdlib.h>
 
 #include "../../src/include/ec_main.h"
-#include "../../src/include/ec_version.h"
 #include "../../src/include/ec_plugins.h"
 #include "../../src/include/ec_inet_structures.h"
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 
+char Ettercap_Version[] = VERSION;                       // required -- for compatibility
+
+char PlugIn_Info[] = "They kill without discretion...";
+
+short PlugIn_Version = 12;
+
 typedef struct killing
 {
    int source_ip;
    int dest_ip;
-   u_short source_port;
-   u_short dest_port;
+   short source_port;
+   short dest_port;
 } KILLING;
 
+int Banshee_ToBeKilled(int source, int psource, int dest, int pdest, KILLING *data);
+void PlugIn_Start(char**argv);
 
-// protos...
+//---------------------------------
 
-int Plugin_Init(void *);
-int Plugin_Fini(void *);
-int banshee(void *dummy);
-int Banshee_ToBeKilled(int source, u_short psource, int dest, u_short pdest, KILLING *data);
-
-// plugin operation
-
-struct plugin_ops banshee_ops = {
-   ettercap_version: VERSION,
-   plug_info:        "They kill without discretion...",
-   plug_version:     16,
-   plug_type:        PT_EXT,
-   hook_point:       HOOK_NONE,
-   hook_function:    &banshee,
-};
-
-//==================================
-
-int Plugin_Init(void *params)
-{
-   return Plugin_Register(params, &banshee_ops);
-}
-
-int Plugin_Fini(void *params)
-{
-   return 0;
-}
-
-// =================================
-
-int banshee(void *dummy)
+void PlugIn_Start(char**argv)
 {
    int sock, MTU, len;
    ETH_header *eth;
    IP_header *ip;
    TCP_header *tcp;
    char source[25], dest[25];
-   char source_ip[20], dest_ip[20];
+   char source_ip[16], dest_ip[16];
    u_long IPS=0, IPD=0;
-   u_short PS=0, PD=0;
+   int PS=0, PD=0;
    KILLING data;
    char c[1]="";
-   struct recv_packet recvpck;
-   char cont[10];
+   char *buf;
+   char cont[5];
 
 
-   if (!strcmp(Host_Source.ip, "")) {
+   if (!strcmp(Host_Source.ip, ""))
+   {
       Plugin_Output("\nEnter a source ip:port (0.0.0.0:0 for all): ");
       Plugin_Input(source, 25, P_BLOCK);
-      sscanf(source, "%16[^:]:%d", source_ip, &PS);
-   } else {
-      strlcpy(source_ip, Host_Source.ip, sizeof(source_ip));
+   }
+   else
+   {
+      strcpy(source, Host_Source.ip);
    }
 
-   if (!strcmp(Host_Dest.ip, "")) {
+   if (!strcmp(Host_Dest.ip, ""))
+   {
       Plugin_Output("\nEnter a dest ip:port (0.0.0.0:0 for all): ");
       Plugin_Input(dest, 25, P_BLOCK);
-      sscanf(dest, "%16[^:]:%d", dest_ip, &PD);
-   } else {
-      strlcpy(dest_ip, Host_Dest.ip, sizeof(dest_ip));
    }
+   else
+   {
+      strcpy(source, Host_Source.ip);
+   }
+
+   sscanf(source, "%16[^:]:%d", source_ip, &PS);
+   sscanf(dest, "%16[^:]:%d", dest_ip, &PD);
 
    IPS = inet_addr(source_ip);
    IPD = inet_addr(dest_ip);
 
-   memset(cont, 0, 10);
 
-   Plugin_Output("\nAre you sure you want to kill from %s:%d to ", int_ntoa(IPS), PS);
-   Plugin_Output("%s:%d ? (yes/no) ", int_ntoa(IPD), PD);
-   Plugin_Input(cont, 5, P_BLOCK);
-   if (strncmp(cont, "yes", 3))  // not sure... ;)
-   {
+    memset(cont, 0, 5);
+
+    Plugin_Output("\nAre you sure you want to kill from %s:%d to ", int_ntoa(IPS), PS);
+    Plugin_Output("%s:%d ? (yes/no) ", int_ntoa(IPD), PD);
+    Plugin_Input(cont, 4, P_BLOCK);
+    if (strncmp(cont, "yes", 3))  // not sure... ;)
+    {
       Plugin_Output("\nIt is safe!  for now...\n");
-      return 0;
-   }
+      return;
+    }
 
    Plugin_Output("\nKilling all connection from %s:%d to ", int_ntoa(IPS), PS);
    Plugin_Output("%s:%d ... (pres return to stop)\n\n", int_ntoa(IPD), PD);
@@ -138,26 +116,28 @@ int banshee(void *dummy)
    if (Options.normal || number_of_connections == 0)  // form command line or in host list interface...
       Inet_SetPromisc(Options.netiface);
 
-   Inet_SetNonBlock(sock);
+   fcntl(sock,F_SETFL,O_NONBLOCK);
 
-   recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD);
-   recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
+   buf = Inet_Forge_packet( MTU );
 
-   LOOP {
+   loop
+   {
       memset(&data, 0, sizeof(KILLING));
-      memset(recvpck.aligned, 0, MTU);
+      memset(buf, 0, MTU);
 
-      len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, NULL);
+      len = Inet_GetRawPacket(sock, buf, MTU, NULL);
 
       if (len > 0)
       {
-         eth = (ETH_header *)recvpck.aligned;
+         eth = (ETH_header *)buf;
+
          if ( ntohs(eth->type) == ETH_P_IP )
          {
             ip = (IP_header *)(eth+1);
 
             data.source_ip = ip->source_ip;
             data.dest_ip = ip->dest_ip;
+
             if ( ip->proto == IPPROTO_TCP)
             {
                unsigned char *payload;
@@ -169,6 +149,7 @@ int banshee(void *dummy)
 
                data.source_port = tcp->source;
                data.dest_port = tcp->dest;
+
                if (Banshee_ToBeKilled(IPS, PS, IPD, PD, &data))
                {
                   u_char *kbuf;
@@ -186,7 +167,6 @@ int banshee(void *dummy)
                                                                  TH_RST,
                                                                  0, 0 );
 
-
                   Inet_SendRawPacket(sock, kbuf, ETH_HEADER + IP_HEADER + TCP_HEADER );
 
                   Inet_Forge_ethernet( kbuf, eth->source_mac, eth->dest_mac, ETH_P_IP );
@@ -214,17 +194,16 @@ int banshee(void *dummy)
       if (Plugin_Input(c, 1, P_NONBLOCK))
       {
          Plugin_Output("Exiting... \n");
-         Inet_Forge_packet_destroy( recvpck.buf );
+         Inet_Forge_packet_destroy( buf );
          Inet_CloseRawSock(sock);
          break;
       }
    } // end loop
 
-   return 0;
 }
 
 
-int Banshee_ToBeKilled(int source, u_short psource, int dest, u_short pdest, KILLING *data)
+int Banshee_ToBeKilled(int source, int psource, int dest, int pdest, KILLING *data)
 {
    char s=0, ps=0, d=0, pd=0;
 
