@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: lurker.c,v 1.2 2001/08/31 13:32:28 alor Exp $
 */
 
 #include <stdlib.h>
@@ -30,10 +32,14 @@
 
 char Ettercap_Version[] = VERSION;                       // required -- for compatibility
 
-char PlugIn_Info[] = "try to search for other ettercap";
+char PlugIn_Info[] = "Try to search for other ettercap";
 
-short PlugIn_Version = 11;
+short PlugIn_Version = 13;
 
+// protos....
+
+void Parse_packet(char *buffer);
+void PlugIn_Start(char**argv);
 
 // ---------------------------
 
@@ -77,7 +83,10 @@ void Parse_packet(char *buffer)        // very lame searching...
                Plugin_Output("\n%s is shadowing (scanning) %s ...\n", IPS, IPD );
                break;
             case 0xabadc0de:
-               Plugin_Output("\n%s is golemizing %s ...\n", IPS, IPD );
+            	if ( ntohs(ip->ident) == 0xe77e && ntohl(tcp->ack_seq) == 0xabadc0de)
+            		Plugin_Output("\nSpectre is flooding the LAN...\n");
+            	else
+               	Plugin_Output("\n%s is golemizing %s ...\n", IPS, IPD );
                break;
          }
       }
@@ -118,7 +127,7 @@ void PlugIn_Start(char**argv)
     }
 
     Inet_Forge_packet_destroy( recv_pck );
-    close(sock);
+    Inet_CloseRawSock(sock);
 }
 
 /* EOF */
