@@ -2,7 +2,7 @@
     shadow -- ettercap plugin -- simple port scan
 
     Copyright (C) 2001  NaGoR
-    
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -17,27 +17,29 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: shadow.c,v 1.7 2002/10/01 21:40:53 alor Exp $
+    $Id: shadow.c,v 1.2 2001/08/31 13:32:28 alor Exp $
 */
-#ifdef CYGWIN
-   #include <windows.h>
-   #include <winsock2.h> 
-#else
-   #include <sys/time.h>
-   #include <unistd.h>
-#endif
 
 #include <stdlib.h>
+#include <fcntl.h>
 #include <time.h>
+#include <sys/time.h>
+#include <unistd.h>
 
 #include "../../src/include/ec_main.h"
-#include "../../src/include/ec_version.h"
 #include "../../src/include/ec_plugins.h"
 #include "../../src/include/ec_inet_structures.h"
 #include "../../src/include/ec_inet.h"
 #include "../../src/include/ec_inet_forge.h"
 #include "../../src/include/ec_decodedata.h"
 
+char Ettercap_Version[] = VERSION;                       // required -- for compatibility
+
+char PlugIn_Info[] = "A very simple SYN/TCP port scanner";
+
+short PlugIn_Version = 18;
+
+
 #define SYN_SEQ 6969
 
 // global data...
@@ -45,37 +47,12 @@
 int IPS, IPD, port_index=0;
 int *PORTREP;
 
-// protos...
+// protos....
 
-int Plugin_Init(void *);
-int Plugin_Fini(void *);
-int shadow_main(void *dummy);
 void Parse_packet(char *buffer);
+void PlugIn_Start(char**argv);
 
-// plugin operation
-
-struct plugin_ops shadow_ops = {
-   ettercap_version: VERSION,
-   plug_info:        "A very simple SYN/TCP port scanner",
-   plug_version:     18,
-   plug_type:        PT_EXT,
-   hook_point:       HOOK_NONE,
-   hook_function:    &shadow_main,
-};
-
-//==================================
-
-int Plugin_Init(void *params)
-{
-   return Plugin_Register(params, &shadow_ops);
-}
-
-int Plugin_Fini(void *params)
-{
-   return 0;
-}
-
-// =================================
+// ---------------------------
 
 void Parse_packet(char *buffer)
 {
@@ -102,26 +79,26 @@ void Parse_packet(char *buffer)
 }
 
 
-int shadow_main(void *dummy)
+void PlugIn_Start(char**argv)
 {
     int i, sock, MTU, startP, finP, OldIndex=0;
     unsigned short PORTS, IP_ID;
     unsigned char MACS[6];
     unsigned char MACD[6];
     char numero[10];
-    struct recv_packet recvpck;
+    char *pck_to_send;
     TIME_DECLARE;
 
     if (!strcmp(Host_Dest.ip,""))
     {
       Plugin_Output("Please select a Dest...\n");
-      return 0;
+      return;
     }
 
     if (!strcmp(Host_Dest.ip, Inet_MyIPAddress()))
     {
        Plugin_Output("\nThis plugin doesn't work on myself !!\n");
-       return 0;
+       return;
     }
 
     Plugin_Output("\nStarting Port: ");
@@ -135,7 +112,7 @@ int shadow_main(void *dummy)
     if (startP>finP)
     {
 		Plugin_Output("\nStopping Port must be greater than Starting Port\n");
-		return 0;
+		return;
     }
 
     IPD = inet_addr(Host_Dest.ip);
@@ -148,31 +125,29 @@ int shadow_main(void *dummy)
     memset(PORTREP,0,(finP-startP+10)*sizeof(int));
     srand(time(0));
     IP_ID = PORTS = rand()%(0xFFFE)+1;
-   
-    recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD);
-    recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
-      
-    Inet_Forge_ethernet( recvpck.aligned, MACS, MACD, ETH_P_IP );
-    Inet_Forge_ip( recvpck.aligned + ETH_HEADER, IPS, IPD, TCP_HEADER, IP_ID++, 0, IPPROTO_TCP);
+
+    pck_to_send = (char *)Inet_Forge_packet(MTU);
+
+    Inet_Forge_ethernet( pck_to_send, MACS, MACD, ETH_P_IP );
+    Inet_Forge_ip( pck_to_send + ETH_HEADER, IPS, IPD, TCP_HEADER, IP_ID++, 0, IPPROTO_TCP);
 
     for (i=startP; i<=finP; i++)
     {
-       Inet_Forge_tcp( recvpck.aligned + ETH_HEADER + IP_HEADER, PORTS, i,  SYN_SEQ, 0, TH_SYN, 0, 0);
-       Inet_SendRawPacket(sock, recvpck.aligned, ETH_HEADER + IP_HEADER + TCP_HEADER );
+       Inet_Forge_tcp( pck_to_send + ETH_HEADER + IP_HEADER, PORTS, i,  SYN_SEQ, 0, TH_SYN, 0, 0);
+       Inet_SendRawPacket(sock, pck_to_send, ETH_HEADER + IP_HEADER + TCP_HEADER );
        if (!(i%5)) usleep(500);
     }
 
     Plugin_Output("\n\nScanning %s  %d -> %d ...\n\n",Host_Dest.ip, startP, finP);
 
+    fcntl(sock, F_SETFL, O_NONBLOCK);
 
-    Inet_SetNonBlock(sock);
-    
     TIME_START;
 
     do
     {
-        Inet_GetRawPacket(sock, recvpck.aligned, MTU, NULL);
-        Parse_packet(recvpck.aligned);
+        Inet_GetRawPacket(sock, pck_to_send, MTU, NULL);
+        Parse_packet(pck_to_send);
 
         TIME_FINISH;
 
@@ -185,10 +160,9 @@ int shadow_main(void *dummy)
 
     } while (TIME_ELAPSED < 2);
 
-    Inet_Forge_packet_destroy( recvpck.buf );
+    Inet_Forge_packet_destroy( pck_to_send );
     free(PORTREP);
     Inet_CloseRawSock(sock);
-    return 0;
 }
 
 /* EOF */
