@@ -19,20 +19,12 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_BSD.c,v 1.19 2002/06/12 16:19:35 alor Exp $
+    $Id: ec_inet_BSD.c,v 1.1.1.1 2001/08/27 19:27:19 alor Exp $
 */
 
-#include "../../include/ec_main.h"
 
-#include "../../include/ec_inet.h"
-#include "../../include/ec_thread.h"
-#include "../../include/ec_buffer.h"
-#include "../../include/ec_inet_forge.h"
-#include "../../include/ec_inet_structures.h"
+// This file is included from ../ec_inet.c
 
-#include <ctype.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
 
 #include <sys/sysctl.h>
 #include <sys/param.h>
@@ -60,66 +52,26 @@
 #endif
 
 
-int IpForward_status;      // old ipforward status
-
-int bpf_in_use;
+int fdprom;
 int size;
 int SocketBuffer = -1;
 
-static char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};
-
-// protos...
-
-int Inet_FindIFace(char *iface);
-int Inet_CorrectIface(char *iface);
-int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, u_long *IP, u_long *NetMask);
-int Inet_SetPromisc(char *iface);
-int Inet_OpenRawSock(char *iface);
-void Inet_CloseRawSock(int sock);
-int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type);
-int Inet_SendRawPacket(int sock, char *buffer, int len);
-void Inet_SetNonBlock(int sock);
-void Inet_Restore_ifr(void);
-void Inet_DisableForwarding(void);
-void Inet_RestoreForwarding(void);
-char *Inet_MacFromIP(unsigned long ip);
-#ifdef PERMIT_HTTPS
-   void Inet_UnSetARPEntry(void);
-   int Inet_SetARPEntry(unsigned long IP, char MAC[6]);
-   void Inet_UnsetRoute(void);
-   void Inet_SetRoute(void);
-#endif
-
-
-// ----------------------------------------------
 
 int Inet_FindIFace(char *iface)
 {
 #ifdef HAVE_GETIFADDRS
    struct ifaddrs *ifalist, *ifa, *ifp = NULL;
-   int minunit, n;
-   char *cp;
 
    if (getifaddrs(&ifalist) != 0)
-      ERROR_MSG("getifaddrs()");
+      Error_msg("ec_inet_BSD:%d  getifaddrs() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
-   minunit = 666;
-   
-   for (ifa = ifalist; ifa; ifa = ifa->ifa_next) {
-      if ((ifa->ifa_flags & IFF_UP) == 0 || (ifa->ifa_flags & IFF_LOOPBACK) || (ifa->ifa_flags & IFF_POINTOPOINT))
+
+   for (ifa = ifalist; ifa; ifa = ifa->ifa_next)
+   {
+      if ((ifa->ifa_flags & IFF_UP) == 0 || (ifa->ifa_flags & IFF_LOOPBACK))
          continue;
-      
-      if ( !strncmp(ifa->ifa_name, "pflog", 5) )
-         continue;
-      
-      for (cp = ifa->ifa_name; !isdigit(*cp); ++cp)
-         continue;
-      
-      n = atoi(cp);
-      if (n < minunit) {
-         minunit = n;
-         ifp = ifa;
-      }
+
+      ifp = ifa;
    }
 
    if (ifp == NULL)  // no device found
@@ -128,9 +80,11 @@ int Inet_FindIFace(char *iface)
       return -1;
    }
 
-   strlcpy(iface, ifp->ifa_name, sizeof(Options.netiface));
+   strcpy(iface, ifp->ifa_name);
 
-   DEBUG_MSG("Inet_FindIFace -- %s found !!", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- %s found !!", iface);
+#endif
 
    free(ifalist);
    return 0;
@@ -149,20 +103,20 @@ int Inet_FindIFace(char *iface)
 
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_linux:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    buf_size = 8192;
 
    for (;;) {
       buf = malloc (buf_size);
       if (buf == NULL)
-         ERROR_MSG("malloc()");
+         Error_msg("ec_Inet_linux:%d malloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       ifc.ifc_len = buf_size;
       ifc.ifc_buf = buf;
       memset (buf, 0, buf_size);
       if (ioctl(fd, SIOCGIFCONF, (char *)&ifc) < 0 && errno != EINVAL)
-         ERROR_MSG("ioctl(SIOCGIFCONF)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFCONF) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       if (ifc.ifc_len < buf_size)
          break;
@@ -198,13 +152,15 @@ int Inet_FindIFace(char *iface)
       {
          if (errno == ENXIO)
             continue;
-         ERROR_MSG("ioctl(SIOCGIFFLAGS)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFFLAGS) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
       }
 
-      DEBUG_MSG("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#endif
 
       /* Must be up and not the loopback */
-      if ((ifr.ifr_flags & IFF_UP) == 0 || (ifr.ifr_flags & IFF_LOOPBACK) != 0  || !strncmp(ifr.ifr_name, "pflog", 5))
+      if ((ifr.ifr_flags & IFF_UP) == 0 || (ifr.ifr_flags & IFF_LOOPBACK) != 0)
          continue;
 
       endcp = ifrp->ifr_name + strlen(ifrp->ifr_name);
@@ -230,11 +186,13 @@ int Inet_FindIFace(char *iface)
       return -1;
    }
 
-   strlcpy(iface, mp->ifr_name, sizeof(Options.netiface));
+   strcpy(iface, mp->ifr_name);
 
    free(buf);
 
-   DEBUG_MSG("Inet_FindIFace -- %s found !!", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- %s found !!", iface);
+#endif
 
    return 0;
 
@@ -243,20 +201,22 @@ int Inet_FindIFace(char *iface)
 
 
 
-int Inet_CorrectIface(char *iface)
+char Inet_CorrectIface(char *iface)
 {
 #ifdef HAVE_GETIFADDRS
    struct ifaddrs *ifalist, *ifa;
 
-   DEBUG_MSG("Inet_CorrectIface -- %s ", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_CorrectIface -- %s ", iface);
+#endif
 
    if (getifaddrs(&ifalist) != 0)
-      ERROR_MSG(" getifaddrs()");
+      Error_msg("ec_inet_BSD:%d  getifaddrs() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
    for (ifa = ifalist; ifa; ifa = ifa->ifa_next)
    {
-      if ( (ifa->ifa_flags & IFF_UP) == 0 || (ifa->ifa_flags & IFF_LOOPBACK) != 0 || strcmp(iface, ifa->ifa_name) )
+      if ( (ifa->ifa_flags & IFF_UP) == 0 || strcmp(iface, ifa->ifa_name) )
          continue;
 
       free(ifalist);
@@ -273,11 +233,13 @@ int Inet_CorrectIface(char *iface)
    int sock;
    struct ifreq ifr;
 
-   DEBUG_MSG("Inet_CorrectIface\t\tIface: %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_CorrectIface\t\tIface: %s", iface);
+#endif
 
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_linux:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
@@ -293,12 +255,6 @@ int Inet_CorrectIface(char *iface)
       return -1;
    }
 
-   if (ifr.ifr_flags & IFF_LOOPBACK )                    // check for loopback
-   {
-      Options.normal = 1;
-      Error_msg("Ettercap can't be run on loopback device");
-   }
-
    if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )             // check for alias
    {
       close(sock);
@@ -327,7 +283,9 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    {
       if ( ioctl(sock, SIOCGIFMTU, &ifr) < 0)
       {
-         DEBUG_MSG("Inet_GetIfaceInfo -- MTU FAILED... assuming 1500");
+         #ifdef DEBUG
+            Debug_msg("Inet_GetIfaceInfo -- MTU FAILED... assuming 1500");
+         #endif
          *MTU = 1500;
       }
       else
@@ -351,13 +309,13 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
       mib[5] = 0;
 
       if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0)
-         ERROR_MSG("sysctl()");
+         Error_msg("ec_Inet_BSD:%d sysctl() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
       if ((buf = (char *)malloc(len)) == NULL )
-         ERROR_MSG("malloc()");
+         Error_msg("ec_Inet_BSD:%d malloc() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
       if (sysctl(mib, 6, buf, &len, NULL, 0) < 0)
-         ERROR_MSG("sysctl()");
+         Error_msg("ec_Inet_BSD:%d sysctl() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
       end = buf + len;
 
@@ -381,14 +339,14 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    if (IP != NULL)
    {
       if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFADDR)");
+         Error_msg("ec_Inet_BSD:%d ioctl(SIOCGIFADDR) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)IP, ifr.ifr_addr.sa_data+2, 4);
    }
 
    if (NetMask != NULL)
    {
       if ( ioctl(sock, SIOCGIFNETMASK, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFNETMASK)");
+         Error_msg("ec_Inet_BSD:%d ioctl(SIOCGIFNETMASK) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)NetMask, ifr.ifr_addr.sa_data+2, 4);
       if (strcmp(Options.netmask, ""))       // specified on command line
          *NetMask = inet_addr(Options.netmask);
@@ -402,21 +360,15 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
 }
 
 
-void Inet_SetNonBlock(int sock)
-{
-   DEBUG_MSG("Inet_SetNonBlock fd = %d", sock);
-
-   fcntl(sock, F_SETFL, O_NONBLOCK);
-
-}
-
 
 void Inet_CloseRawSock(int sock)
 {
 
-   DEBUG_MSG("Inet_CloseRawSock \t wrappered to NULL");
+#ifdef DEBUG
+   Debug_msg("Inet_CloseRawSock \t fd = %d", sock);
+#endif
 
-   //close(sock);    // the bpf is close when ettercap exits
+   close(sock);
 
 }
 
@@ -452,12 +404,6 @@ int Inet_OpenRawSock(char *iface)
     u_int spoof_eth_src = 1;
 #endif
 
-   if (bpf_in_use != 0) {
-      DEBUG_MSG("Inet_OpenRawSock %s", iface);
-      DEBUG_MSG("Inet_OpenRawSock \t bpf_in_use = %d ", bpf_in_use);
-      return bpf_in_use;
-   }
-
    Inet_GetIfaceInfo(iface, NULL, MyMAC, NULL, NULL);
 
    insns[1].k = htons(*(short *)MyMAC);         // put MyMac in the filter...
@@ -465,7 +411,10 @@ int Inet_OpenRawSock(char *iface)
    insns[5].k = htons(*(short *)(MyMAC+4));
 
 
-   DEBUG_MSG("Inet_OpenRawSock %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_OpenRawSock %s", iface);
+#endif
+
 
    do    // find an available bpf device
    {
@@ -474,12 +423,14 @@ int Inet_OpenRawSock(char *iface)
    } while (fd < 0 && errno == EBUSY);
 
    if (fd < 0)
-      Error_msg("ec_inet_bsd:%d no /dev/bpf* available (tried to open %d) | ERRNO : %d | %s", __LINE__, i, errno, strerror(errno));
+      Error_msg("ec_inet_BSD:%d  no /dev/bpf* available (tried to open %d) | ERRNO : %d | %s", __LINE__, i, errno, strerror(errno));
 
-   DEBUG_MSG("Inet_OpenRawSock \t fd = %d -- /dev/bpf%d ", fd, i-1);
+#ifdef DEBUG
+   Debug_msg("Inet_OpenRawSock \t fd = %d -- /dev/bpf%d ", fd, i-1);
+#endif
 
    if (ioctl(fd, BIOCVERSION, (caddr_t)&bv) < 0)      // get bpf version
-      ERROR_MSG(" ioctl(BIOCVERSION)");
+      Error_msg("ec_inet_BSD:%d  ioctl(BIOCVERSION) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
    if (bv.bv_major != BPF_MAJOR_VERSION || bv.bv_minor < BPF_MINOR_VERSION)
@@ -494,35 +445,35 @@ int Inet_OpenRawSock(char *iface)
          break;  /* that size worked; we're done */
 
       if (errno != ENOBUFS)
-         ERROR_MSG(" ioctl(BIOCSETIF)");
+         Error_msg("ec_inet_BSD:%d  ioctl(BIOCSETIF) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
    }
 
    if (size == 0) Error_msg("BIOCSBLEN: No buffer size worked");
 
    if (ioctl(fd, BIOCGBLEN, (caddr_t)&size) < 0)
-     ERROR_MSG(" ioctl(BIOCGBLEN)");
+     Error_msg("ec_inet_BSD:%d  ioctl(BIOCGBLEN) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    if (ioctl(fd, BIOCGDLT, (caddr_t)&type) == -1)           // Get the data link layer type.
-      ERROR_MSG(" ioctl(BIOCGDLT)");
+      Error_msg("ec_inet_BSD:%d  ioctl(BIOCGDLT) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    if (type != DLT_EN10MB)
-      Error_msg("%s : Interface not supported ( only DLT_EN10MB) | %d", iface, type);
+      Error_msg("Interface not supported ( only DLT_EN10MB) | %d", type);
 
-#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT)     // auto fill the source mac address now set to OFF
+#if defined(BIOCGHDRCMPLT) && defined(BIOCSHDRCMPLT)     // auoto fill the source mac address now set to OFF
     if (ioctl(fd, BIOCSHDRCMPLT, &spoof_eth_src) == -1)
-      ERROR_MSG(" ioctl(BIOCSHDRCMPLT)");
+      Error_msg("ec_inet_BSD:%d  ioctl(BIOCSHDRCMPLT) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 #endif
 
    i = 1;
    if (ioctl(fd, BIOCIMMEDIATE, &i) < 0)                    // Set immediate mode so packets are processed as they arrive.
-      ERROR_MSG(" ioctl(BIOCIMMEDIATE)");
+      Error_msg("ec_inet_BSD:%d  ioctl(BIOCIMMEDIATE) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
    if (ioctl(fd, BIOCSETF, (caddr_t)&filter) < 0)           // Set filter program.
-      ERROR_MSG(" ioctl(BIOCSETF)");
+      Error_msg("ec_inet_BSD:%d  ioctl(BIOCSETF) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
-   bpf_in_use = fd;
+   fdprom = fd;
 
    return fd;
 
@@ -537,10 +488,7 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
    static char MyMAC[6]={0x65,0x74,0x74,0x65,0x72,0x63};
 
    if (SocketBuffer == -1)                   // only the first time
-   {
       SocketBuffer = Buffer_Create(1.0e5);   // 100 K buffer
-      DEBUG_MSG("Inet_GetRawPacket creates the buffer for the first time -- buf id = %d", SocketBuffer);
-   }
 
    Buffer_Get(SocketBuffer, &pktlen, sizeof(u_int));
    len = Buffer_Get(SocketBuffer, buffer, pktlen );
@@ -550,9 +498,9 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
        if (!strncmp(MyMAC,"etterc",6))    // only the first time...
            Inet_GetIfaceInfo(Options.netiface, NULL, MyMAC, NULL, NULL);
        if (!memcmp(MyMAC,buffer,6))
-           *type = PACKET_HOST;
+           *type = 0; // PACKET_HOST
        else
-           *type = !PACKET_HOST;
+           *type = 1; // !PACKET_HOST
    }
 
    if (len > 0) return len;                     // there was pending fata.
@@ -586,9 +534,9 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
    if (type != NULL)
    {
       if (!memcmp(MyMAC,buffer,6))
-           *type = PACKET_HOST;
-       else
-           *type = !PACKET_HOST;
+         *type = 0; // PACKET_HOST
+      else
+         *type = 1; // !PACKET_HOST
    }
 
    free(buf);
@@ -605,19 +553,12 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
    int sent;
 
    sent = write(sock, buffer, len);
-   if (sent < len)
-   {
-      while (errno == ENOBUFS)
-      {
-         usleep(5000);
-         sent = write(sock, buffer, len);
-         if (sent == len) return (sent);
-      }
 
-      Error_msg("ec_inet_bsd:%d write() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
-   }
+   if (sent < len)
+      Error_msg("ec_Inet_BSD:%d write() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
 
    return (sent);
+
 }
 
 
@@ -625,10 +566,12 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 int Inet_SetPromisc(char *iface)
 {
 
-   DEBUG_MSG("Inet_SetPromisc %s %d", iface, bpf_in_use);
+#ifdef DEBUG
+   Debug_msg("Inet_SetPromisc %s %d", iface, fdprom);
+#endif
 
-   if ( ioctl(bpf_in_use, BIOCPROMISC, NULL) < 0 )
-      ERROR_MSG("ioctl(BIOCPROMISC)");
+   if ( ioctl(fdprom, BIOCPROMISC, NULL) < 0 )
+      Error_msg("ec_Inet_BSD:%d ioctl(BIOCPROMISC) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    return 0;
 }
@@ -660,11 +603,13 @@ void Inet_DisableForwarding(void)
    len = sizeof(IpForward_status);
 
    if( (sysctl(mib, 4, &IpForward_status, &len, &val, sizeof(val))) == -1)
-    ERROR_MSG("sysctl() | net.inet.ip.forwarding");
+    Error_msg("ec_inet_BSD:%d sysctl() | net.inet.ip.forwarding | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
-   DEBUG_MSG("Inet_DisableForwarding | net.inet.ip.forwarding = %d  old_value = %d\n", val, IpForward_status);
+#ifdef DEBUG
+   Debug_msg("Inet_DisableForwarding | net.inet.ip.forwarding = %d  old_value = %d\n", val, IpForward_status);
+#endif
 
-   atexit(Inet_RestoreForwarding);
+   exit_func(Inet_RestoreForwarding);
 
 }
 
@@ -680,12 +625,15 @@ void Inet_RestoreForwarding(void)
    mib[2] = IPPROTO_IP;
    mib[3] = IPCTL_FORWARDING;
 
-   if (strcmp(ECThread_getname(pthread_self()), PROGRAM)) return;
+   if (!strstr(program_argv0, PROGRAM)) return;
 
    if( (sysctl(mib, 4, NULL, NULL, &IpForward_status, sizeof(IpForward_status))) == -1)
-      ERROR_MSG("sysctl()");
+      Error_msg("ec_inet_BSD:%d sysctl() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
+
+#ifdef DEBUG
+   Debug_msg("Inet_RestoreForwarding | net.inet.ip.forwarding = %d\n", IpForward_status);
+#endif
 
-   DEBUG_MSG("Inet_RestoreForwarding | net.inet.ip.forwarding = %d\n", IpForward_status);
 
 }
 
@@ -700,7 +648,9 @@ char *Inet_MacFromIP(unsigned long ip)
    struct sockaddr_dl *sdl;
    static char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
 
-   DEBUG_MSG("Inet_MacFromIP");
+#ifdef DEBUG
+   Debug_msg("Inet_MacFromIP");
+#endif
 
    mib[0] = CTL_NET;
    mib[1] = AF_ROUTE;
@@ -710,15 +660,15 @@ char *Inet_MacFromIP(unsigned long ip)
    mib[5] = RTF_LLINFO;
 
    if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0)
-      ERROR_MSG("sysctl()");
+      Error_msg("ec_inet_BSD:%d sysctl() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    if ((buf = (char *)malloc(len)) == NULL)
-      ERROR_MSG("malloc()");
+      Error_msg("ec_inet_BSD:%d malloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    if (sysctl(mib, 6, buf, &len, NULL, 0) < 0)
    {
       free(buf);
-      ERROR_MSG("sysctl()");
+      Error_msg("ec_inet_BSD:%d sysctl() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
    }
    end = buf + len;
 
@@ -735,13 +685,15 @@ char *Inet_MacFromIP(unsigned long ip)
       }
       else     // not in cache... try to find it...
       {
-      	struct recv_packet recvpck;
+         u_char *buf;
          char MyMAC[6];
          u_long MyIP;
          int MTU, sock;
          TIME_DECLARE;
 
-         DEBUG_MSG("Inet_MacFromIP -- try to find it");
+#ifdef DEBUG
+   Debug_msg("Inet_MacFromIP -- try to find it");
+#endif
 
          sock = Inet_OpenRawSock(Options.netiface);
 
@@ -749,24 +701,24 @@ char *Inet_MacFromIP(unsigned long ip)
 
          if (ip == MyIP)
          {
-            DEBUG_MSG("Inet_MacFromIP -- try to find me... ;)");
+            #ifdef DEBUG
+               Debug_msg("Inet_MacFromIP -- try to find me... ;)");
+            #endif
             memcpy(LLADDR(sdl), MyMAC, ETHER_ADDR_LEN);
             Inet_CloseRawSock(sock);
             return (char *) LLADDR(sdl);
          }
 
-      	recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD );
-      	recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
+         buf = Inet_Forge_packet( ETH_HEADER + ARP_HEADER );
+         Inet_Forge_ethernet( buf, MyMAC, ETH_BROADCAST, ETH_P_ARP );
 
-         Inet_Forge_ethernet( recvpck.aligned, MyMAC, ETH_BROADCAST, ETH_P_ARP );
-
-         Inet_Forge_arp( recvpck.aligned + ETH_HEADER, ARPOP_REQUEST,
+         Inet_Forge_arp( buf+ETH_HEADER, ARPOP_REQUEST,
                          MyMAC, MyIP,
                          ARP_BROADCAST, ip );
 
          Inet_SendRawPacket(sock, buf, ETH_HEADER + ARP_HEADER);
-
-			memset(recvpck.aligned, 0, MTU);
+         Inet_Forge_packet_destroy( buf );
+         buf = Inet_Forge_packet( MTU );
 
          fcntl(sock, F_SETFL, O_NONBLOCK);
          TIME_START;
@@ -778,10 +730,10 @@ char *Inet_MacFromIP(unsigned long ip)
             ETH_header *ethpkt;
             ARP_header *arppkt;
 
-            len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, &pkttype);
+            len = Inet_GetRawPacket(sock, buf, MTU, &pkttype);
 
-            ethpkt = (ETH_header *)recvpck.aligned;
-            arppkt = (ARP_header *)(recvpck.aligned + ETH_HEADER);
+            ethpkt = (ETH_header *)buf;
+            arppkt = (ARP_header *)(buf + ETH_HEADER);
 
             TIME_FINISH;
 
@@ -790,14 +742,12 @@ char *Inet_MacFromIP(unsigned long ip)
                if ( *(unsigned long *)arppkt->source_ip == ip )
                {
                   memcpy(LLADDR(sdl), &arppkt->source_add, ETHER_ADDR_LEN);
-                  free(buf);
-                  Inet_Forge_packet_destroy( recvpck.buf );
+                  Inet_Forge_packet_destroy( buf );
                   Inet_CloseRawSock(sock);
                   return (char *) LLADDR(sdl);
                }
             }
          } while ( TIME_ELAPSED < 0.5 );
-         Inet_Forge_packet_destroy( recvpck.buf );
          Inet_CloseRawSock(sock);
       }
    }
@@ -880,13 +830,13 @@ doit:
 
    if ((rlen = write(s, (char *)&m_rtmsg, l)) < 0) {
       if (errno != ESRCH || cmd != RTM_DELETE)
-         ERROR_MSG("writing on PF_ROUTE socket");
+         Error_msg("ec_inet_BSD:%d writing on PF_ROUTE socket | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
    }
    do {
       l = read(s, (char *)&m_rtmsg, sizeof(m_rtmsg));
    } while (l > 0 && (rtm->rtm_seq != seq || rtm->rtm_pid != pid));
    if (l < 0)
-      ERROR_MSG("reading on PF_ROUTE socket");
+      Error_msg("ec_inet_BSD:%d reading on PF_ROUTE socket | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    return (0);
 }
@@ -899,11 +849,13 @@ void Inet_UnSetARPEntry(void)
    register struct rt_msghdr *rtm = &m_rtmsg.m_rtm;
    struct sockaddr_dl *sdl;
 
-   DEBUG_MSG("Inet_UnSetARPEntry");
+#ifdef DEBUG
+   Debug_msg("Inet_UnSetARPEntry");
+#endif
 
    s = socket(PF_ROUTE, SOCK_RAW, 0);
    if (s < 0)
-      ERROR_MSG("socket(PF_ROUTE)");
+      Error_msg("ec_inet_BSD:%d socket(PF_ROUTE) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    sin_m = blank_sin;
    sin->sin_addr.s_addr = SavedIP;
@@ -928,13 +880,15 @@ int Inet_SetARPEntry(unsigned long IP, char MAC[6])
    u_char *ea;
    int ret;
 
-   DEBUG_MSG("Inet_SetARPEntry");
+#ifdef DEBUG
+   Debug_msg("Inet_SetARPEntry");
+#endif
 
    pid = getpid();
 
    s = socket(PF_ROUTE, SOCK_RAW, 0);
    if (s < 0)
-      ERROR_MSG("socket(PF_ROUTE)");
+      Error_msg("ec_inet_BSD:%d socket(PF_ROUTE) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    sdl_m = blank_sdl;
    sin_m = blank_sin;
@@ -955,7 +909,7 @@ int Inet_SetARPEntry(unsigned long IP, char MAC[6])
    SavedIP = Inet_Fake_Host();
    memcpy(SavedMAC, MAC, 6);
 
-   atexit(Inet_UnSetARPEntry);
+   exit_func(Inet_UnSetARPEntry);
 
    ret = rtmsg_arp(RTM_ADD);
 
@@ -986,7 +940,7 @@ typedef union sockunion *sup;
 int rtmsg_route(int cmd, int flags);
 void inet_makenetandmask(u_long net, struct sockaddr_in *sin, u_long bits);
 int getaddr(int which, char *s, struct hostent **hpp);
-void mask_addr(void);
+void mask_addr();
 int rtmsg_route(int cmd, int flags);
 
 //====================
@@ -1095,7 +1049,7 @@ netdone:
           MIN(hp->h_length, sizeof(su->sin.sin_addr)));
       return (1);
    }
-   Error_msg("ec_inet_BSD:%d  bad address %s | ERRNO : %d | %s", __LINE__, s, errno, strerror(errno));
+   Error_msg("ec_inet_BSD:%d bad address %s | ERRNO : %d | %s", __LINE__, s, errno, strerror(errno));
    return 0;
 }
 
@@ -1166,7 +1120,7 @@ int rtmsg_route(int cmd, int flags)
    NEXTADDR(RTA_IFA, so_ifa);
    rtm.rtm_msglen = l = cp - (char *)&m_rtmsg;
    if ((rlen = write(s, (char *)&m_rtmsg, l)) < 0)
-      ERROR_MSG("writing to routing socket");
+      Error_msg("ec_inet_BSD:%d writing to routing socket | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 #undef rtm
 #undef NEXTADDR
@@ -1181,13 +1135,15 @@ void Inet_UnsetRoute(void)
    struct hostent *hp = 0;
    int IP;
 
-   DEBUG_MSG("Inet_UnsetRoute");
+#ifdef DEBUG
+   Debug_msg("Inet_UnsetRoute");
+#endif
 
    pid = getpid();
 
    s = socket(PF_ROUTE, SOCK_RAW, 0);
    if (s < 0)
-      ERROR_MSG("socket(PF_ROUTE)");
+      Error_msg("ec_inet_BSD:%d socket(PF_ROUTE) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
    forcehost = 0;
@@ -1219,13 +1175,15 @@ void Inet_SetRoute(void)
    struct hostent *hp = 0;
    int IP;
 
-   DEBUG_MSG("Inet_SetRoute NET");
+#ifdef DEBUG
+   Debug_msg("Inet_SetRoute NET");
+#endif
 
    pid = getpid();
 
    s = socket(PF_ROUTE, SOCK_RAW, 0);
    if (s < 0)
-      ERROR_MSG("socket(PF_ROUTE)");
+      Error_msg("ec_inet_BSD:%d socket(PF_ROUTE) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 
    forcehost = 0;
@@ -1247,12 +1205,9 @@ void Inet_SetRoute(void)
    forcenet = 0;
 
    close(s);
-   atexit(Inet_UnsetRoute);
+   exit_func(Inet_UnsetRoute);
 }
 
 #endif
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
