@@ -19,50 +19,12 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_linux.c,v 1.13 2003/02/25 11:03:32 alor Exp $
+    $Id: ec_inet_linux.c,v 1.1.1.1 2001/08/27 19:27:30 alor Exp $
 */
 
-#include "../../include/ec_main.h"
-
 #include <ctype.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
 
-#include "../../include/ec_thread.h"
-#include "../../include/ec_inet.h"
-#include "../../include/ec_inet_forge.h"
-#include "../../include/ec_inet_structures.h"
-
-struct ifreq old_ifr;      // old iface flags
-char IpForward_status;     // old ipforward status
-
-static char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
-static char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};
-
-// protos...
-
-int Inet_FindIFace(char *iface);
-int Inet_CorrectIface(char *iface);
-int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, u_long *IP, u_long *NetMask);
-int Inet_SetPromisc(char *iface);
-int Inet_OpenRawSock(char *iface);
-void Inet_CloseRawSock(int sock);
-int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type);
-int Inet_SendRawPacket(int sock, char *buffer, int len);
-void Inet_SetNonBlock(int sock);
-void Inet_Restore_ifr(void);
-void Inet_DisableForwarding(void);
-void Inet_RestoreForwarding(void);
-char *Inet_MacFromIP(unsigned long ip);
-#ifdef PERMIT_HTTPS
-   void Inet_UnSetARPEntry(void);
-   int Inet_SetARPEntry(unsigned long IP, char MAC[6]);
-   void Inet_UnsetRoute(void);
-   void Inet_SetRoute(void);
-#endif
-
-
-// ----------------------------------------------
+// This file is included from ../ec_inet.c
 
 int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 {
@@ -76,20 +38,20 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_linux:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    buf_size = 8192;
 
    for (;;) {
       buf = malloc (buf_size);
       if (buf == NULL)
-         ERROR_MSG("malloc()");
+         Error_msg("ec_Inet_linux:%d malloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       ifc.ifc_len = buf_size;
       ifc.ifc_buf = buf;
       memset (buf, 0, buf_size);
       if (ioctl(fd, SIOCGIFCONF, (char *)&ifc) < 0 && errno != EINVAL)
-         ERROR_MSG("ioctl(SIOCGIFCONF)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFCONF) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       if (ifc.ifc_len < buf_size)
          break;
@@ -126,22 +88,17 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
          if (errno == ENXIO)
             continue;
 
-         ERROR_MSG("ioctl(SIOCGIFFLAGS)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFFLAGS) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
       }
 
-      DEBUG_MSG("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#endif
 
       /* Must be up and not the loopback */
       if ((ifr.ifr_flags & IFF_UP) == 0 || (ifr.ifr_flags & IFF_LOOPBACK) != 0)
          continue;
 
-      if ( ioctl(fd, SIOCGIFHWADDR, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFHWADDR)");
-
-      /* Must be ethernet */
-      if ( ifr.ifr_hwaddr.sa_family != ARPHRD_ETHER )
-         continue;
-
       endcp = ifrp->ifr_name + strlen(ifrp->ifr_name);
       for (cp = ifrp->ifr_name; cp < endcp && !isdigit(*cp); ++cp)
          continue;
@@ -165,27 +122,31 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
       return -1;
    }
 
-   strlcpy(iface, mp->ifr_name, sizeof(Options.netiface));
+   strcpy(iface, mp->ifr_name);
 
    free(buf);
 
-   DEBUG_MSG("Inet_FindIFace\t\tIface: %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace\t\tIface: %s", iface);
+#endif
 
    return 0;
 
 }
 
 
-int Inet_CorrectIface(char *iface)
+char Inet_CorrectIface(char *iface)
 {
    int sock;
    struct ifreq ifr;
 
-   DEBUG_MSG("Inet_CorrectIface\t\tIface: %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_CorrectIface\t\tIface: %s", iface);
+#endif
 
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_linux:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
@@ -201,30 +162,12 @@ int Inet_CorrectIface(char *iface)
       return -1;
    }
 
-   if (ifr.ifr_flags & IFF_LOOPBACK )                    // check for loopback
-   {
-      Options.normal = 1;
-      Error_msg("Ettercap can't be run on loopback device");
-   }
-
    if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )             // check for alias
    {
       close(sock);
       return -1;
    }
 
-   if ( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )
-      ERROR_MSG("ioctl(SIOCGIFHWADDR)");
-
-   switch (ifr.ifr_hwaddr.sa_family)
-   {
-        case ARPHRD_ETHER:    // only ethernet is supported
-            break;
-
-        default:
-            Error_msg("Device type not supported (only ethernet)");
-   }
-
    close(sock);
 
    return 0;
@@ -246,7 +189,9 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    {
       if ( ioctl(sock, SIOCGIFMTU, &ifr) < 0)
       {
-         DEBUG_MSG("Inet_IPBased_Run -- MTU FAILED... assuming 1500");
+         #ifdef DEBUG
+            Debug_msg("Inet_IPBased_Run -- MTU FAILED... assuming 1500");
+         #endif
          *MTU = 1500;
       }
       else
@@ -257,21 +202,21 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    if (MyMAC != NULL)
    {
       if ( ioctl(sock, SIOCGIFHWADDR, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFHWADDR)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFHWADDR) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy(MyMAC, ifr.ifr_hwaddr.sa_data, 6);
    }
 
    if (IP != NULL)
    {
       if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFADDR)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFADDR) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)IP, ifr.ifr_addr.sa_data+2, 4);
    }
 
    if (NetMask != NULL)
    {
       if ( ioctl(sock, SIOCGIFNETMASK, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFNETMASK)");
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFNETMASK) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)NetMask, ifr.ifr_addr.sa_data+2, 4);
       if (strcmp(Options.netmask, ""))       // specified on command line
          *NetMask = inet_addr(Options.netmask);
@@ -284,19 +229,12 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
 }
 
 
-void Inet_SetNonBlock(int sock)
-{
-   DEBUG_MSG("Inet_SetNonBlock fd = %d", sock);
-
-   fcntl(sock, F_SETFL, O_NONBLOCK);
-
-}
-
-
 void Inet_CloseRawSock(int sock)
 {
 
-   DEBUG_MSG("Inet_CloseRawSock \t fd = %d", sock);
+#ifdef DEBUG
+   Debug_msg("Inet_CloseRawSock \t fd = %d", sock);
+#endif
 
    close(sock);
 
@@ -307,30 +245,32 @@ void Inet_CloseRawSock(int sock)
 int Inet_OpenRawSock(char *iface)
 {
    int sock;
-#if defined (HAVE_PF_PACKET) && defined (NETPACKET_PACKET_H)
+#ifdef HAVE_PF_PACKET
    struct ifreq ifr;
    struct sockaddr_ll sll;
 #else
    struct sockaddr sa;
 #endif
 
-#if defined (HAVE_PF_PACKET) && defined (NETPACKET_PACKET_H)
+#ifdef HAVE_PF_PACKET
    sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 #else
    sock = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
 #endif
    if (sock < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_linux:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
-   DEBUG_MSG("Inet_OpenRawSock %s \t fd = %d", iface, sock);
+#ifdef DEBUG
+   Debug_msg("Inet_OpenRawSock %s \t fd = %d", iface, sock);
+#endif
 
-#if defined (HAVE_PF_PACKET) && defined (NETPACKET_PACKET_H)
+#ifdef HAVE_PF_PACKET
 
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
 
    if ( ioctl(sock, SIOCGIFINDEX, &ifr) < 0)
-      ERROR_MSG("ioctl(SIOCGIFINDEX)");
+      Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFINDEX) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    memset(&sll, 0, sizeof(sll));
    sll.sll_family = AF_PACKET;
@@ -338,13 +278,13 @@ int Inet_OpenRawSock(char *iface)
    sll.sll_protocol = htons(ETH_P_ALL);
 
    if ( bind(sock, (struct sockaddr *) &sll, sizeof(sll)) == -1)
-      ERROR_MSG("bind()");
+      Error_msg("ec_Inet_linux:%d bind() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 #else
 
    memset(&sa, 0, sizeof(sa));
    strncpy(sa.sa_data, iface, sizeof(sa.sa_data));
    if ( bind(sock, &sa, sizeof(sa)) == -1)
-      ERROR_MSG("bind()");
+      Error_msg("ec_Inet_linux:%d bind() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
 #endif
 
@@ -357,8 +297,12 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
 {
    int len = 0;
    socklen_t fromlen;
+//#ifdef HAVE_PF_PACKET
+//   struct sockaddr_ll from;
+//#else
    struct sockaddr from;
    static char MyMAC[6]={0x65,0x74,0x74,0x65,0x72,0x63};
+//#endif
 
    fromlen = sizeof(from);
    len = recvfrom(sock, buffer, MTU+64, MSG_TRUNC, (struct sockaddr *)&from, &fromlen);
@@ -367,6 +311,9 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
                                                          // this kernel passes us the MTU + ETH_HEADER + FCS
                                                          // for a total of 1518 byte !!!
 
+//#ifdef HAVE_PF_PACKET
+//   if (type != NULL) *type = from.sll_pkttype;
+//#else
    // the address returned for SOCK_PACKET lacks the packet type information.
    if (type != NULL)
    {
@@ -377,6 +324,7 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
        else
            *type = 1; // !PACKET_HOST
    }
+//#endif
 
    // TODO
    // handle fragmented packets...
@@ -390,7 +338,7 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 {
    int sent;
    static char first_time = 1;
-#if defined (HAVE_PF_PACKET) && defined (NETPACKET_PACKET_H)
+#ifdef HAVE_PF_PACKET
    static struct sockaddr_ll dest;
    static struct ifreq ifr;
 #else
@@ -402,13 +350,13 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
       memset(&dest, 0, sizeof (dest));
       first_time = 0;
 
-      #if defined (HAVE_PF_PACKET) && defined (NETPACKET_PACKET_H)
+      #ifdef HAVE_PF_PACKET
 
          memset(&ifr, 0, sizeof(ifr));
          strncpy(ifr.ifr_name, Options.netiface, sizeof(ifr.ifr_name));
 
          if ( ioctl(sock, SIOCGIFINDEX, &ifr) < 0)
-            ERROR_MSG("ioctl(SIOCGIFINDEX)");
+            Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFINDEX) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
          dest.sll_family = AF_PACKET;
          dest.sll_ifindex = ifr.ifr_ifindex;
@@ -420,16 +368,7 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 
    sent = sendto(sock, buffer, len, 0, (struct sockaddr *)&dest, sizeof(dest));
    if (sent < len)
-   {
-      while (errno == ENOBUFS || errno == EAGAIN)
-      {
-         usleep(5000);
-         sent = sendto(sock, buffer, len, 0, (struct sockaddr *)&dest, sizeof(dest));
-         if (sent == len) return (sent);
-      }
-
-      Error_msg("ec_inet_linux:%d sendto() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
-   }
+      Error_msg("ec_Inet_linux:%d sendto() %d(%d) | ERRNO : %d | %s \n", __LINE__, len, sent, errno, strerror(errno));
 
    return (sent);
 
@@ -443,7 +382,9 @@ int Inet_SetPromisc(char *iface)
    int sock;
    struct ifreq ifr;
 
-   DEBUG_MSG("Inet_SetPromisc\tiface: %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_SetPromisc\tiface: %s", iface);
+#endif
 
    sock = Inet_OpenRawSock(iface);
 
@@ -451,7 +392,7 @@ int Inet_SetPromisc(char *iface)
    strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
 
    if ( ioctl(sock, SIOCGIFFLAGS, &ifr) < 0 )
-      ERROR_MSG("ioctl(SIOCGIFFLAGS)");
+      Error_msg("ec_Inet_linux:%d ioctl(SIOCGIFFLAGS) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    memset(&old_ifr, 0, sizeof(old_ifr));
    old_ifr.ifr_flags = ifr.ifr_flags;              //save old flags
@@ -460,8 +401,8 @@ int Inet_SetPromisc(char *iface)
    {
       ifr.ifr_flags |= IFF_PROMISC;
       if ( ioctl(sock, SIOCSIFFLAGS, &ifr) < 0 )      // promisc mode
-         ERROR_MSG("ioctl(SIOCSIFFLAGS) | promisc on");
-      atexit(Inet_Restore_ifr);
+         Error_msg("ec_Inet_linux:%d ioctl(SIOCSIFFLAGS) | promisc on | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
+      exit_func(Inet_Restore_ifr);
    }
 
 
@@ -477,7 +418,9 @@ void Inet_Restore_ifr(void)
    int sock;
    struct ifreq ifr;
 
-   DEBUG_MSG("Inet_Restore_ifr");
+#ifdef DEBUG
+   Debug_msg("Inet_Restore_ifr");
+#endif
 
    sock = Inet_OpenRawSock(Options.netiface);
 
@@ -487,7 +430,7 @@ void Inet_Restore_ifr(void)
    ifr.ifr_flags = old_ifr.ifr_flags;
 
    if ( ioctl(sock, SIOCSIFFLAGS, &ifr) < 0 )     // flag restoring
-      ERROR_MSG("ioctl(SIOCSIFFLAGS) | flag restoring");
+      Error_msg("ec_Inet_linux:%d ioctl(SIOCSIFFLAGS) | flag restoring | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    Inet_CloseRawSock(sock);
 }
@@ -498,19 +441,31 @@ void Inet_DisableForwarding(void)
    FILE *fd;
 
    fd = fopen("/proc/sys/net/ipv4/ip_forward", "r");
-   if (fd == NULL)
-      Error_msg("failed to open /proc/sys/net/ipv4/ip_forward");
+   if (fd < 0 )
+   {
+      #ifdef DEBUG
+         Debug_msg("ec_Inet_linux:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
+      #endif
+      return;
+   }
 
-   fscanf(fd, "%c", &IpForward_status);
+   fscanf(fd, "%s", IpForward_status);
    fclose(fd);
 
-   DEBUG_MSG("Inet_DisableForwarding from %c", IpForward_status);
+#ifdef DEBUG
+   Debug_msg("Inet_DisableForwarding from %c", IpForward_status[0]);
+#endif
 
-   atexit(Inet_RestoreForwarding);
+   exit_func(Inet_RestoreForwarding);
 
    fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
-   if (fd == NULL)
-      Error_msg("failed to open /proc/sys/net/ipv4/ip_forward");
+   if (fd < 0 )
+   {
+      #ifdef DEBUG
+         Debug_msg("ec_Inet_linux:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
+      #endif
+      return;
+   }
 
    fprintf(fd, "0");
    fclose(fd);
@@ -522,15 +477,22 @@ void Inet_RestoreForwarding(void)
 {
    FILE *fd;
 
-   if (strcmp(ECThread_getname(pthread_self()), PROGRAM)) return;
+   if (!strstr(program_argv0, PROGRAM)) return;
 
    fd = fopen("/proc/sys/net/ipv4/ip_forward", "w");
-   if (fd == NULL)
-      Error_msg("failed to open /proc/sys/net/ipv4/ip_forward");
+   if (fd < 0 )
+   {
+      #ifdef DEBUG
+         Debug_msg("ec_Inet_linux:%d fopen(/proc/sys/net/ipv4/ip_forward) | ERRNO : %d | %s \n", __LINE__, errno, sys_errlist[errno]);
+      #endif
+      return;
+   }
 
-   DEBUG_MSG("Inet_RestoreForwarding to %c", IpForward_status);
+#ifdef DEBUG
+   Debug_msg("Inet_RestoreForwarding to %c", IpForward_status[0]);
+#endif
 
-   fprintf(fd, "%c", IpForward_status );
+   fprintf(fd, "%c", IpForward_status[0] );
    fclose(fd);
 }
 
@@ -542,7 +504,9 @@ char *Inet_MacFromIP(unsigned long ip)
    static struct arpreq ar;
    struct sockaddr_in *sa_in;
 
-   DEBUG_MSG("Inet_MacFromIP");
+#ifdef DEBUG
+   Debug_msg("Inet_MacFromIP");
+#endif
 
    memset((char *)&ar, 0, sizeof(ar));
    strncpy(ar.arp_dev, Options.netiface, sizeof(ar.arp_dev));
@@ -554,39 +518,40 @@ char *Inet_MacFromIP(unsigned long ip)
 
    if (ioctl(sock_raw, SIOCGARP, (caddr_t)&ar) == -1)  // not in cache... try to find it...
    {
-      struct recv_packet recvpck;
+      u_char *buf;
       char MyMAC[6];
       u_long MyIP;
       int MTU, sock;
       TIME_DECLARE;
 
-      DEBUG_MSG("Inet_MacFromIP -- try to find it");
-
+#ifdef DEBUG
+   Debug_msg("Inet_MacFromIP -- try to find it");
+#endif
       sock = Inet_OpenRawSock(Options.netiface);
 
       Inet_GetIfaceInfo(Options.netiface, &MTU, MyMAC, &MyIP, NULL);
 
-      if (ip == MyIP) {
-         DEBUG_MSG("Inet_MacFromIP -- try to find me... ;)");
-
+      if (ip == MyIP)
+      {
+         #ifdef DEBUG
+            Debug_msg("Inet_MacFromIP -- try to find me... ;)");
+         #endif
          memcpy(&ar.arp_ha.sa_data, MyMAC, ETHER_ADDR_LEN);
          Inet_CloseRawSock(sock);
          Inet_CloseRawSock(sock_raw);
          return (char *) ar.arp_ha.sa_data;
       }
 
-      recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD );
-      recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
+      buf = Inet_Forge_packet( ETH_HEADER + ARP_HEADER );
+      Inet_Forge_ethernet( buf, MyMAC, ETH_BROADCAST, ETH_P_ARP );
 
-      Inet_Forge_ethernet( recvpck.aligned, MyMAC, ETH_BROADCAST, ETH_P_ARP );
-
-      Inet_Forge_arp(recvpck.aligned + ETH_HEADER, ARPOP_REQUEST,
+      Inet_Forge_arp( buf+ETH_HEADER, ARPOP_REQUEST,
                          MyMAC, MyIP,
                          ARP_BROADCAST, ip );
 
-      Inet_SendRawPacket(sock, recvpck.aligned, ETH_HEADER + ARP_HEADER);
-
-		memset(recvpck.aligned, 0, MTU);
+      Inet_SendRawPacket(sock, buf, ETH_HEADER + ARP_HEADER);
+      Inet_Forge_packet_destroy( buf );
+      buf = Inet_Forge_packet( MTU );
 
       fcntl(sock, F_SETFL, O_NONBLOCK);
       TIME_START;
@@ -598,10 +563,10 @@ char *Inet_MacFromIP(unsigned long ip)
          ETH_header *ethpkt;
          ARP_header *arppkt;
 
-         len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, &pkttype);
+         len = Inet_GetRawPacket(sock, buf, MTU, &pkttype);
 
-         ethpkt = (ETH_header *)recvpck.aligned;
-         arppkt = (ARP_header *)(ethpkt + 1);
+         ethpkt = (ETH_header *)buf;
+         arppkt = (ARP_header *)(buf + ETH_HEADER);
 
          TIME_FINISH;
 
@@ -610,7 +575,7 @@ char *Inet_MacFromIP(unsigned long ip)
             if ( *(unsigned long *)arppkt->source_ip == ip )
             {
                memcpy(&ar.arp_ha.sa_data, &arppkt->source_add, ETHER_ADDR_LEN);
-               Inet_Forge_packet_destroy( recvpck.buf );
+               Inet_Forge_packet_destroy( buf );
                Inet_CloseRawSock(sock);
                Inet_CloseRawSock(sock_raw);
                return (char *) ar.arp_ha.sa_data;
@@ -618,7 +583,6 @@ char *Inet_MacFromIP(unsigned long ip)
          }
       } while ( TIME_ELAPSED < 0.5 );
 
-		Inet_Forge_packet_destroy( recvpck.buf );
       Inet_CloseRawSock(sock);
       Inet_CloseRawSock(sock_raw);
       return ETH_BROADCAST;  // workaround for non local ip
@@ -638,58 +602,62 @@ unsigned char SavedMAC[6];
 
 void Inet_UnSetARPEntry(void)
 {
-   struct arpreq req;
-   int sockfd;
-   struct sockaddr_in *sa_in;
+    struct arpreq req;
+    int sockfd;
+    struct sockaddr_in *sa_in;
 
-   DEBUG_MSG("Inet_UnSetARPEntry");
+#ifdef DEBUG
+   Debug_msg("Inet_UnSetARPEntry");
+#endif
 
-   sockfd = socket(AF_INET, SOCK_DGRAM, 0);
+    sockfd = socket(AF_INET, SOCK_DGRAM, 0);
 
-   memset((char *) &req, 0, sizeof(req));
-   sa_in = (struct sockaddr_in *)&req.arp_pa;
-   sa_in->sin_family = AF_INET;
-   sa_in->sin_addr.s_addr = SavedIP;
-   req.arp_flags = ATF_PERM | ATF_COM;
+    memset((char *) &req, 0, sizeof(req));
+    sa_in = (struct sockaddr_in *)&req.arp_pa;
+    sa_in->sin_family = AF_INET;
+    sa_in->sin_addr.s_addr = SavedIP;
+    req.arp_flags = ATF_PERM | ATF_COM;
 
-   memcpy(req.arp_ha.sa_data, SavedMAC, 6);
-   strlcpy(req.arp_dev, Options.netiface, sizeof(req.arp_dev));
+    memcpy(req.arp_ha.sa_data, SavedMAC, 6);
+    strcpy(req.arp_dev, Options.netiface);
 
-   ioctl(sockfd, SIOCDARP, &req);
+    ioctl(sockfd, SIOCDARP, &req);
 
-   close(sockfd);
+    close(sockfd);
 }
 
 
 int Inet_SetARPEntry(unsigned long IP, char MAC[6])
 {
-   struct arpreq req;
-   int sockfd;
-   struct sockaddr_in *sa_in;
-   int retval;
+    struct arpreq req;
+    int sockfd;
+    struct sockaddr_in *sa_in;
+    int retval;
 
-   DEBUG_MSG("Inet_SetARPEntry");
+#ifdef DEBUG
+   Debug_msg("Inet_SetARPEntry");
+#endif
 
-   sockfd = socket(AF_INET, SOCK_DGRAM, 0);
+    sockfd = socket(AF_INET, SOCK_DGRAM, 0);
 
-   memset((char *) &req, 0, sizeof(req));
-   sa_in = (struct sockaddr_in *)&req.arp_pa;
-   sa_in->sin_family = AF_INET;
-   sa_in->sin_addr.s_addr = IP;
-   req.arp_flags = ATF_PERM | ATF_COM;
+    memset((char *) &req, 0, sizeof(req));
+    sa_in = (struct sockaddr_in *)&req.arp_pa;
+    sa_in->sin_family = AF_INET;
+    sa_in->sin_addr.s_addr = IP;
+    req.arp_flags = ATF_PERM | ATF_COM;
 
-   memcpy(req.arp_ha.sa_data, MAC, 6);
-   strlcpy(req.arp_dev, Options.netiface, sizeof(req.arp_dev));
+    memcpy(req.arp_ha.sa_data, MAC, 6);
+    strcpy(req.arp_dev, Options.netiface);
 
-   retval = ioctl(sockfd, SIOCSARP, &req);
+    retval = ioctl(sockfd, SIOCSARP, &req);
 
-   close(sockfd);
+    close(sockfd);
 
-   SavedIP=IP;
-   memcpy(SavedMAC, MAC, 6);
-   atexit(Inet_UnSetARPEntry);
+    SavedIP=IP;
+    memcpy(SavedMAC, MAC, 6);
+    exit_func(Inet_UnSetARPEntry);
 
-   return retval;
+    return retval;
 }
 
 // ROUTE MANAGEMENT ---------------------------
@@ -718,7 +686,9 @@ void Inet_UnsetRoute(void)
     int skfd;
     struct rtentry rt;
 
-    DEBUG_MSG("Inet_UnsetRoute");
+#ifdef DEBUG
+   Debug_msg("Inet_UnsetRoute");
+#endif
 
     skfd = socket(AF_INET, SOCK_DGRAM, 0);
 
@@ -749,7 +719,9 @@ void Inet_SetRoute(void)
     int skfd;
     struct rtentry rt;
 
-    DEBUG_MSG("Inet_SetRoute");
+#ifdef DEBUG
+   Debug_msg("Inet_SetRoute");
+#endif
 
     skfd = socket(AF_INET, SOCK_DGRAM, 0);
 
@@ -774,12 +746,9 @@ void Inet_SetRoute(void)
 
     close(skfd);
 
-    atexit(Inet_UnsetRoute);
+    exit_func(Inet_UnsetRoute);
 }
 
 #endif
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
