@@ -17,20 +17,12 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_solaris.c,v 1.28 2002/05/13 20:57:46 alor Exp $
+    $Id: ec_inet_solaris.c,v 1.3 2001/08/28 08:57:13 alor Exp $
 */
 
-#include "../../include/ec_main.h"
 
-#include "../../include/ec_thread.h"
-#include "../../include/ec_buffer.h"
-#include "../../include/ec_inet.h"
-#include "../../include/ec_inet_forge.h"
-#include "../../include/ec_inet_structures.h"
+// This file is included from ../ec_inet.c
 
-#include <ctype.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
 
 #include <sys/ioctl.h>
 #include <sys/sockio.h>
@@ -50,10 +42,6 @@
 #include <sys/systeminfo.h>
 #endif
 
-#include <sys/stropts.h>
-#include <inet/nd.h>
-#include <inet/common.h>
-
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -75,38 +63,6 @@ struct ether_addr
 
 #define  MAXDLBUF 8192
 
-char IpForward_status[2];      // old ipforward status
-
-static u_int ctlbuf[MAXDLBUF];
-static struct strbuf ctl = {
-   MAXDLBUF,
-   0,
-   (char *)ctlbuf
-};
-char ebuf[100];
-
-int bufsize, offset;
-int SocketBuffer = -1;
-int dlpi_in_use;
-
-static char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
-static char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};
-
-// protos...
-
-int Inet_FindIFace(char *iface);
-int Inet_CorrectIface(char *iface);
-int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, u_long *IP, u_long *NetMask);
-int Inet_SetPromisc(char *iface);
-int Inet_OpenRawSock(char *iface);
-void Inet_CloseRawSock(int sock);
-int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type);
-int Inet_SendRawPacket(int sock, char *buffer, int len);
-void Inet_SetNonBlock(int sock);
-void Inet_Restore_ifr(void);
-void Inet_DisableForwarding(void);
-void Inet_RestoreForwarding(void);
-char *Inet_MacFromIP(unsigned long ip);
 /* protos... */
 static int dlattachreq(int, u_int, char *);
 static int dlbindack(int, char *, char *);
@@ -114,7 +70,7 @@ static int dlbindreq(int, u_int, char *);
 static int dlinfoack(int, char *, char *);
 static int dlinforeq(int, char *);
 static int dlokack(int, const char *, char *, char *);
-static int send_request(int, char *, int, char *, char *);
+static int send_request(int, char *, int, char *, char *, int);
 static int recv_ack(int, int, const char *, char *, char *);
 static int dlpromisconreq(int, u_int, char *);
 #if defined(SOLARIS) && defined(HAVE_SYS_BUFMOD_H)
@@ -126,7 +82,18 @@ static int strioctl(int, int, int, char *);
 #ifdef HAVE_DEV_DLPI
 static int get_dlpi_ppa(int, const char *, int, char *);
 #endif
-static char * split_dname(char *device, int *unitp);
+
+int fdprom;
+static u_int ctlbuf[MAXDLBUF];
+static struct strbuf ctl = {
+   MAXDLBUF,
+   0,
+   (char *)ctlbuf
+};
+char ebuf[100];
+
+int bufsize, offset;
+int SocketBuffer = -1;
 
 // ==============================================
 
@@ -142,20 +109,20 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
 
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_solaris:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    buf_size = 8192;
 
    for (;;) {
       buf = malloc (buf_size);
       if (buf == NULL)
-         ERROR_MSG("malloc()");
+         Error_msg("ec_Inet_solaris:%d malloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       ifc.ifc_len = buf_size;
       ifc.ifc_buf = buf;
       memset (buf, 0, buf_size);
       if (ioctl(fd, SIOCGIFCONF, (char *)&ifc) < 0 && errno != EINVAL)
-         ERROR_MSG("ioctl(SIOCGIFCONF)");
+         Error_msg("ec_Inet_solaris:%d ioctl(SIOCGIFCONF) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
       if (ifc.ifc_len < buf_size)
          break;
@@ -191,10 +158,12 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
       {
          if (errno == ENXIO)
             continue;
-         ERROR_MSG("ioctl(SIOCGIFFLAGS)");
+         Error_msg("ec_Inet_solaris:%d ioctl(SIOCGIFFLAGS) | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
       }
 
-      DEBUG_MSG("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- check for [%s]", ifr.ifr_name);
+#endif
 
       /* Must be up and not the loopback */
       if ((ifr.ifr_flags & IFF_UP) == 0 || (ifr.ifr_flags & IFF_LOOPBACK) != 0)
@@ -223,27 +192,31 @@ int Inet_FindIFace(char *iface)     // adapded from pcap_lookupdev
       return -1;
    }
 
-   strlcpy(iface, mp->ifr_name, sizeof(Options.netiface));
+   strcpy(iface, mp->ifr_name);
 
    free(buf);
 
-   DEBUG_MSG("Inet_FindIFace -- %s found !!", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_FindIFace -- %s found !!", iface);
+#endif
 
    return 0;
 
 }
 
 
-int Inet_CorrectIface(char *iface)
+char Inet_CorrectIface(char *iface)
 {
    int sock;
    struct ifreq ifr;
 
-   DEBUG_MSG("Inet_CorrectIface\t\tIface: %s", iface);
+#ifdef DEBUG
+   Debug_msg("Inet_CorrectIface\t\tIface: %s", iface);
+#endif
 
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
-      ERROR_MSG("socket()");
+      Error_msg("ec_Inet_solaris:%d socket() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
 
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
@@ -259,12 +232,6 @@ int Inet_CorrectIface(char *iface)
       return -1;
    }
 
-   if (ifr.ifr_flags & IFF_LOOPBACK )                    // check for loopback
-   {
-      Options.normal = 1;
-      Error_msg("Ettercap can't be run on loopback device");
-   }
-
    if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )             // check for alias
    {
       close(sock);
@@ -295,13 +262,15 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    {
       if ( ioctl(sock, SIOCGIFMTU, &ifr) < 0)
       {
-         DEBUG_MSG("Inet_GetIfaceInfo -- MTU FAILED... assuming 1500");
+         #ifdef DEBUG
+            Debug_msg("Inet_GetIfaceInfo -- MTU FAILED... assuming 1500");
+         #endif
          *MTU = 1500;
       }
       else
 // FIXME
          //*MTU = ifr.ifr_mtu;
-         *MTU = 1500;
+         *MTU = ifr.ifr_metric;  // ?? possible ?? i've found this example on the web...
    }
 
    if (MyMAC != NULL)
@@ -314,11 +283,11 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
       dlp->physaddr_req.dl_primitive = DL_PHYS_ADDR_REQ;
       dlp->physaddr_req.dl_addr_type = DL_CURR_PHYS_ADDR;
 
-      if (send_request(dlpi, (char *)dlp, DL_PHYS_ADDR_REQ_SIZE, "physaddr", ebuf) < 0)
-         Error_msg("send_request(DL_PHYS_ADDR_REQ_SIZE) | %s \n", ebuf);
+      if (send_request(dlpi, (char *)dlp, DL_PHYS_ADDR_REQ_SIZE, "physaddr", ebuf, 0) < 0)
+         Error_msg("ec_Inet_solaris:%d send_request(DL_PHYS_ADDR_REQ_SIZE) | %s \n", __LINE__, ebuf);
 
       if (recv_ack(dlpi, DL_PHYS_ADDR_ACK_SIZE, "physaddr", (char *)dlp, ebuf) < 0)
-         Error_msg("recv_ack(DL_PHYS_ADDR_ACK_SIZE) | %s \n", ebuf);
+         Error_msg("ec_Inet_solaris:%d recv_ack(DL_PHYS_ADDR_ACK_SIZE) | %s \n", __LINE__, ebuf);
 
       memcpy( MyMAC,(struct ether_addr *) ((char *) dlp + dlp->physaddr_ack.dl_addr_offset), ETHER_ADDR_LEN);
 
@@ -328,14 +297,14 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
    if (IP != NULL)
    {
       if ( ioctl(sock, SIOCGIFADDR, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFADDR)");
+         Error_msg("ec_Inet_solaris:%d ioctl(SIOCGIFADDR) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)IP, ifr.ifr_addr.sa_data+2, 4);
    }
 
    if (NetMask != NULL)
    {
       if ( ioctl(sock, SIOCGIFNETMASK, &ifr) < 0 )
-         ERROR_MSG("ioctl(SIOCGIFNETMASK)");
+         Error_msg("ec_Inet_solaris:%d ioctl(SIOCGIFNETMASK) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
       memcpy((char *)NetMask, ifr.ifr_addr.sa_data+2, 4);
       if (strcmp(Options.netmask, ""))       // specified on command line
          *NetMask = inet_addr(Options.netmask);
@@ -348,19 +317,14 @@ int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *IP, uns
 }
 
 
-void Inet_SetNonBlock(int sock)
-{
-   DEBUG_MSG("Inet_SetNonBlock \t WRAPPERED TO NULL");
-
-}
-
-
 void Inet_CloseRawSock(int sock)
 {
 
-   DEBUG_MSG("Inet_CloseRawSock \t wrappered to NULL");
+#ifdef DEBUG
+   Debug_msg("Inet_CloseRawSock \t fd = %d", sock);
+#endif
 
-   //close(sock);    // the bpf is close when ettercap exits
+   close(sock);
 
 }
 
@@ -370,12 +334,12 @@ void Inet_CloseRawSock(int sock)
 int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
 {
    register char *cp;
+   char *eos;
    int fd;
-   int ppa;
+   register int ppa;
    register dl_info_ack_t *infop;
-   struct timeval to;
 #ifdef HAVE_SYS_BUFMOD_H
-   u_int32 flag, ss;
+   u_long flag, ss;
    char *release;
    u_int osmajor, osminor, osmicro;
 #endif
@@ -385,11 +349,6 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    char dname2[100];
 #endif
 
-   if (dlpi_in_use != 0) {
-      DEBUG_MSG("Inet_OpenRawSock %s", iface);
-      DEBUG_MSG("Inet_OpenRawSock \t dlpi_in_use = %d ", dlpi_in_use);
-      return dlpi_in_use;
-   }
 
 #ifdef HAVE_DEV_DLPI
    /*
@@ -402,17 +361,18 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
       cp++;
    strlcpy(dname, cp, sizeof(dname));
 
-   DEBUG_MSG("Inet_OpenRawSock -- HAVE_DEV_DLPI [%s]", dname);
-
    /*
     * Split the name into a device type and a unit number.
     */
-
-   cp = split_dname(dname, &ppa);
+   cp = strpbrk(dname, "0123456789");
    if (cp == NULL)
-      Error_msg("ec_inet_solaris:%d HAVE_DEV_DLPI: %s missing or bad unit number\n", __LINE__, iface);
+      Error_msg("ec_Inet_solaris:%d dlpi: %s missing unit number\n", __LINE__, iface);
 
-	*cp = '\0';
+   ppa = strtol(cp, &eos, 10);
+   if (*eos != '\0')
+      Error_msg("ec_Inet_solaris:%d dlpi: %s bad unit number\n", __LINE__, iface);
+
+   *cp = '\0';
 
    /*
     * Use "/dev/dlpi" as the device.
@@ -420,7 +380,7 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
 
    cp = "/dev/dlpi";
    if ((fd = open(cp, O_RDWR)) < 0)
-      ERROR_MSG("open()");
+      Error_msg("ec_Inet_solaris:%d open() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    /*
     * Get a table of all PPAs for that device, and search that
@@ -428,37 +388,38 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
     */
    ppa = get_dlpi_ppa(fd, dname, ppa, ebuf);
    if (ppa < 0)
-      ERROR_MSG("get_dlpi_ppa()");
+      Error_msg("ec_Inet_solaris:%d get_dlpi_ppa() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
 #else
    /*
    ** Determine device and ppa
    */
-   cp = split_dname(iface, &ppa);
+      cp = strpbrk(dname, "0123456789");
    if (cp == NULL)
-      Error_msg("ec_inet_solaris:%d dlpi: %s missing or bad unit number\n", __LINE__, iface);
+      Error_msg("ec_Inet_solaris:%d dlpi: %s missing unit number\n", __LINE__, iface);
+
+   ppa = strtol(cp, &eos, 10);
+   if (*eos != '\0')
+      Error_msg("ec_Inet_solaris:%d dlpi: %s bad unit number\n", __LINE__, iface);
+
 
    if (*iface == '/')
       strlcpy(dname, iface, sizeof(dname));
    else
       snprintf(dname, sizeof(dname), "%s/%s", DLPI_DEV_PREFIX, iface);
 
-   strlcpy(dname2, dname, sizeof(dname2));
-   *(dname + strlen(dname) - strlen(cp)) = '\0';
-
-   DEBUG_MSG("Inet_OpenRawSock -- opening [%s]", dname);
-
    /* Try device without unit number */
+   strlcpy(dname2, dname, sizeof(dname2));
+   cp = strchr(dname, *cp);
+   *cp = '\0';
    if ((fd = open(dname, O_RDWR)) < 0)
    {
       if (errno != ENOENT)
-         ERROR_MSG("open()");
-
-      DEBUG_MSG("Inet_OpenRawSock -- open(%s) failed... trying [%s]", dname, dname2);
+         Error_msg("ec_Inet_solaris:%d open() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
       /* Try again with unit number */
       if ((fd = open(dname2, O_RDWR)) < 0)
-         ERROR_MSG("open()");
+         Error_msg("ec_Inet_solaris:%d open() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
       /* XXX Assume unit zero */
       ppa = 0;
@@ -469,40 +430,29 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    ** Attach if "style 2" provider
    */
    if (dlinforeq(fd, ebuf) < 0 || dlinfoack(fd, (char *)buf, ebuf) < 0)
-      ERROR_MSG("dlinforeq() || dlinfoack()");
-
+      Error_msg("ec_Inet_solaris:%d dlinforeq() || dlinfoack() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    infop = &((union DL_primitives *)buf)->info_ack;
    if (infop->dl_provider_style == DL_STYLE2 && (dlattachreq(fd, ppa, ebuf) < 0 ||
        dlokack(fd, "attach", (char *)buf, ebuf) < 0))
-      ERROR_MSG("dlattachreq()");
+      Error_msg("ec_Inet_solaris:%d dlattachreq() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    /*
    ** Bind
    */
    if (dlbindreq(fd, 0, ebuf) < 0 || dlbindack(fd, (char *)buf, ebuf) < 0)
-      ERROR_MSG("dlbindreq()");
+      Error_msg("ec_Inet_solaris:%d dlbindreq() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
 
-   if (dlpromisconreq(fd, DL_PROMISC_PHYS, ebuf) < 0 || dlokack(fd, "promisc_phys", (char *)buf, ebuf) < 0)
-      ERROR_MSG("dlpromisconreq(DL_PROMISC_PHYS)");
-
-   if (dlpromisconreq(fd, DL_PROMISC_MULTI, ebuf) < 0 || dlokack(fd, "promisc_multi", (char *)buf, ebuf) < 0)
-      DEBUG_MSG("WARNING: DL_PROMISC_MULTI failed (%s)", ebuf);
-
-
-	if ( (dlpromisconreq(fd, DL_PROMISC_SAP, ebuf) < 0 || dlokack(fd, "promisc_sap", (char *)buf, ebuf) < 0))
-      ERROR_MSG("dlpromisconreq(DL_PROMISC_SAP)");
-
    /*
    ** Determine link type
    */
    if (dlinforeq(fd, ebuf) < 0 || dlinfoack(fd, (char *)buf, ebuf) < 0)
-      ERROR_MSG("dlinforeq()");
-
+      Error_msg("ec_Inet_solaris:%d dlinforeq() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    infop = &((union DL_primitives *)buf)->info_ack;
-   switch (infop->dl_mac_type) {
+   switch (infop->dl_mac_type)
+   {
       case DL_CSMACD:
       case DL_ETHER:
          offset = 2;
@@ -512,12 +462,12 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
          Error_msg("Interface not supported ( only DLT_EN10MB) | %d", infop->dl_mac_type);
    }
 
-#ifdef DLIOCRAW
+#ifdef   DLIOCRAW
    /*
    ** This is a non standard SunOS hack to get the ethernet header.
    */
    if (strioctl(fd, DLIOCRAW, 0, NULL) < 0)
-      ERROR_MSG("strioctl(DLIOCRAW)");
+      Error_msg("ec_Inet_solaris:%d strioctl(DLIOCRAW) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
 #endif
 
@@ -526,7 +476,7 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    ** Another non standard call to get the data nicely buffered
    */
    if (ioctl(fd, I_PUSH, "bufmod") != 0)
-      ERROR_MSG("ioclt(I_PUSH)");
+      Error_msg("ec_Inet_solaris:%d ioclt(I_PUSH) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    /*
    ** Now that the bufmod is pushed lets configure it.
@@ -542,33 +492,24 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    ss = MAXDLBUF;
 
    release = get_release(&osmajor, &osminor, &osmicro);
-   if (osmajor == 5 && (osminor <= 2 || (osminor == 3 && osmicro < 2)) && getenv("BUFMOD_FIXED") == NULL) {
+   if (osmajor == 5 && (osminor <= 2 || (osminor == 3 && osmicro < 2)) && getenv("BUFMOD_FIXED") == NULL)
+   {
       fprintf(stderr, "WARNING: bufmod is broken in SunOS %s; ignoring snaplen.\n", release);
       ss = 0;
    }
 
-
    if (ss > 0 && strioctl(fd, SBIOCSSNAP, sizeof(ss), (char *)&ss) != 0)
-      ERROR_MSG("strioctl(SBIOCSSNAP)");
+      Error_msg("ec_Inet_solaris:%d strioctl(SBIOCSSNAP) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    /*
    ** Set up the bufmod flags
    */
    if (strioctl(fd, SBIOCGFLAGS, sizeof(flag), (char *)&flag) < 0)
-      ERROR_MSG("strioctl(SBIOCGFLAGS)");
+      Error_msg("ec_Inet_solaris:%d strioctl(SBIOCGFLAGS) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    flag |= SB_NO_DROPS;
    if (strioctl(fd, SBIOCSFLAGS, sizeof(flag), (char *)&flag) != 0)
-      ERROR_MSG("strioctl(SBIOCSFLAGS)");
-
-   /*
-    * Set the bufmod timeout
-    */
-
-   to.tv_sec = 1 / 1000;
-   to.tv_usec = (1 * 1000) % 1000000;
-   if (strioctl(fd, SBIOCSTIME, sizeof(to), (char *)&to) != 0)
-      ERROR_MSG("strioctl(SBIOCSTIME)");
+      Error_msg("ec_Inet_solaris:%d strioctl(SBIOCSFLAGS) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
 #endif
 
@@ -576,13 +517,11 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
    ** As the last operation flush the read side.
    */
    if (ioctl(fd, I_FLUSH, FLUSHR) != 0)
-      ERROR_MSG("ioctl(I_FLUSH)");
+      Error_msg("ec_Inet_solaris:%d ioctl(I_FLUSH) | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
-   bufsize = MAXDLBUF * sizeof(u_int32);             // global var...
+   bufsize = MAXDLBUF * sizeof(u_int);             // global var...
 
-   DEBUG_MSG("Inet_OpenRawSock \t fd = %d", fd);
-
-   dlpi_in_use = fd;   // for later use in the Set_Promisc
+   fdprom = fd;   // for later use in the Set_Promisc
 
    return (fd);
 
@@ -592,29 +531,31 @@ int Inet_OpenRawSock(char *iface)      // adapted from libpcap source code
 
 int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
 {
+//    register int cc, n, caplen, origlen;
    register u_char *bp, *ep, *pk;
+//    register struct bpf_insn *fcode;
    #ifdef HAVE_SYS_BUFMOD_H
       register struct sb_hdr *sbp;
       #ifdef LBL_ALIGN
          struct sb_hdr sbhdr;
       #endif
    #endif
-   int flags;
+//    int flags;
    struct strbuf data;
+//    struct pcap_pkthdr pkthdr;
    int len = 0, pktlen = 0, caplen;
    char *buf;
    static char MyMAC[6]={0x65,0x74,0x74,0x65,0x72,0x63};
 
 
-   if (SocketBuffer == -1) {                  // only the first time
+   if (SocketBuffer == -1)                   // only the first time
       SocketBuffer = Buffer_Create(1.0e5);   // 100 K buffer
-      DEBUG_MSG("Inet_GetRawPacket creates the buffer for the first time -- buf id = %d", SocketBuffer);
-   }
 
    Buffer_Get(SocketBuffer, &pktlen, sizeof(u_int));
    len = Buffer_Get(SocketBuffer, buffer, pktlen );
 
-   if (type != NULL) {
+   if (type != NULL)
+   {
        if (!strncmp(MyMAC,"etterc",6))    // only the first time...
            Inet_GetIfaceInfo(Options.netiface, NULL, MyMAC, NULL, NULL);
        if (!memcmp(MyMAC,buffer,6))
@@ -627,50 +568,94 @@ int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type)
 
    buf = (u_char *)calloc(bufsize + offset, sizeof(char));     // buffer is global
 
-   flags = 0;
+// flags = 0;
+// cc = p->cc;
+// if (cc == 0) {
    data.buf = (char *)buf + offset;
    data.maxlen = MAXDLBUF;
    data.len = 0;
-   if ( getmsg(sock, &ctl, &data, &flags) < 0 ) {
-      if (errno != EINTR) ERROR_MSG("getmsg()");
-   }
+//    do {
+//       if (getmsg(sock, &ctl, &data, &flags) < 0)
+   getmsg(sock, &ctl, &data, 0);
+//       {
+//          /* Don't choke when we get ptraced */
+//          if (errno == EINTR) {
+//             cc = 0;
+//             continue;
+//          }
+//          strlcpy(p->errbuf, pcap_strerror(errno), sizeof(p->errbuf));
+//          return (-1);
+//       }
+//       cc = data.len;
+//    } while (cc == 0);
+//    bp = p->buffer + p->offset;
+// } else
+//    bp = p->bp;
 
    bp = buf + offset;
 
  /* Loop through packets */
+// fcode = p->fcode.bf_insns;
+// ep = bp + cc;
    ep = bp + data.len;
+//    n = 0;
 
 #ifdef HAVE_SYS_BUFMOD_H
-   while (bp < ep) {
+   while (bp < ep)
+   {
    #ifdef LBL_ALIGN
-      if ((long)bp & 3) {
+      if ((long)bp & 3)
+      {
          sbp = &sbhdr;
          memcpy(sbp, bp, sizeof(*sbp));
       }
       else
    #endif
-      sbp = (struct sb_hdr *)bp;
+         sbp = (struct sb_hdr *)bp;
 
+//    p->md.stat.ps_drop += sbp->sbh_drops;
       pk = bp + sizeof(*sbp);
       bp += sbp->sbh_totlen;
+//       origlen = sbp->sbh_origlen;
       caplen = sbp->sbh_msglen;
 #else
+//       origlen = data.len;
       caplen = min(MAXDLBUF, data.len);
       pk = bp;
       bp += caplen;
 #endif
+//    ++p->md.stat.ps_recv;
+//       if (bpf_filter(fcode, pk, origlen, caplen))
+//       {
+//          pkthdr.len = origlen;
+//          pkthdr.caplen = caplen;
+//          /* Insure caplen does not exceed snapshot */
+//          if (pkthdr.caplen > MAXDLBUF)
+//             pkthdr.caplen = MAXDLBUF;
+//
+//          (*callback)(user, &pkthdr, pk);
 
-      Buffer_Put(SocketBuffer, &caplen, sizeof(u_int) );
-      Buffer_Put(SocketBuffer, pk, caplen );
+         Buffer_Put(SocketBuffer, &caplen, sizeof(u_int) );
+         Buffer_Put(SocketBuffer, pk, caplen );
 
+//          if (++n >= cnt && cnt >= 0)
+//          {
+//             p->cc = ep - bp;
+//             p->bp = bp;
+//             return (n);
+//          }
+//       }
 #ifdef HAVE_SYS_BUFMOD_H
    }
 #endif
+//    p->cc = 0;
+//    return (n);
 
    Buffer_Get(SocketBuffer, &pktlen, sizeof(u_int));
    len = Buffer_Get(SocketBuffer, buffer, pktlen );
 
-   if (type != NULL) {
+   if (type != NULL)
+   {
       if (!memcmp(MyMAC,buffer,6))
          *type = 0; // PACKET_HOST
       else
@@ -718,7 +703,7 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 
    c = putmsg(sock, NULL, &data, 0);
    if (c == -1)
-      ERROR_MSG("putmsg()");
+      Error_msg("ec_Inet_solaris:%d putmsg() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    return (len);
 
@@ -728,11 +713,14 @@ int Inet_SendRawPacket(int sock, char *buffer, int len)
 
 int Inet_SetPromisc(char *iface)
 {
-   /*
-    * we have already set the promisc mode when we opened the device
-    */
+   int fd = fdprom;  // fdprom is global and set by Inet_OpenRawSock. it is the last opened socket
+   u_long buf[MAXDLBUF];
 
-   DEBUG_MSG("Inet_SetPromisc \t fd = %d -- WRAPPERED TO NULL (already set)", dlpi_in_use);
+   /*
+   ** Enable promiscuous
+   */
+   if (dlpromisconreq(fd, DL_PROMISC_PHYS, ebuf) < 0 || dlokack(fd, "promisc_phys", (char *)buf, ebuf) < 0)
+      Error_msg("ec_Inet_solaris:%d dlpromisconreq() | ERRNO : %d | %s \n", __LINE__, errno, strerror(errno));
 
    return 0;
 }
@@ -742,10 +730,8 @@ int Inet_SetPromisc(char *iface)
 void Inet_Restore_ifr(void)
 {
 
-   /*
-    *  this function is not needed !!
-    *  when a dlpi is closed, the interface is restored
-    */
+   // this function is not needed !!
+   // when a dlpi is closed, the interface is restored
 
 }
 
@@ -753,189 +739,21 @@ void Inet_Restore_ifr(void)
 
 void Inet_DisableForwarding(void)
 {
-   struct strioctl strIo;
-	int fd;
-
-	char buf[65536];
-	char *cp;
-
-   cp = "ip_forwarding";
-	memset(buf, '\0', sizeof(buf));
-	sprintf(buf, "%s", cp);
-
-	if ((fd = open("/dev/ip", O_RDWR)) < 0)
-		ERROR_MSG("open failed for /dev/ip");
-
-	strIo.ic_cmd = ND_GET;
-	strIo.ic_timout = 0;
-	strIo.ic_len = sizeof(buf);
-	strIo.ic_dp = buf;
-
-	/* Call IOCTL to return status */
-
-	if ( (ioctl(fd, I_STR, (char *)&strIo)) == -1 ) {
-		ERROR_MSG("ioctl(I_STR)");
-	}
-
-	if (strIo.ic_cmd == ND_GET) {
-		strcpy(IpForward_status, buf);
-	}
-
-   DEBUG_MSG("Inet_DisableForwarding -- previous value = %s", IpForward_status);
-
-	memset(buf, '\0', sizeof(buf));
-	sprintf(buf, "%s", cp);
-
-	buf[strlen(buf)+1] = '0'; 	/* the format is "element"\0"value"\0 */
-
-	strIo.ic_cmd = ND_SET;
-	strIo.ic_timout = 0;
-	strIo.ic_len = sizeof(buf);
-	strIo.ic_dp = buf;
-
-	if ( (ioctl(fd, I_STR, (char *)&strIo)) == -1 ) {
-		ERROR_MSG("ioctl(I_STR)");
-	}
-
-   DEBUG_MSG("Inet_DisableForwarding -- NEW value = 0");
-
-	close(fd);
-
-	atexit(Inet_RestoreForwarding);
-
+   Error_msg("SOLARIS PORTING NOT YET FINISHED...\n");
 }
 
 
 
 void Inet_RestoreForwarding(void)
 {
-   struct strioctl strIo;
-	int fd;
-
-	char buf[65536];
-	char *cp;
-
-   cp = "ip_forwarding";
-	memset(buf, '\0', sizeof(buf));
-	sprintf(buf, "%s", cp);
-
-	sprintf(buf + strlen(buf)+1, "%s", IpForward_status); 	/* the format is "element"\0"value"\0 */
-
-   DEBUG_MSG("Inet_RestoreForwarding -- restoring to value = %s", IpForward_status);
-
-	if ((fd = open("/dev/ip", O_RDWR)) < 0)
-		ERROR_MSG("open failed for /dev/ip");
-
-	strIo.ic_cmd = ND_SET;
-	strIo.ic_timout = 0;
-	strIo.ic_len = sizeof(buf);
-	strIo.ic_dp = buf;
-
-	/* Call IOCTL to return status */
-
-	if ( (ioctl(fd, I_STR, (char *)&strIo)) == -1 ) {
-		ERROR_MSG("ioctl(I_STR)");
-	}
-
-	close(fd);
-
+   Error_msg("SOLARIS PORTING NOT YET FINISHED...\n");
 }
 
 
 char *Inet_MacFromIP(unsigned long ip)
 {
-	int sockfd;
-	static struct arpreq arpreq;
-	struct sockaddr_in *sin;
-
-   DEBUG_MSG("Inet_MacFromIP");
-
-	bzero(&arpreq, sizeof(struct arpreq));
-	arpreq.arp_pa.sa_family = AF_INET;
-
-	sin = (struct sockaddr_in *)&arpreq.arp_pa;
-	sin->sin_family = AF_INET;
-	sin->sin_addr.s_addr = ip;
-
-
-	if ( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 )
-		ERROR_MSG("socket()");
-
-	if (ioctl(sockfd, SIOCGARP, (caddr_t)&arpreq) < 0) {
-
-      struct recv_packet recvpck;
-      char MyMAC[6];
-      u_long MyIP;
-      int MTU, sock;
-      TIME_DECLARE;
-
-      DEBUG_MSG("Inet_MacFromIP -- try to find it");
-
-      sock = Inet_OpenRawSock(Options.netiface);
-
-      Inet_GetIfaceInfo(Options.netiface, &MTU, MyMAC, &MyIP, NULL);
-
-      if (ip == MyIP) {
-         DEBUG_MSG("Inet_MacFromIP -- try to find me... ;)");
-
-         memcpy(&arpreq.arp_ha.sa_data, MyMAC, ETHER_ADDR_LEN);
-         Inet_CloseRawSock(sock);
-         close(sockfd);
-         return (char *) arpreq.arp_ha.sa_data;
-      }
-
-      recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD );
-      recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
-
-      Inet_Forge_ethernet( recvpck.aligned, MyMAC, ETH_BROADCAST, ETH_P_ARP );
-
-      Inet_Forge_arp(recvpck.aligned + ETH_HEADER, ARPOP_REQUEST,
-                         MyMAC, MyIP,
-                         ARP_BROADCAST, ip );
-
-      Inet_SendRawPacket(sock, recvpck.aligned, ETH_HEADER + ARP_HEADER);
-
-		memset(recvpck.aligned, 0, MTU);
-
-      fcntl(sock, F_SETFL, O_NONBLOCK);
-      TIME_START;
-
-      do
-      {
-         int len;
-         short pkttype;
-         ETH_header *ethpkt;
-         ARP_header *arppkt;
-
-         len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, &pkttype);
-
-         ethpkt = (ETH_header *)recvpck.aligned;
-         arppkt = (ARP_header *)(ethpkt + 1);
-
-         TIME_FINISH;
-
-         if (len > 0 && pkttype == PACKET_HOST && ethpkt->type == htons(ETH_P_ARP) && arppkt->opcode == htons(ARPOP_REPLY))
-         {
-            if ( *(unsigned long *)arppkt->source_ip == ip )
-            {
-               memcpy(&arpreq.arp_ha.sa_data, &arppkt->source_add, ETHER_ADDR_LEN);
-               Inet_Forge_packet_destroy( recvpck.buf );
-               Inet_CloseRawSock(sock);
-               close(sockfd);
-               return (char *) arpreq.arp_ha.sa_data;
-            }
-         }
-      } while ( TIME_ELAPSED < 0.5 );
-
-		Inet_Forge_packet_destroy( recvpck.buf );
-      Inet_CloseRawSock(sock);
-      close(sockfd);
-      return ETH_BROADCAST;  // workaround for non local ip
-   }
-
-   close(sockfd);
-   return (char *) arpreq.arp_ha.sa_data;
-
+   Error_msg("SOLARIS PORTING NOT YET FINISHED...\n");
+   return "";
 }
 
 
@@ -945,45 +763,43 @@ char *Inet_MacFromIP(unsigned long ip)
  */
 
 static int
-send_request(int fd, char *ptr, int len, char *what, char *ebuf)
+send_request(int fd, char *ptr, int len, char *what, char *ebuf, int flags)
 {
-   struct strbuf ctl;
-	int flags;
+    struct strbuf ctl;
 
-   ctl.maxlen = 0;
-   ctl.len = len;
-   ctl.buf = ptr;
+    ctl.maxlen = 0;
+    ctl.len = len;
+    ctl.buf = ptr;
 
-	flags = 0;
-
-   if (putmsg(fd, &ctl, (struct strbuf *) NULL, flags) < 0) {
-      sprintf(ebuf, "send_request: putmsg \"%s\": %s", what, strerror(errno));
-      DEBUG_MSG("\tDLPI : send_request -- ERROR %s", ebuf);
-      return (-1);
-   }
-   return (0);
+    if (putmsg(fd, &ctl, (struct strbuf *) NULL, flags) < 0)
+    {
+        sprintf(ebuf, "send_request: putmsg \"%s\": %s", what, strerror(errno));
+        return (-1);
+    }
+    return (0);
 }
 
 static int
 recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
 {
-   union DL_primitives *dlp;
-   struct strbuf ctl;
-   int flags;
+    union DL_primitives *dlp;
+    struct strbuf ctl;
+    int flags;
 
-   ctl.maxlen = MAXDLBUF;
-   ctl.len = 0;
-   ctl.buf = bufp;
+    ctl.maxlen = MAXDLBUF;
+    ctl.len = 0;
+    ctl.buf = bufp;
 
-   flags = 0;
-   if (getmsg(fd, &ctl, (struct strbuf*)NULL, &flags) < 0) {
-      sprintf(ebuf, "recv_ack: %s getmsg: %s", what, strerror(errno));
-      DEBUG_MSG("\tDLPI : recv_ack -- ERROR %s", ebuf);
-      return (-1);
-   }
+    flags = 0;
+    if (getmsg(fd, &ctl, (struct strbuf*)NULL, &flags) < 0)
+    {
+        sprintf(ebuf, "recv_ack: %s getmsg: %s", what, strerror(errno));
+        return (-1);
+    }
 
     dlp = (union DL_primitives *)ctl.buf;
-    switch (dlp->dl_primitive) {
+    switch (dlp->dl_primitive)
+    {
         case DL_INFO_ACK:
         case DL_PHYS_ADDR_ACK:
         case DL_BIND_ACK:
@@ -1027,7 +843,6 @@ recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
     return (ctl.len);
 }
 
-/*
 static int
 dlpromiscoffreq(int fd, u_int level, char *ebuf)
 {
@@ -1036,9 +851,8 @@ dlpromiscoffreq(int fd, u_int level, char *ebuf)
     req.dl_primitive = DL_PROMISCOFF_REQ;
     req.dl_level     = level;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "promiscoff", ebuf));
+    return (send_request(fd, (char *)&req, sizeof(req), "promiscoff", ebuf, 0));
 }
-*/
 
 static int
 dlpromisconreq(int fd, u_int level, char *ebuf)
@@ -1048,9 +862,10 @@ dlpromisconreq(int fd, u_int level, char *ebuf)
     req.dl_primitive = DL_PROMISCON_REQ;
     req.dl_level     = level;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "promiscon", ebuf));
+    return (send_request(fd, (char *)&req, sizeof(req), "promiscon", ebuf, 0));
 }
 
+
 static int
 dlattachreq(int fd, u_int ppa, char *ebuf)
 {
@@ -1059,7 +874,7 @@ dlattachreq(int fd, u_int ppa, char *ebuf)
     req.dl_primitive = DL_ATTACH_REQ;
     req.dl_ppa       = ppa;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "attach", ebuf));
+    return (send_request(fd, (char *)&req, sizeof(req), "attach", ebuf, 0));
 }
 
 static int
@@ -1083,7 +898,7 @@ dlbindreq(int fd, u_int sap, char *ebuf)
     req.dl_service_mode = DL_CLDLS;
 #endif
 #endif
-    return (send_request(fd, (char *)&req, sizeof(req), "bind", ebuf));
+    return (send_request(fd, (char *)&req, sizeof(req), "bind", ebuf, 0));
 }
 
 
@@ -1108,7 +923,7 @@ dlinforeq(int fd, char *ebuf)
 
     req.dl_primitive = DL_INFO_REQ;
 
-    return (send_request(fd, (char *)&req, sizeof(req), "info", ebuf));
+    return (send_request(fd, (char *)&req, sizeof(req), "info", ebuf, RS_HIPRI));
 }
 
 static int
@@ -1162,36 +977,5 @@ get_release(u_int *majorp, u_int *minorp, u_int *microp)
 #endif
 
 
-static char *
-split_dname(char *device, int *unitp)
-{
-	char *cp;
-	char *eos;
-	int unit;
-
-	/*
-	 * Look for a number at the end of the device name string.
-	 */
-	cp = device + strlen(device) - 1;
-
-	if (*cp < '0' || *cp > '9')
-		return (NULL);
-
-	/* Digits at end of string are unit number */
-	while (cp-1 >= device && *(cp-1) >= '0' && *(cp-1) <= '9')
-		cp--;
-
-	unit = strtol(cp, &eos, 10);
-	if (*eos != '\0')
-		return (NULL);
-
-	*unitp = unit;
-	return (cp);
-}
-
-
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
