@@ -17,36 +17,39 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector.c,v 1.22 2003/05/28 10:10:52 lordnaga Exp $
+    $Id: ec_dissector.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
-#include "include/ec_packet.h"
 #include "include/ec_decodedata.h"
+#include "include/ec_error.h"
 
-#define ALL_P 0
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
-
-struct state_machine   // state machine double linked list for some protocol dissector
+typedef struct    // state machine double linked list for some protocol dissector
 {
-   pthread_t id;
-   time_t timeout;
    long source_ip;
    long dest_ip;
-   u_short source_port;
-   u_short dest_port;
+   short source_port;
+   short dest_port;
    char proto;
    char state;
    char info[20];
-   LIST_ENTRY (state_machine) next;
-};
+   struct STATE_MACHINE *next;
+   struct STATE_MACHINE *prev;
+} STATE_MACHINE;
 
 
-LIST_HEAD(, state_machine) SM_head;
-
+STATE_MACHINE *SM_head;
 
 DISSECTOR Available_Dissectors[] = {
 
@@ -57,15 +60,11 @@ DISSECTOR Available_Dissectors[] = {
    {MACBASED,  IPPROTO_TCP,    23, &Dissector_telnet,    1, "TELNET"},
    {MACBASED,  IPPROTO_TCP,    80, &Dissector_http,      1, "HTTP"},
    {MACBASED,  IPPROTO_TCP,   110, &Dissector_pop,       1, "POP"},
-   {MACBASED,  IPPROTO_TCP,   111, &Dissector_portmapTCP,1, "RPC"},
-   {MACBASED,  IPPROTO_UDP,   111, &Dissector_portmapUDP,1, "RPC"},
    {MACBASED,  IPPROTO_TCP,   119, &Dissector_nntp,      1, "NNTP"},
    {MACBASED,  IPPROTO_TCP,   139, &Dissector_smb,       1, "SMB"},
    {MACBASED,  IPPROTO_TCP,   143, &Dissector_imap,      1, "IMAP"},
-   {MACBASED,  IPPROTO_UDP,   161, &Dissector_snmp,      1, "SNMP"},
    {MACBASED,  IPPROTO_TCP,   179, &Dissector_bgp,       1, "BGP"},
    {MACBASED,  IPPROTO_TCP,   220, &Dissector_imap,      1, "IMAP"},
-   {MACBASED,  IPPROTO_TCP,   389, &Dissector_ldap,      1, "LDAP"},
 #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
    {PUBLICARP, IPPROTO_TCP,   443, &Dissector_https,     1, "HTTPS"},
 #endif
@@ -78,11 +77,8 @@ DISSECTOR Available_Dissectors[] = {
    {PUBLICARP, IPPROTO_TCP,  Proxy_Local_Port, &Dissector_https,     1, "REVHTTPS"},
 #endif
    {MACBASED,  IPPROTO_TCP,  1080, &Dissector_socks,     1, "SOCKS"},
-   {MACBASED,  IPPROTO_TCP,  1863, &Dissector_msn,       1, "MSN"},
    {MACBASED,  IPPROTO_TCP,  3306, &Dissector_mysql,     1, "MYSQL"},
-   {MACBASED,  IPPROTO_UDP,  4000, &Dissector_icqv5,     1, "ICQ"},
-   {MACBASED,  IPPROTO_TCP,  5050, &Dissector_ymsg,      1, "YMSG"},
-   {MACBASED,  IPPROTO_TCP,  5190, &Dissector_icqv7,     1, "ICQ"},
+   {MACBASED,  IPPROTO_UDP,  4000, &Dissector_icq,       1, "ICQ"},
    {MACBASED,  IPPROTO_TCP,  5900, &Dissector_vnc,       1, "VNC"},
    {MACBASED,  IPPROTO_TCP,  5901, &Dissector_vnc,       1, "VNC"},
    {MACBASED,  IPPROTO_TCP,  5902, &Dissector_vnc,       1, "VNC"},
@@ -101,31 +97,15 @@ DISSECTOR Available_Dissectors[] = {
 #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
    {PUBLICARP, IPPROTO_TCP,  8080, &Dissector_https,     1, "PROXYHTTPS"},    // for proxies https
 #endif
-   {MACBASED,  IPPROTO_TCP,  8080, &Dissector_proxy,     1, "PROXY"},         // for proxies
+   {MACBASED,  IPPROTO_TCP,  8080, &Dissector_http,      1, "HTTP"},          // for proxies
    {MACBASED,  IPPROTO_TCP,  8888, &Dissector_napster,   1, "NAPSTER"},
-   {MACBASED,  IPPROTO_UDP, 27015, &Dissector_hl_rcon,   1, "HL-RCON"},
-   {MACBASED,  IPPROTO_UDP, 27960, &Dissector_quake3,    1, "QUAKE3"},
-   {MACBASED,  IPPROTO_UDP, 27961, &Dissector_quake3,    1, "QUAKE3"},
-   {MACBASED,  IPPROTO_UDP, 27962, &Dissector_quake3,    1, "QUAKE3"},
    {MACBASED,  IPPROTO_TCP, 65301, &Dissector_pcanywhere,1, "PCANYWHERE"},
-   {MACBASED,  IPPROTO_UDP, ALL_P, &Dissector_icqv5,     1, "ICQ"},
    {       0,            0,     0,                 NULL, 0, ""}
 };
 
-RPC_DISSECTOR Available_RPC_Dissectors[] = {
-   {100005,  1, IPPROTO_TCP, &Dissector_mountdTCP, NULL},
-   {100005,  1, IPPROTO_UDP, &Dissector_mountdUDP, NULL},
-   {100005,  2, IPPROTO_TCP, &Dissector_mountdTCP, NULL},
-   {100005,  2, IPPROTO_UDP, &Dissector_mountdUDP, NULL},
-   {100005,  3, IPPROTO_TCP, &Dissector_mountdTCP, NULL},
-   {100005,  3, IPPROTO_UDP, &Dissector_mountdUDP, NULL},
-   {     0,  0,           0, NULL,                 NULL}
-};
-
 // protos....
 
-int Dissector_Connections( char mode, short proto, u_char *data, CONNECTION *data_to_ettercap, SNIFFED_DATA *sniff_data_to_ettercap, int Conn_Mode );
-int Dissector_SetDissector(struct packet_object *po);
+int Dissector_Connections( char mode, short proto, u_char *data, void *data_to_ettercap, int Conn_Mode );
 void Dissector_SetHandle( char *name, char active, short port, short proto);
 int Dissector_StateMachine_GetStatus(CONNECTION *data_to_ettercap, char *info);
 int Dissector_StateMachine_SetStatus(CONNECTION *data_to_ettercap, char status, char *info);
@@ -134,143 +114,70 @@ int Dissector_base64decode(char *bufplain, const char *bufcoded);  // stolen fro
 // -------------------------------------
 
 
-int Dissector_Connections( char mode, short proto, u_char *data, CONNECTION *data_to_ettercap, SNIFFED_DATA *sniff_data_to_ettercap, int Conn_Mode )
+int Dissector_Connections( char mode, short proto, u_char *data, void *data_to_ettercap, int Conn_Mode )
 {
 
    TCP_header *tcp;
    UDP_header *udp;
    DISSECTOR *ds;
-   RPC_DISSECTOR *rds;
 
    switch(proto)
    {
       case IPPROTO_TCP:
                         tcp = (TCP_header *) data;
 
-                        for( ds = Available_Dissectors; ds->dissector != NULL; ds++)
+                        for( ds = Available_Dissectors; ds->port != 0; ds++)
                         {
-                           if ( ds->active && (ntohs(tcp->source) == ds->port || ntohs(tcp->dest) == ds->port || ds->port == ALL_P)
-                             && ds->proto == IPPROTO_TCP && mode <= ds->mode  )
+                           if ( ds->active && (ntohs(tcp->source) == ds->port || ntohs(tcp->dest) == ds->port) && ds->proto == IPPROTO_TCP && mode <= ds->mode  )
                            {
                               if (ds->mode <= PUBLICARP)
                               {
                                  if (active_dissector)      // activated by user in iterface_sniff
-                                    return ds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ds->port);
+                                    return ds->dissector(data, data_to_ettercap, Conn_Mode, ds->port);
                               }
                               else
-                                 return ds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ds->port);
+                                 return ds->dissector(data, data_to_ettercap, Conn_Mode, ds->port);
 
                               break;
                            }
                         }
-
-                        for ( rds = Available_RPC_Dissectors;  rds->program != 0; rds ++)
-                        {
-                           if ( rds->proto == IPPROTO_TCP)
-                           {
-                              RPC_PORTS *ports;
-                              ports = rds->ports;
-
-                              while (ports)
-                              {
-                                 if (ports->port == ntohs(tcp->dest) || ports->port == ntohs(tcp->source))
-                                    return rds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ports->port);
-
-                                 ports = (RPC_PORTS *)ports->next;
-                              }
-                           }
-                        }
-
                         break;
       case IPPROTO_UDP:
                         udp = (UDP_header *) data;
 
-                        for( ds = Available_Dissectors; ds->dissector != NULL; ds++)
+                        for( ds = Available_Dissectors; ds->port != 0; ds++)
                         {
-                           if ( ds->active && (ntohs(udp->source) == ds->port || ntohs(udp->dest) == ds->port || ds->port == ALL_P)
-                             && ds->proto == IPPROTO_UDP && mode <= ds->mode  )
+                           if ( ds->active && (ntohs(udp->source) == ds->port || ntohs(udp->dest) == ds->port) && ds->proto == IPPROTO_UDP && mode <= ds->mode  )
                            {
                               if (ds->mode <= PUBLICARP)
                               {
                                  if (active_dissector)      // activated by user in iterface_sniff
-                                    return ds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ds->port);
+                                    return ds->dissector(data, data_to_ettercap, Conn_Mode, ds->port);
                               }
                               else
-                                 return ds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ds->port);
+                                 return ds->dissector(data, data_to_ettercap, Conn_Mode, ds->port);
 
                               break;
                            }
                         }
-
-                        for ( rds = Available_RPC_Dissectors;  rds->program != 0; rds ++)
-                        {
-                           if ( rds->proto == IPPROTO_UDP)
-                           {
-                              RPC_PORTS *ports;
-                              ports = rds->ports;
-
-                              while (ports)
-                              {
-                                 if (ports->port == ntohs(udp->dest) || ports->port == ntohs(udp->source))
-                                    return rds->dissector(data, data_to_ettercap, sniff_data_to_ettercap, Conn_Mode, ports->port);
-
-                                 ports = (RPC_PORTS *)ports->next;
-                              }
-                           }
-                        }
-
                         break;
    }
    return 0;
 }
 
-
-
-int Dissector_SetDissector(struct packet_object *po)
-{
-   DISSECTOR *ds;
-   RPC_DISSECTOR *rds;
-
-   for( ds = Available_Dissectors; ds->dissector != NULL; ds++) {
-      if ( po->L4.proto == ds->proto && (ds->port == *po->L4.sport || ds->port == *po->L4.dport) ) {
-
-         po->decode_mode = ds->mode;
-         // po->decode = ds->dissector;
-
-         break;
-      }
-   }
-
-   for ( rds = Available_RPC_Dissectors;  rds->program != 0; rds ++) {
-      if ( po->L4.proto == rds->proto ) {
-
-         RPC_PORTS *ports;
-         ports = rds->ports;
-
-         while (ports) {
-            if (ports->port == *po->L4.dport || ports->port == *po->L4.sport) {
-               // po->decode = rds->dissector;
-               break;
-            }
-            ports = (RPC_PORTS *)ports->next;
-         }
-      }
-   }
-
-   return 0;
-}
-
-
-
 void Dissector_SetHandle( char *name, char active, short port, short proto)
 {
    DISSECTOR *ds;
    char found=0;
 
-   DEBUG_MSG("\tDissector_SetHandle - %s %d %d %d", name, active, port, proto);
+#ifdef DEBUG
+   Debug_msg("\tDissector_SetHandle - %s %d %d %d", name, active, port, proto);
+#endif
 
-   for( ds = Available_Dissectors; ds->dissector != NULL; ds++) {
-      if (!strcasecmp(name, ds->name)) {
+   for( ds = Available_Dissectors; ds->port != 0; ds++)
+   {
+      if (!strcasecmp(name, ds->name))
+      {
          ds->active = active;
          found = 1;
 
@@ -290,122 +197,109 @@ void Dissector_SetHandle( char *name, char active, short port, short proto)
 
 int Dissector_StateMachine_GetStatus(CONNECTION *data_to_ettercap, char *info)
 {
-   struct state_machine *ptr;
-   struct state_machine *to_free=NULL;
-   
-   LIST_FOREACH(ptr, &SM_head, next) {
-      
-      if (to_free) free(to_free);
-      to_free=NULL;
-      
-      if (ptr->timeout < time(NULL) - Options.refresh_delay) {      // remove old entries...
-         LIST_REMOVE(ptr, next);
-         //free(ptr);
-         //return 0;
-         to_free=ptr;     
-      }
-      else
-      if ( ptr->id == pthread_self() &&
-           ptr->proto == data_to_ettercap->proto &&
-           ((ptr->source_ip == inet_addr(data_to_ettercap->source_ip) &&     // straight
-             ptr->dest_ip == inet_addr(data_to_ettercap->dest_ip) &&
-             ptr->source_port == data_to_ettercap->source_port &&
-             ptr->dest_port == data_to_ettercap->dest_port)
-             ||
-             (ptr->source_ip == inet_addr(data_to_ettercap->dest_ip) &&       // reverse
-             ptr->dest_ip == inet_addr(data_to_ettercap->source_ip) &&
-             ptr->source_port == data_to_ettercap->dest_port &&
-             ptr->dest_port == data_to_ettercap->source_port))
-         )
-      {
-         if (info != NULL) strlcpy(info, ptr->info, 20);
-         return ptr->state;
-      }
-   }
-   
-   if (to_free) free(to_free);
-   
+   STATE_MACHINE *ptr = SM_head;
+
+   if (SM_head)
+      for(ptr = SM_head; ptr != NULL; ptr = (STATE_MACHINE *)ptr->next)
+         if ( ptr->proto == data_to_ettercap->proto &&
+              ((ptr->source_ip == inet_addr(data_to_ettercap->source_ip) &&     // straight
+                ptr->dest_ip == inet_addr(data_to_ettercap->dest_ip) &&
+                ptr->source_port == data_to_ettercap->source_port &&
+                ptr->dest_port == data_to_ettercap->dest_port)
+                ||
+                (ptr->source_ip == inet_addr(data_to_ettercap->dest_ip) &&       // reverse
+                ptr->dest_ip == inet_addr(data_to_ettercap->source_ip) &&
+                ptr->source_port == data_to_ettercap->dest_port &&
+                ptr->dest_port == data_to_ettercap->source_port))
+            )
+         {
+               if (info != NULL) strncpy(info, ptr->info, 20);
+               return ptr->state;
+         }
+
    return 0;
 }
 
 
 int Dissector_StateMachine_SetStatus(CONNECTION *data_to_ettercap, char status, char *info)
 {
-   struct state_machine *ptr, *current;
-   struct state_machine *to_free=NULL;
-   
-   LIST_FOREACH(ptr, &SM_head, next) {
-      
-      if (to_free) free(to_free);
-      to_free=NULL;
-   
-      if (ptr->timeout < time(NULL) - Options.refresh_delay) {    // remove old entries...
-         LIST_REMOVE(ptr, next);
-         //free(ptr);
-         //return 0;
-	 to_free=ptr;
-      }
-      else
-      if ( ptr->id == pthread_self() &&
-           ptr->proto == data_to_ettercap->proto &&
-           ((ptr->source_ip == inet_addr(data_to_ettercap->source_ip) &&     // straight
-             ptr->dest_ip == inet_addr(data_to_ettercap->dest_ip) &&
-             ptr->source_port == data_to_ettercap->source_port &&
-             ptr->dest_port == data_to_ettercap->dest_port)
-             ||
-             (ptr->source_ip == inet_addr(data_to_ettercap->dest_ip) &&       // reverse
-             ptr->dest_ip == inet_addr(data_to_ettercap->source_ip) &&
-             ptr->source_port == data_to_ettercap->dest_port &&
-             ptr->dest_port == data_to_ettercap->source_port))
-          )
+   STATE_MACHINE *sm_index = SM_head;
+   STATE_MACHINE *ptr = sm_index;
+   STATE_MACHINE *current;
+   char found = 0;
+
+   if (SM_head)
+   {
+      for(ptr = SM_head; ptr != NULL; ptr = (STATE_MACHINE *)ptr->next)
       {
-            DEBUG_MSG("\tDissector_StateMachine_SetStatus - (%d)! %c %s:%d - %s:%d -- [%s]",
-                        status, data_to_ettercap->proto,
-                        data_to_ettercap->source_ip,
-                        data_to_ettercap->source_port,
-                        data_to_ettercap->dest_ip,
-                        data_to_ettercap->dest_port,
-                        info);
-         if (status) {
-            ptr->timeout = time(NULL);
-            ptr->state = status;
-            if (info != NULL) strlcpy(ptr->info, info, 20);
+         sm_index = ptr;
+
+         if ( ptr->proto == data_to_ettercap->proto &&
+              ((ptr->source_ip == inet_addr(data_to_ettercap->source_ip) &&     // straight
+                ptr->dest_ip == inet_addr(data_to_ettercap->dest_ip) &&
+                ptr->source_port == data_to_ettercap->source_port &&
+                ptr->dest_port == data_to_ettercap->dest_port)
+                ||
+                (ptr->source_ip == inet_addr(data_to_ettercap->dest_ip) &&       // reverse
+                ptr->dest_ip == inet_addr(data_to_ettercap->source_ip) &&
+                ptr->source_port == data_to_ettercap->dest_port &&
+                ptr->dest_port == data_to_ettercap->source_port))
+             )
+         {
+            #ifdef DEBUG
+               Debug_msg("\tDissector_StateMachine_SetStatus - (%d)! %c %s:%d - %s:%d -- [%s]", status, data_to_ettercap->proto,
+                                                                                                data_to_ettercap->source_ip,
+                                                                                                data_to_ettercap->source_port,
+                                                                                                data_to_ettercap->dest_ip,
+                                                                                                data_to_ettercap->dest_port,
+                                                                                                info);
+            #endif
+            if (status)
+            {
+               if (info != NULL) strncpy(ptr->info, info, 20);
+               ptr->state = status;
+            }
+            else
+            {
+               if (ptr->next) ((STATE_MACHINE *)ptr->next)->prev = (struct STATE_MACHINE *)ptr->prev;
+               if (ptr->prev) ((STATE_MACHINE *)ptr->prev)->next = (struct STATE_MACHINE *)ptr->next;
+               if (ptr == SM_head) SM_head = (STATE_MACHINE *)ptr->next;
+               free(ptr);
+            }
+            found = 1;
             return 0;
-         } else {
-            LIST_REMOVE(ptr, next);
-            free(ptr);
          }
-         return 0;
       }
    }
-   
-   if (to_free) free(to_free);
-   
-   if (status) {
-         DEBUG_MSG("\tDissector_StateMachine_SetStatus - new item - state (%d)! %c %s:%d - %s:%d -- [%s]",
-                      status, data_to_ettercap->proto,
-                      data_to_ettercap->source_ip,
-                      data_to_ettercap->source_port,
-                      data_to_ettercap->dest_ip,
-                      data_to_ettercap->dest_port,
-                      info);
 
-      current = (struct state_machine *)calloc(1, sizeof(struct state_machine));
+   if (!found && status)
+   {
+      #ifdef DEBUG
+         Debug_msg("\tDissector_StateMachine_SetStatus - new item - state (%d)! %c %s:%d - %s:%d -- [%s]", status, data_to_ettercap->proto,
+                                                                                                           data_to_ettercap->source_ip,
+                                                                                                           data_to_ettercap->source_port,
+                                                                                                           data_to_ettercap->dest_ip,
+                                                                                                           data_to_ettercap->dest_port,
+                                                                                                           info);
+      #endif
+
+      current = (STATE_MACHINE *)calloc(1, sizeof(STATE_MACHINE));
       if (current == NULL)
          ERROR_MSG("calloc()");
 
-      current->id = pthread_self();
-      current->timeout = time(NULL);
       current->source_ip = inet_addr(data_to_ettercap->source_ip);
       current->dest_ip = inet_addr(data_to_ettercap->dest_ip);
       current->source_port = data_to_ettercap->source_port;
       current->dest_port = data_to_ettercap->dest_port;
       current->proto = data_to_ettercap->proto;
       current->state = status;
-      if (info != NULL) strlcpy(current->info, info, 20);
+      if (info != NULL) strncpy(current->info, info, 20);
 
-      LIST_INSERT_HEAD(&SM_head, current, next);
+      current->prev = (struct STATE_MACHINE *)sm_index;
+      if (SM_head == NULL) SM_head = current;
+      if (sm_index != NULL) sm_index->next = (struct STATE_MACHINE *)current;
    }
+
    return 0;
 }
 
@@ -476,6 +370,3 @@ int Dissector_base64decode(char *bufplain, const char *bufcoded)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
