@@ -17,18 +17,26 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_https.c,v 1.6 2002/11/04 22:25:33 alor Exp $
+    $Id: ec_dissector_https.c,v 1.1.1.1 2001/08/27 19:26:27 alor Exp $
 */
 
 #include "include/ec_main.h"
 
 #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
 
 #include "include/ec_dissector.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 extern int Grell_ProxyPort;
 
@@ -42,7 +50,6 @@ typedef struct {
     struct peer match;
     u_long  source_ip;
     u_long dest_ip;
-    char source_mac[6];
     struct ssl_state *next;
 } ssl_state;
 
@@ -83,11 +90,22 @@ FUNC_DISSECTOR(Dissector_https)
 
    tcp = (TCP_header *) data;
 
-   IPS = inet_addr(data_to_ettercap->source_ip);
-   IPD = inet_addr(data_to_ettercap->dest_ip);
-   PORTS = data_to_ettercap->source_port;
-   PORTD = data_to_ettercap->dest_port;
-   datalen = data_to_ettercap->datalen;
+   if (Conn_Mode)
+   {
+      IPS = inet_addr(data_to_ettercap->source_ip);
+      IPD = inet_addr(data_to_ettercap->dest_ip);
+      PORTS = data_to_ettercap->source_port;
+      PORTD = data_to_ettercap->dest_port;
+      datalen = data_to_ettercap->datalen;
+   }
+   else
+   {
+      IPS = inet_addr(sniff_data_to_ettercap->source_ip);
+      IPD = inet_addr(sniff_data_to_ettercap->dest_ip);
+      PORTS = sniff_data_to_ettercap->source_port;
+      PORTD = sniff_data_to_ettercap->dest_port;
+      datalen = sniff_data_to_ettercap->datasize;
+   }
 
    if (PORTD==443 || PORTD==Grell_ProxyPort) // client side packets
    {
@@ -117,7 +135,6 @@ FUNC_DISSECTOR(Dissector_https)
           memcpy(&(ssl_conn->match), &pckpeer, sizeof(pckpeer));
           ssl_conn->source_ip = IPS;
           ssl_conn->dest_ip = IPD;
-          memcpy(ssl_conn->source_mac, data_to_ettercap->source_mac, 6);
           ssl_conn->next = (struct ssl_state *)temp;
       }
 
@@ -149,11 +166,10 @@ FUNC_DISSECTOR(Dissector_https)
    {
       ssl_state **index_ssl;
       struct peer pckpeer;
-      //char *DestMAC = NULL;
-      //char StaticMAC[6];
-      //int i;
+      char *DestMAC = NULL;
+      char StaticMAC[6];
       char *pck_to_send;
-
+      int i;
 
       if ( (IPD & htonl(0xff000000)) != htonl(0x01000000) )
           return 0;  // Not caught: forward
@@ -181,7 +197,7 @@ FUNC_DISSECTOR(Dissector_https)
       index_ssl = (ssl_state **)&((*index_ssl)->next);
 
       if (*index_ssl==NULL) return -1; // Oh my God!!! Where are you from????
-/*
+
       for(i = 1; i<number_of_hosts_in_lan; i++)
       {
          if (inet_addr(Host_In_LAN[i].ip) == (*index_ssl)->source_ip)
@@ -193,9 +209,9 @@ FUNC_DISSECTOR(Dissector_https)
       }
       if (i>=number_of_hosts_in_lan)
          DestMAC = Inet_MacFromIP( (*index_ssl)->source_ip );
-*/
+
       pck_to_send = buff;
-      pck_to_send += Inet_Forge_ethernet( pck_to_send, MyMAC, (*index_ssl)->source_mac, ETH_P_IP );
+      pck_to_send += Inet_Forge_ethernet( pck_to_send, MyMAC, DestMAC, ETH_P_IP );
       pck_to_send += Inet_Forge_ip( pck_to_send, (*index_ssl)->dest_ip, (*index_ssl)->source_ip, (tcp->doff * 4)+datalen, 0xE77E, 0, IPPROTO_TCP);
 
       tcp->checksum=0;
@@ -205,7 +221,10 @@ FUNC_DISSECTOR(Dissector_https)
       Inet_SendRawPacket(sockpck, buff, ETH_HEADER + IP_HEADER + (tcp->doff * 4) + datalen);
    }
    else
-      if (IPD != MyIP && (PORTS==443 || PORTS==Grell_ProxyPort)) return 0; 
+   {
+      // Not Again!!!!!!!!!!!!!!
+      // return 0;
+   }
 
     return -1;  // dont forward and dont put in the buffer.
 }
@@ -213,6 +232,3 @@ FUNC_DISSECTOR(Dissector_https)
 #endif
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
