@@ -17,13 +17,22 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_imap.c,v 1.8 2002/04/18 19:23:54 alor Exp $
+    $Id: ec_dissector_imap.c,v 1.1.1.1 2001/08/27 19:26:01 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos
 
@@ -42,35 +51,27 @@ FUNC_DISSECTOR(Dissector_imap)
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
-   if (data_to_ettercap->datalen == 0) return 0;      // no data...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;           // skip server messages...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
 
-   if ( ((fromhere = strstr(payload, "LOGIN")) || (fromhere = strstr(payload, "login"))) && !strstr(payload, "AUTHENTICATE") )    // plain text login
+   if ( (fromhere = strstr(payload, "LOGIN")) && !strstr(payload, "AUTHENTICATE") )    // plain text login
    {
       char collector[80];
-      char *q;
+      char *p;
 
-		/*
-		 * the format is:
-		 *   n login "user" "pass".
-		 */
+      #ifdef DEBUG
+         Debug_msg("\tDissector_IMAP LOGIN ");
+      #endif
 
-      DEBUG_MSG("\tDissector_IMAP LOGIN ");
-
-      strlcpy(collector, fromhere + strlen("LOGIN "), sizeof(collector) );
-
-      do {
-			q = strchr(collector, '\"');
-			if (!q) break;
-			*q = ' ';
-		} while(q < collector + strlen(collector));
-
-		sscanf(collector, "%45s %45s", data_to_ettercap->user, data_to_ettercap->pass);
-
-      strcat(data_to_ettercap->user, "\n");
-      strcat(data_to_ettercap->pass, "\n");
+      strncpy(collector, fromhere + strlen("LOGIN ") , 80 );
+      p = strtok(collector, " ");
+      if (p)
+      {
+         snprintf(data_to_ettercap->user, 25, "%s\n", p);
+         p = strtok(NULL, " ");
+         if (p) snprintf(data_to_ettercap->pass, 25, "%s\n", p);
+      }
 
       return 0;
    }
@@ -78,57 +79,65 @@ FUNC_DISSECTOR(Dissector_imap)
    switch (Dissector_StateMachine_GetStatus(data_to_ettercap, NULL))
    {
       case 0: // look for authenticate type
-               if ( (fromhere = strstr(payload, "AUTHENTICATE")) ) {
+               if ( (fromhere = strstr(payload, "AUTHENTICATE")) )
+               {
                   char collector[80];
 
-                  DEBUG_MSG("\tDissector_IMAP %s", payload);
+                  #ifdef DEBUG
+                     Debug_msg("\tDissector_IMAP %s", payload);
+                  #endif
 
-                  strlcpy(collector, fromhere + strlen("AUTHENTICATE "), sizeof(collector) );
+                  strncpy(collector, fromhere + strlen("AUTHENTICATE ") , 80 );
 
                   if (!strncasecmp(collector, "LOGIN", 5))
+                  {
                      Dissector_StateMachine_SetStatus(data_to_ettercap, 1, NULL);   // wait for the next packet in which there will be the login (base64)
+                  }
                   else if (!strncasecmp(collector, "SKEY", 5))
+                  {
                      Dissector_StateMachine_SetStatus(data_to_ettercap, 3, NULL);   // wait for the next packet in which there will be the login (base64)
-                  else {
-                     sprintf(data_to_ettercap->user, "\n");
-                     sprintf(data_to_ettercap->pass, "\n");
-                     snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "Authenticated with %s\n", collector);
+                  }
+                  else
+                  {
+                     snprintf(data_to_ettercap->user, 25, "\n");
+                     snprintf(data_to_ettercap->pass, 25, "\n");
+                     snprintf(data_to_ettercap->info, 80, "Authenticated with %s\n", collector);
                   }
                }
                break;
       case 1: // waiting for user
                {
                   char collector[strlen(payload)];
-                  
-                  DEBUG_MSG("\tDissector_IMAP AUTH LOGIN USER");
-                  
+                  #ifdef DEBUG
+                     Debug_msg("\tDissector_IMAP AUTH LOGIN USER");
+                  #endif
                   Dissector_base64decode(collector, payload);
-                  snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user)-1, "%s\n", collector);
+                  snprintf(data_to_ettercap->user, 25, "%s\n", collector);
                   Dissector_StateMachine_SetStatus(data_to_ettercap, 2, NULL);
                }
                break;
       case 2: // waiting for pass
                {
                   char collector[strlen(payload)];
-                  
-                  DEBUG_MSG("\tDissector_IMAP AUTH LOGIN PASS");
-                  
+                  #ifdef DEBUG
+                     Debug_msg("\tDissector_IMAP AUTH LOGIN PASS");
+                  #endif
                   Dissector_base64decode(collector, payload);
-                  snprintf(data_to_ettercap->pass, sizeof(data_to_ettercap->pass)-1, "%s\n", collector);
-                  snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "Authenticated LOGIN\n");
+                  snprintf(data_to_ettercap->pass, 25, "%s\n", collector);
+                  snprintf(data_to_ettercap->info, 80, "Authenticated LOGIN\n");
                   Dissector_StateMachine_SetStatus(data_to_ettercap, 0, NULL);
                }
                break;
       case 3: // auth SKEY we can know only the login... the pass is One-Timed... argh!!
                {
                   char collector[strlen(payload)];
-                  
-                  DEBUG_MSG("\tDissector_IMAP AUTH SKEY USER");
-                  
+                  #ifdef DEBUG
+                     Debug_msg("\tDissector_IMAP AUTH SKEY USER");
+                  #endif
                   Dissector_base64decode(collector, payload);
-                  snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user)-1, "%s\n", collector);
-                  sprintf(data_to_ettercap->pass, "\n");
-                  snprintf(data_to_ettercap->info, sizeof(data_to_ettercap->info), "Authenticated with %s\n", collector);
+                  snprintf(data_to_ettercap->user, 25, "%s\n", collector);
+                  snprintf(data_to_ettercap->pass, 25, "\n");
+                  snprintf(data_to_ettercap->info, 80, "Authenticated with %s\n", collector);
                   Dissector_StateMachine_SetStatus(data_to_ettercap, 0, NULL);
                }
                break;
@@ -139,6 +148,3 @@ FUNC_DISSECTOR(Dissector_imap)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
