@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_dissector_mysql.c,v 1.1.1.1 2001/08/27 19:25:40 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -45,7 +47,7 @@ FUNC_DISSECTOR(Dissector_mysql)
    TCP_header *tcp;
    u_char *payload;
    u_char collector[MAX_DATA];
-   char seed[8];
+   char seed[9];
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
@@ -58,7 +60,7 @@ FUNC_DISSECTOR(Dissector_mysql)
    memcpy(collector, payload, data_to_ettercap->datalen);
 
 
-   if ( (Dissector_StateMachine_GetStatus(data_to_ettercap) == 0) && ntohs(tcp->source) == SERV_PORT)            // server messages... collect the random seed
+   if ( (Dissector_StateMachine_GetStatus(data_to_ettercap, NULL) == 0) && ntohs(tcp->source) == SERV_PORT)            // server messages... collect the random seed
    {
       int i = 5;  // skip first five byte
 
@@ -70,15 +72,15 @@ FUNC_DISSECTOR(Dissector_mysql)
 
       while(collector[i] != collector[i-1] != collector[i-2] != 0)   // search for 000 padding
          i++;
-
+      memset(seed, 0, 9);
       strncpy(seed, collector + i + 1, 8);
       sprintf(data_to_ettercap->info, "Encrypted (one way) seed: %s  pass: ", seed);
 
-      Dissector_StateMachine_SetStatus(data_to_ettercap, 1);
+      Dissector_StateMachine_SetStatus(data_to_ettercap, 1, NULL);
       return 0;
    }
 
-   if ( (Dissector_StateMachine_GetStatus(data_to_ettercap) == 1) && ntohs(tcp->dest) == SERV_PORT)  // client response crypt pass with seed
+   if ( (Dissector_StateMachine_GetStatus(data_to_ettercap, NULL) == 1) && ntohs(tcp->dest) == SERV_PORT)  // client response crypt pass with seed
    {
       char user[25];
       char pass[25];
@@ -98,7 +100,7 @@ FUNC_DISSECTOR(Dissector_mysql)
       strcat(data_to_ettercap->info, "\n");
       sprintf(data_to_ettercap->type, "MySQL");
 
-      Dissector_StateMachine_SetStatus(data_to_ettercap, 0);
+      Dissector_StateMachine_SetStatus(data_to_ettercap, 0, NULL);
     }
 
     return 0;
