@@ -17,13 +17,22 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_nntp.c,v 1.5 2002/03/08 08:49:55 alor Exp $
+    $Id: ec_dissector_nntp.c,v 1.1.1.1 2001/08/27 19:25:46 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos
 
@@ -41,21 +50,28 @@ FUNC_DISSECTOR(Dissector_nntp)
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;     // skip server messages...
-   if (data_to_ettercap->datalen == 0) return 0;      // no data...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
 
    if ( !strncasecmp(payload, "AUTHINFO USER", 13) )
    {
-      DEBUG_MSG("\tDissector_NNTP USER");
-      snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user), "%s\n", payload + 14);
+      char user[25];
+      #ifdef DEBUG
+         Debug_msg("\tDissector_NNTP USER");
+      #endif
+      strncpy(user, payload + 14 , 25 );
+      strcat(data_to_ettercap->user, user);
    }
 
    if ( !strncasecmp(payload, "AUTHINFO PASS", 13) )
    {
-      DEBUG_MSG("\tDissector_NNTP PASS");
-      snprintf(data_to_ettercap->pass, sizeof(data_to_ettercap->pass), "%s\n", payload + 14);
+      char pass[25];
+      #ifdef DEBUG
+         Debug_msg("\tDissector_NNTP PASS");
+      #endif
+      strncpy(pass, payload + 14 , 25 );
+      strcat(data_to_ettercap->pass, pass);
    }
 
    return 0;
@@ -63,6 +79,3 @@ FUNC_DISSECTOR(Dissector_nntp)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
