@@ -17,17 +17,25 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_rlogin.c,v 1.6 2002/06/27 17:27:37 alor Exp $
+    $Id: ec_dissector_rlogin.c,v 1.1.1.1 2001/08/27 19:25:25 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
 #ifdef HAVE_CTYPE_H
    #include <ctype.h>
 #endif
 
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos
 
@@ -50,20 +58,20 @@ FUNC_DISSECTOR(Dissector_rlogin)
 
    TCP_header *tcp;
    u_char *payload;
-   char collector[31];
+   char collector[30];
    int datalen;
 
    ONLY_CONNECTION;
 
    tcp = (TCP_header *) data;
 
-   if (ntohs(tcp->source) == SERV_PORT) return 0;      // skip server messages...
+   if (ntohs(tcp->source) == SERV_PORT) return 0;            // skip server messages...
    if (data_to_ettercap->datalen == 0) return 0;       // no data...
 
    payload = (char *)((int)tcp + tcp->doff * 4);
 
-   memset(collector, 0, sizeof(collector));
-   datalen = (data_to_ettercap->datalen >= sizeof(collector)) ? (sizeof(collector)-1) : data_to_ettercap->datalen;
+   memset(collector, 0, 30);
+   datalen = (data_to_ettercap->datalen > 30) ? 30 : data_to_ettercap->datalen;
    memcpy(collector, payload, datalen);
 
    if (strcmp(collector, ""))
@@ -81,18 +89,20 @@ FUNC_DISSECTOR(Dissector_rlogin)
 
       if (strcmp(collector, ""))                      // again on modified collector
       {
+         char user[30];
          char *remoteuser;
 
          remoteuser = collector+strlen(collector)+1;
-         snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user)-2, "[%s]:[%s]\n", collector, remoteuser);
-         data_to_ettercap->user[sizeof(data_to_ettercap->user)-1] = 0;
+         snprintf(user, 29, "[%s]:[%s]\n", collector, remoteuser);
+//         snprintf(data_to_ettercap->info, 100, "TERM INFO : %s\n", remoteuser+strlen(remoteuser)+1);
 
+         strncpy(data_to_ettercap->user, user, 29);
          if ( !strcmp(remoteuser, "") )  // it isn't the login
-            strlcpy(data_to_ettercap->pass, collector, sizeof(data_to_ettercap->pass));
+            strncpy(data_to_ettercap->pass, collector, 29);
       }
       else if (end)                                   // password needs to be terminated...
       {
-         sprintf(data_to_ettercap->pass, "\n");
+         strcpy(data_to_ettercap->pass, "\n");
       }
    }
 
@@ -101,6 +111,3 @@ FUNC_DISSECTOR(Dissector_rlogin)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
