@@ -17,13 +17,22 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_dissector_x11.c,v 1.5 2002/03/08 08:49:55 alor Exp $
+    $Id: ec_dissector_x11.c,v 1.1.1.1 2001/08/27 19:25:47 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "include/ec_dissector.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos
 
@@ -39,8 +48,6 @@ FUNC_DISSECTOR(Dissector_x11)
    u_char *payload;
    ONLY_CONNECTION;
 
-   if (data_to_ettercap->datalen == 0) return 0;      // no data...
-
    tcp = (TCP_header *) data;
 
    payload = (char *)((int)tcp + tcp->doff * 4);
@@ -51,26 +58,24 @@ FUNC_DISSECTOR(Dissector_x11)
    {
       int i;
 
-      DEBUG_MSG("\tDissector_X11");
-      
+      #ifdef DEBUG
+         Debug_msg("\tDissector_X11");
+      #endif
       if (ntohs(tcp->dest) >= SERV_PORT && ntohs(tcp->dest) <= SERV_PORT+4)
-         snprintf(data_to_ettercap->user, sizeof(data_to_ettercap->user), "On display :%d\n", ntohs(tcp->dest)-SERV_PORT);
+         sprintf(data_to_ettercap->user, "On display :%d\n", ntohs(tcp->dest)-SERV_PORT);
       else
          sprintf(data_to_ettercap->user, "\n");
-      sprintf(data_to_ettercap->pass, "\n");
+      strcat(data_to_ettercap->pass, "\n");
       sprintf(data_to_ettercap->info, "MIT-MAGIC-COOKIE-1 ");
-      sprintf(data_to_ettercap->type, "X11");
+      strcat(data_to_ettercap->type, "X11");
 
       for (i = 0; i < 16; i++)
          sprintf(data_to_ettercap->info + (i * 2) + 19, "%.2x", payload[i+20]);
 
-      strlcat(data_to_ettercap->info, "\n", sizeof(data_to_ettercap->info));
+      strcat(data_to_ettercap->info, "\n");
    }
 
    return 0;
 }
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
