@@ -17,15 +17,24 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_error.c,v 1.8 2002/07/20 09:41:25 alor Exp $
+    $Id: ec_error.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
-#include "include/ec_main.h"
-
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <stdarg.h>
 #include <errno.h>
 
-#include "include/ec_interface.h"
+#include "include/ec_main.h"
+
+#ifdef HAVE_NCURSES
+    #include "include/ec_interface.h"
+#endif
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos
 
@@ -37,16 +46,18 @@ void Error_critical_msg(char *file, char *function, int line, char *message);
 void Error_msg(char *message, ...)
 {
    va_list ap;
-   char errmsg[201];    // should be enough
+   char errmsg[150];    // should be enough
 
    va_start(ap, message);
-   vsnprintf(errmsg, 200, message, ap);
+   vsnprintf(errmsg, 150, message, ap);
    va_end(ap);
 
-   DEBUG_MSG("Error_msg -- %s", errmsg);
+#ifdef DEBUG
+      Debug_msg("Error_msg -- %s", errmsg);
+#endif
 
 #ifdef HAVE_NCURSES
-   if (Options.onscreen)
+   if (!Options.normal && !strcmp(program_argv0, PROGRAM))
       Interface_WExit(errmsg);
 #endif
 
@@ -57,15 +68,16 @@ void Error_msg(char *message, ...)
 
 void Error_critical_msg(char *file, char *function, int line, char *message)
 {
-   char err[201];
+   char err[200];
 
    snprintf(err, 200, "[%s:%s:%d] %s | ERRNO %d | %s", file, function, line, message, errno, strerror(errno));
-   err[200] = 0;
 
-   DEBUG_MSG("Error_msg -- %s",  err);
+#ifdef DEBUG
+      Debug_msg("Error_msg -- %s",  err);
+#endif
 
 #ifdef HAVE_NCURSES
-   if (Options.onscreen)
+   if (!Options.normal && !strcmp(program_argv0, PROGRAM))
       Interface_WExit(err);
 #endif
 
@@ -75,6 +87,3 @@ void Error_critical_msg(char *file, char *function, int line, char *message)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
