@@ -17,12 +17,19 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_filterdrop.c,v 1.6 2002/03/08 08:49:55 alor Exp $
+    $Id: ec_filterdrop.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <stdio.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <errno.h>
+#if defined (LINUX) && !defined (__USE_GNU)  // for memmem()
+   #define __USE_GNU
+#endif
+#include <string.h>
 #include <sys/types.h>
 #ifdef HAVE_CTYPE_H
    #include <ctype.h>
@@ -31,14 +38,17 @@
 #include "include/ec_inet_structures.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_logtofile.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 #define MOD_LOG      3
 #define MOD_DROP     2
 #define MOD_REPLACED 1
 #define MOD_ORIG     0
 
-pthread_mutex_t filter_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 DROP_FILTER *Filter_Array_Source;   // ec_main.h
 DROP_FILTER *Filter_Array_Dest;
 int Filter_Source;
@@ -59,7 +69,7 @@ void FilterDrop_DelFilter(DROP_FILTER *ptr, int i);
 void FilterDrop_SaveFilter(void);
 int FilterDrop_CheckMode(DROP_FILTER *ptr, short mode);
 int FilterDrop_MatchPorts(char p, char fp, int s, int fs, int d, int fd);
-int FilterDrop_ParseWildcard(char *dst, char *src, size_t siz);
+int FilterDrop_ParseWildcard(char *dst, char *src);
 int FilterDrop_Validation(DROP_FILTER *ptr);
 int Filter_DoValidate(DROP_FILTER *ptr, short i);
 
@@ -84,8 +94,6 @@ int FilterDrop_MakefilterTCP(u_char *buf_ip, int *buflen, short maxlen, DROP_FIL
    *mod = MOD_ORIG;  // no modification
    if (!filters) return 0;  // no filters... no action...
 
-   pthread_mutex_lock(&filter_mutex);
-
    do
    {
       if ( !FilterDrop_MatchPorts('T', filters[i].proto, ntohs(tcp->source), filters[i].source, ntohs(tcp->dest), filters[i].dest))
@@ -94,7 +102,7 @@ int FilterDrop_MakefilterTCP(u_char *buf_ip, int *buflen, short maxlen, DROP_FIL
       }
       else
       {
-         ret = FilterDrop_DoFilter(data, &datalen, maxlen*2, &filters[i], &next_filter, mod);
+         ret = FilterDrop_DoFilter(data, &datalen, maxlen, &filters[i], &next_filter, mod);
 
          if (*mod == MOD_LOG)    // log the packet
             LogToFile_FilteredData(buf_ip);
@@ -115,8 +123,6 @@ int FilterDrop_MakefilterTCP(u_char *buf_ip, int *buflen, short maxlen, DROP_FIL
 
    *buflen += delta;
 
-   pthread_mutex_unlock(&filter_mutex);
-
    return delta;
 }
 
@@ -139,8 +145,6 @@ int FilterDrop_MakefilterUDP(u_char *buf_ip, int *buflen, short maxlen, DROP_FIL
    *mod = MOD_ORIG;  // no modification
    if (!filters) return 0;  // no filters... no action...
 
-   pthread_mutex_lock(&filter_mutex);
-
    do
    {
       if ( !FilterDrop_MatchPorts('U', filters[i].proto, ntohs(udp->source), filters[i].source, ntohs(udp->dest), filters[i].dest))
@@ -169,9 +173,6 @@ int FilterDrop_MakefilterUDP(u_char *buf_ip, int *buflen, short maxlen, DROP_FIL
    udp->len = htons(ntohs(udp->len) + delta);
 
    *buflen += delta;
-
-   pthread_mutex_unlock(&filter_mutex);
-
    return delta;
 
 }
@@ -226,7 +227,9 @@ int FilterDrop_DoFilter(u_char *buf, int *buflen, short maxlen, DROP_FILTER *fil
          end = *buflen + delta;
          if (end > maxlen)
          {
-            DEBUG_MSG("FilterDrop_DoFilter -- maxlen reached !! %d/%d from %d [%d][%d]", end, maxlen, *buflen, found, (filter->rlen - filter->slen) );
+            #ifdef DEBUG
+               Debug_msg("FilterDrop_DoFilter -- maxlen reached !! %d/%d from %d [%d][%d]", end, maxlen, *buflen, found, (filter->rlen - filter->slen) );
+            #endif
             return 0;
          }
          ptr = (u_char *)((int)ptr + (filter->slen + filter->wildcard));   // move the ptr after the replaced string
@@ -309,12 +312,12 @@ int FilterDrop_CheckMode(DROP_FILTER *ptr, short mode)
 
 
 
-int FilterDrop_ParseWildcard(char *dst, char *src, size_t size)
+int FilterDrop_ParseWildcard(char *dst, char *src)
 {
    int j=0, k=0;
    char *p, *q;
 
-   strlcpy(dst, src, size);
+   strcpy(dst, src);
    p = dst;
 
    if ( (q = strstr(p, "*]")) )
@@ -362,7 +365,9 @@ int FilterDrop_Validation(DROP_FILTER *ptr)
 {
    int i, V=0;
 
-   DEBUG_MSG("FilterDrop_Validation");
+#ifdef DEBUG
+   Debug_msg("FilterDrop_Validation");
+#endif
 
    validation = 1;
 
@@ -393,7 +398,9 @@ int FilterDrop_Validation(DROP_FILTER *ptr)
       V = Filter_DoValidate(Filter_Array_Dest, 0);
    }
 
-   DEBUG_MSG("FilterDrop_Validation -- %d", V);
+#ifdef DEBUG
+   Debug_msg("FilterDrop_Validation -- %d", V);
+#endif
 
    return V;
 
@@ -406,9 +413,9 @@ void FilterDrop_AddFilter(DROP_FILTER *ptr)
 
    DROP_FILTER newfilter;
 
-   DEBUG_MSG("FilterDrop_AddFilter");
-
-   pthread_mutex_lock(&filter_mutex);
+#ifdef DEBUG
+   Debug_msg("FilterDrop_AddFilter");
+#endif
 
    memset(&newfilter, 0, sizeof(DROP_FILTER));
    newfilter.go_to = -1;
@@ -428,7 +435,6 @@ void FilterDrop_AddFilter(DROP_FILTER *ptr)
       memcpy(&Filter_Array_Dest[Filter_Dest-1], &newfilter, sizeof(DROP_FILTER));
    }
 
-   pthread_mutex_unlock(&filter_mutex);
 }
 
 
@@ -437,9 +443,9 @@ void FilterDrop_DelFilter(DROP_FILTER *ptr, int i)
 {
    int num_filter=0, j, k;
 
-   DEBUG_MSG("FilterDrop_DelFilter -- %d", i);
-
-   pthread_mutex_lock(&filter_mutex);
+#ifdef DEBUG
+   Debug_msg("FilterDrop_DelFilter -- %d", i);
+#endif
 
    if (ptr == Filter_Array_Source)
    {
@@ -472,8 +478,6 @@ void FilterDrop_DelFilter(DROP_FILTER *ptr, int i)
       }
    }
 
-   pthread_mutex_unlock(&filter_mutex);
-
 }
 
 
@@ -670,6 +674,3 @@ strend:
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
