@@ -17,15 +17,28 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_fingerprint.c,v 1.12 2002/09/06 08:57:31 alor Exp $
+    $Id: ec_fingerprint.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <fcntl.h>
+#include <sys/time.h>
+#include <time.h>
+#include <unistd.h>
+
 #include "include/ec_inet_structures.h"
 #include "include/ec_inet.h"
 #include "include/ec_inet_forge.h"
 #include "include/ec_decodedata.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 #ifdef HAVE_SYS_UTSNAME_H
    #include <sys/utsname.h>
@@ -152,100 +165,98 @@ void Fingerprint_parse_probes()
    TCP_header *tcp;
    short type;
    int len;
-   struct recv_packet recvpck;
+   char *probe_pck;
 
    TIME_DECLARE;
    TIME_START;
+   probe_pck=(char *)Inet_Forge_packet(MTU);
 
-   recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD);
-   recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
-   
-   do {
-           
-      len = Inet_GetRawPacket(sock, recvpck.aligned, MTU, &type);
-      TIME_FINISH;
-      
-      if (len>0 && type == PACKET_HOST)
+   do
+   {
+   len=Inet_GetRawPacket(sock,probe_pck,MTU,&type);
+   TIME_FINISH;
+
+   if (len>0 && type==PACKET_HOST)
+   {
+      eth=(ETH_header *)probe_pck;
+      if (ntohs(eth->type)==ETH_P_IP)
       {
-         eth = (ETH_header *)recvpck.aligned;
-         if (ntohs(eth->type)==ETH_P_IP)
+      ip=(IP_header *)(probe_pck+ETH_HEADER);
+      if (ip->proto==IPPROTO_TCP && ip->source_ip==IPD)
+      {
+         tcp=(TCP_header *)(probe_pck+ETH_HEADER+IP_HEADER);
+
+         if ( (u_short)ntohs(tcp->dest)>=PORTS && (u_short)ntohs(tcp->dest)<PORTS+NUM_TESTS)
          {
-            ip=(IP_header *)(eth + 1);
-            if (ip->proto==IPPROTO_TCP && ip->source_ip==IPD)
-            {
-               tcp=(TCP_header *)(ip + 1);
-            
-               if ( (u_short)ntohs(tcp->dest)>=PORTS && (u_short)ntohs(tcp->dest)<PORTS+NUM_TESTS)
-               {
-               char *p, *q, *arrive;
-               unsigned int num_test;
-            
-               num_test=ntohs(tcp->dest)-PORTS;
-            
-               // Replied
-               strcpy(&packet_stamp[num_test][0][0],"Y");
-                  // DF flag
-               if(ntohs(ip->frag_and_flags) & 0x4000)
-               {
-                  strcpy(&packet_stamp[num_test][1][0],"Y");
-               }
-               else strcpy(&packet_stamp[num_test][1][0], "N");
-               // Window Size
-               sprintf(&packet_stamp[num_test][2][0],"%hX",ntohs(tcp->window));
-               // ACK Sequence
-               if (ntohl(tcp->ack_seq) == SYN_SEQ+1)
-                  strcpy(&packet_stamp[num_test][3][0],"S++");
-               else if (ntohl(tcp->ack_seq) == SYN_SEQ)
-                  strcpy(&packet_stamp[num_test][3][0],"S");
-               else
-                  strcpy(&packet_stamp[num_test][3][0],"O");
-               // Flags
-               p=&packet_stamp[num_test][4][0];
-               *p='\0';
-               if (tcp->flags & 0x40)   *p++='B'; // BOGUS
-               if (tcp->flags & TH_URG) *p++='U';
-               if (tcp->flags & TH_ACK) *p++='A';
-               if (tcp->flags & TH_PSH) *p++='P';
-               if (tcp->flags & TH_RST) *p++='R';
-               if (tcp->flags & TH_SYN) *p++='S';
-               if (tcp->flags & TH_FIN) *p++='F';
-               *p++='\0';
-               // TCP Options
-               p=&packet_stamp[num_test][5][0];
-               q=((char *)tcp) + TCP_HEADER;
-               arrive = ((char *)tcp) + (tcp->doff*4);
-               while(q<arrive)
-               {
-                  int opcode;
-            
-                  opcode=*q++;
-                  if (!opcode) {
-                  *p++ = 'L';
-                  break;
-                  } else if (opcode == 1) {
-                    *p++ = 'N';
-                  } else if (opcode == 2) {
-                    *p++ = 'M';
-                    q++;
-                    if (ntohs(ptohs(q)) == MSS)
-                     *p++ = 'E';
-                    q+=2;
-                  } else if (opcode == 3) {
-                    *p++ = 'W';
-                    q+=2;
-                  } else if (opcode == 8) {
-                    *p++ = 'T';
-                    q+=9;
-                  }
-               }
-               *p++='\0';
-               }
+         char *p, *q, *arrive;
+         unsigned int num_test;
+
+         num_test=ntohs(tcp->dest)-PORTS;
+
+         // Replied
+         strcpy(&packet_stamp[num_test][0][0],"Y");
+            // DF flag
+         if(ntohs(ip->frag_and_flags) & 0x4000)
+         {
+            strcpy(&packet_stamp[num_test][1][0],"Y");
+         }
+         else strcpy(&packet_stamp[num_test][1][0], "N");
+         // Window Size
+         sprintf(&packet_stamp[num_test][2][0],"%hX",ntohs(tcp->window));
+         // ACK Sequence
+         if (ntohl(tcp->ack_seq) == SYN_SEQ+1)
+            strcpy(&packet_stamp[num_test][3][0],"S++");
+         else if (ntohl(tcp->ack_seq) == SYN_SEQ)
+            strcpy(&packet_stamp[num_test][3][0],"S");
+         else
+            strcpy(&packet_stamp[num_test][3][0],"O");
+         // Flags
+         p=&packet_stamp[num_test][4][0];
+         *p='\0';
+         if (tcp->flags & 0x40)   *p++='B'; // BOGUS
+         if (tcp->flags & TH_URG) *p++='U';
+         if (tcp->flags & TH_ACK) *p++='A';
+         if (tcp->flags & TH_PSH) *p++='P';
+         if (tcp->flags & TH_RST) *p++='R';
+         if (tcp->flags & TH_SYN) *p++='S';
+         if (tcp->flags & TH_FIN) *p++='F';
+         *p++='\0';
+         // TCP Options
+         p=&packet_stamp[num_test][5][0];
+         q=((char *)tcp) + TCP_HEADER;
+         arrive = ((char *)tcp) + (tcp->doff*4);
+         while(q<arrive)
+         {
+            int opcode;
+
+            opcode=*q++;
+            if (!opcode) {
+            *p++ = 'L';
+            break;
+            } else if (opcode == 1) {
+              *p++ = 'N';
+            } else if (opcode == 2) {
+              *p++ = 'M';
+              q++;
+              if (ntohs(ptohs(q)) == MSS)
+               *p++ = 'E';
+              q+=2;
+            } else if (opcode == 3) {
+              *p++ = 'W';
+              q+=2;
+            } else if (opcode == 8) {
+              *p++ = 'T';
+              q+=9;
             }
          }
+         *p++='\0';
+         }
       }
+      }
+   }
    }while(TIME_ELAPSED<1);
 
-   Inet_Forge_packet_destroy( recvpck.buf );
+   Inet_Forge_packet_destroy( probe_pck );
 }
 
 
@@ -276,9 +287,13 @@ char *Fingerprint_Make_Finger_print()
    char *condition;
    char *test;
    static char osname[500];
+   char database[sizeof(INSTALL_PATH)+22];
    static char res_list[2000];
 
-   file = fopen(DATA_PATH "/nmap-os-fingerprints", "r");
+   strcpy(database, INSTALL_PATH);
+   strcat(database, "nmap-os-fingerprints");
+
+   file = fopen(database, "r");
    if (!file)
       file = fopen("./share/nmap-os-fingerprints","r");
       if (!file)
@@ -290,7 +305,8 @@ char *Fingerprint_Make_Finger_print()
    Fingerprint_parse_probes();
    res_list[0]=0;
 
-   LOOP {
+   loop
+   {
       char *To_EOF;
 
       while( (To_EOF=fgets(osname,500,file)) && !strstr(osname,"Fingerprint") );
@@ -300,7 +316,7 @@ char *Fingerprint_Make_Finger_print()
       {
          char *deadend;
 
-         snprintf(num_test, sizeof(num_test), "T%1d", i);
+         sprintf(num_test,"T%1d",i);
          while(fgets(dummy,500,file) && !strstr(dummy,num_test));
 
          test=dummy+3;
@@ -320,7 +336,7 @@ char *Fingerprint_Make_Finger_print()
       }
       if (!no_match)
       {
-         strlcat(res_list, osname+sizeof("Fingerprint"), sizeof(res_list));
+         strcat(res_list,osname+sizeof("Fingerprint"));
       }
    }
    fclose(file);
@@ -352,33 +368,30 @@ void Fingerprint_Parse_packet(char *buffer)
 void Fingerprint_Simple_Scan()
 {
    int i, startP, finP;
-   struct recv_packet recvpck;
+   char *pck_to_send;
    TIME_DECLARE;
 
    startP = 10;  finP = 150;
 
-   recvpck.buf = Inet_Forge_packet( MTU + ALIGN_ETH_TO_WORD);
-   recvpck.aligned = recvpck.buf + ALIGN_ETH_TO_WORD;
-   
-   Inet_Forge_ethernet( recvpck.aligned, MACS, MACD, ETH_P_IP );
-   Inet_Forge_ip( recvpck.aligned + ETH_HEADER, IPS, IPD, TCP_HEADER, IP_ID++, 0, IPPROTO_TCP);
+   pck_to_send = (char *)Inet_Forge_packet(MTU);
+   Inet_Forge_ethernet( pck_to_send, MACS, MACD, ETH_P_IP );
+   Inet_Forge_ip( pck_to_send + ETH_HEADER, IPS, IPD, TCP_HEADER, IP_ID++, 0, IPPROTO_TCP);
 
    for (i=startP; i<=finP; i++)
    {
-      Inet_Forge_tcp( recvpck.aligned + ETH_HEADER + IP_HEADER, PORTS, i,  SYN_SEQ, 0, TH_SYN, 0, 0);
-      Inet_SendRawPacket(sock, recvpck.aligned, ETH_HEADER + IP_HEADER + TCP_HEADER );
+      Inet_Forge_tcp( pck_to_send + ETH_HEADER + IP_HEADER, PORTS, i,  SYN_SEQ, 0, TH_SYN, 0, 0);
+      Inet_SendRawPacket(sock, pck_to_send, ETH_HEADER + IP_HEADER + TCP_HEADER );
       if (!(i%5)) usleep(500);
    }
 
    TIME_START;
    do
    {
-      Inet_GetRawPacket(sock, recvpck.aligned, MTU, NULL);
-      Fingerprint_Parse_packet(recvpck.aligned);
+      Inet_GetRawPacket(sock, pck_to_send, MTU, NULL);
+      Fingerprint_Parse_packet(pck_to_send);
       TIME_FINISH;
    } while (TIME_ELAPSED <2 && (!open_port || !closed_port) );
-
-   Inet_Forge_packet_destroy( recvpck.buf );
+   Inet_Forge_packet_destroy( pck_to_send );
 }
 
 
@@ -387,7 +400,9 @@ char * Fingerprint_OS(char *IP)
 {
    char *res_name;
 
-   DEBUG_MSG("Fingerprint_OS -- [%s]", IP);
+#ifdef DEBUG
+   Debug_msg("Fingerprint_OS -- [%s]", IP);
+#endif
 
    if (!strcmp(IP, Inet_MyIPAddress()))
    {
@@ -396,9 +411,9 @@ char * Fingerprint_OS(char *IP)
 
          res_name = (char *)malloc(50);
          uname(&buf);
-         strlcpy(res_name, buf.sysname, 50);
-         strlcat(res_name, " ", 50);
-         strlcat(res_name, buf.release, 50);
+         strcpy(res_name, buf.sysname);
+         strcat(res_name, " ");
+         strcat(res_name, buf.release);
          return res_name;
       #else
          return "Yourself ;) try `uname -sr`";
@@ -407,7 +422,7 @@ char * Fingerprint_OS(char *IP)
 
    IPD = inet_addr(IP);
    sock = Inet_OpenRawSock(Options.netiface);
-   Inet_SetNonBlock(sock);
+   fcntl(sock, F_SETFL, O_NONBLOCK);
    Inet_GetIfaceInfo(Options.netiface, &MTU, MACS, (unsigned long *)&IPS, 0);
    memcpy(MACD, Inet_MacFromIP(inet_addr(IP)), 6);
    srand(time(0)); IP_ID = PORTS = SYN_SEQ = rand()%(0xFFFE)+1;
@@ -441,7 +456,9 @@ char * Fingerprint_MAC(char *MAC)
    char line[1024];
    mac_database *mac_index;
 
-   DEBUG_MSG("Fingerprint_MAC -- [%s]", MAC);
+#ifdef DEBUG
+   Debug_msg("Fingerprint_MAC -- [%s]", MAC);
+#endif
 
    if (!strcmp(MAC, "")) return "unknown";
 
@@ -453,7 +470,7 @@ char * Fingerprint_MAC(char *MAC)
 
       mac_list = mac_index;
 
-      fto = fopen(DATA_PATH "/mac-fingerprints", "r");
+      fto = fopen(INSTALL_PATH "mac-fingerprints", "r");
       if (!fto)
          fto = fopen("./share/mac-fingerprints","r");
          if (!fto)
@@ -470,8 +487,8 @@ char * Fingerprint_MAC(char *MAC)
          if ( (mac_index->next = ( struct mac_database *) calloc (1, sizeof(mac_database))) == NULL)
             ERROR_MSG("calloc()");
 
-         strlcpy(mac_index->mac, line, sizeof(mac_index->mac));
-         strlcpy(mac_index->vendor, line+10, 60);
+         strncpy(mac_index->mac, line, 8);
+         strncpy(mac_index->vendor, line+10, 60);
 
          mac_index = (mac_database *) mac_index->next;
       }
@@ -493,6 +510,3 @@ char * Fingerprint_MAC(char *MAC)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
