@@ -17,11 +17,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet_forge.c,v 1.7 2002/05/10 20:09:15 alor Exp $
+    $Id: ec_inet_forge.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
 #include "include/ec_inet_structures.h"
+#include "include/ec_error.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 // protos....
 
@@ -94,9 +99,9 @@ int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da,
    arp.pa_len        = 4;                      // protocol address length
    arp.opcode        = htons(op);              // opcode
    memcpy(arp.source_add, sa, 6);              // source mac
-   memcpy(&arp.source_ip, &sip, 4);             // source ip
+   *(u_long *)arp.source_ip = sip;             // source ip
    memcpy(arp.dest_add, da, 6);                // dest mac
-   memcpy(&arp.dest_ip, &dip, 4);               // dest ip
+   *(u_long *)arp.dest_ip = dip;               // dest ip
 
    memcpy(buf, &arp, sizeof(arp));
 
@@ -111,7 +116,7 @@ int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ide
 
    ip.h_len           = 5;                             // 20 byte header
    ip.version         = 4;                             // version 4
-   ip.tos             = 7;                             // IP tos     // we need 7 to enable GRE sniffing/injecting
+   ip.tos             = 0;                             // IP tos
    ip.t_len           = htons(IP_HEADER + len);        // total length
    ip.ident           = htons(ident);                  // IP ident
    ip.frag_and_flags  = htons(f_f);                    // fragmentation flags
@@ -268,7 +273,7 @@ u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long
 
    len = size;
 
-   while( size > 1 )
+   while(size >1)
    {
       cksum += *buffer++;
       size -= sizeof(u_short);
@@ -281,7 +286,7 @@ u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long
    cksum += htons(proto);
    cksum += htons(len);
 
-   if ( size ) cksum += *(u_char*)buffer;
+   if(size ) cksum += *(u_char*)buffer;
 
    cksum = (cksum >> 16) + (cksum & 0xffff);
    cksum += (cksum >> 16);
@@ -338,6 +343,3 @@ u_long Inet_Forge_CRC(u_char *buffer, int len)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
