@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_interface.c,v 1.4 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -23,16 +25,29 @@
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
 #include <string.h>
-#include <ncurses.h>
+#ifdef HAVE_NCURSES_H
+   #include <ncurses.h>
+#else
+   #include <curses.h>
+#endif
 #include <unistd.h>
+#include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <signal.h>
 
+#ifdef HAVE_TERMIOS_H
+   #include <termios.h>
+   struct termios original_term_info;
+#endif
+
 #include "include/ec_interface_sniff.h"
+#include "include/ec_interface_passive.h"
 #include "include/ec_interface_factory.h"
 #include "include/ec_inet.h"
 #include "include/ec_buffer.h"
 #include "include/ec_fingerprint.h"
+#include "include/ec_error.h"
 
 
 #ifdef PERMIT_PLUGINS
@@ -69,24 +84,25 @@
 // prototyping...
 
 void Interface_InitTitle(char *ip, char *mac, char *subnet);
-void Interface_InitScreen();
+void Interface_InitScreen(void);
 void Interface_CloseScreen(void);
 void Interface_Winch(void);
 void Interface_Redraw(void);
 void Interface_WExit(char *buffer);
 void Interface_Run(void);
-void Interface_GetWindowSize();
+void Interface_GetWindowSize(void);
 void Interface_PointItem(char direction, char hor_direction);
 void Interface_InitList(void);
 void Interface_KeyTab(void);
 void Interface_SelectItem(void);
 void Interface_Connect(void);
-char Interface_PopUp(char *question);
+char Interface_PopUp(char *question, ...);
 void Interface_RefreshList(void);
 void Interface_HelpWindow(char *help[]);
 void Interface_OldStyleSniff(short mode);
 void Interface_CheckForPoisoner(void);
 void Interface_FingerPrint(void);
+void Interface_EntryRemoval(void);
 
 // global variables for Interface...
 
@@ -274,6 +290,10 @@ void Interface_InitScreen(void)
    Debug_msg("Interface_InitScreen");
 #endif
 
+#ifdef HAVE_TERMIOS_H
+   tcgetattr(0, &original_term_info);
+#endif
+
    initscr();
    cbreak();
    noecho();
@@ -332,10 +352,6 @@ void Interface_InitList(void)
    int j;
    int Top_Pointer;
 
-#ifdef DEBUG
-   Debug_msg("Interface_InitList");
-#endif
-
    if (number_of_hosts_in_lan == 0) return;
 
    if (has_colors())
@@ -423,26 +439,26 @@ void Interface_PointItem(char direction, char hor_direction)
    else
       wattroff(main_window,A_REVERSE);
 
-	if (hor_direction)
-	{
-		switch(hor_direction)
-		{
-			case LEFT:
-					      if (Pointer == &Dest_Pointer)
-      					{
-			      			Pointer = &Source_Pointer;
-			      			*Pointer = Dest_Pointer;
-			      		}
-							break;
-			case RIGHT:
-					      if (Pointer == &Source_Pointer)
-      					{
-								Pointer = &Dest_Pointer;
-								*Pointer = Source_Pointer;
-							}
-							break;
-		}
-	}
+   if (hor_direction)
+   {
+      switch(hor_direction)
+      {
+         case LEFT:
+                     if (Pointer == &Dest_Pointer)
+                     {
+                        Pointer = &Source_Pointer;
+                        *Pointer = Dest_Pointer;
+                     }
+                     break;
+         case RIGHT:
+                     if (Pointer == &Source_Pointer)
+                     {
+                        Pointer = &Dest_Pointer;
+                        *Pointer = Source_Pointer;
+                     }
+                     break;
+      }
+   }
 
    if ( (Old_Pointer >= Base_Pointer) && (Old_Pointer <= Base_Pointer + Sel_Number -1))   // DON'T redraw previous selected item if it is out of view
    {
@@ -549,17 +565,23 @@ void Interface_SelectItem(void)        // TODO  implement color selection
 
 
 
-char Interface_PopUp(char *question)
+char Interface_PopUp(char *question, ...)
 {
    WINDOW *question_window;
    char answer;
+   char message[150];
+   va_list ap;
 
-   question_window = newwin(5, strlen(question) + 4,0,0);
-   mvwin(question_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(question)+4)/2 );
+   va_start(ap, question);
+   vsnprintf(message, 150, question, ap);
+   va_end(ap);
+
+   question_window = newwin(5, strlen(message) + 4,0,0);
+   mvwin(question_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(message)+4)/2 + 2 );
    wbkgdset(question_window, COLOR_PAIR(TITLE_COLOR));
    wattron(question_window, A_BOLD);
    box(question_window,ACS_VLINE,ACS_HLINE);
-   mvwprintw(question_window,  2, 2, question);
+   mvwprintw(question_window,  2, 2, message);
    wnoutrefresh(question_window);
    doupdate();
    answer = wgetch(question_window);
@@ -581,7 +603,7 @@ char Interface_PopUp(char *question)
 void Interface_RefreshList(void)
 {
    WINDOW *message_window;
-   char *mess = "updating the list...";
+   char mess[21] = "updating the list...";
 
 #ifdef DEBUG
    Debug_msg("Interface_RefreshList");
@@ -666,7 +688,7 @@ void Interface_Connect(void)
    {
       char answer;
 
-      if (Options.silent)  answer = 'y';
+      if (Options.silent || Options.arpsniff)  answer = 'y';
       else answer = Interface_PopUp("Do U really want to poison the ARP cache of the targets (y/n) ?");
 
       if ((answer == 'y') || (answer == 'Y'))
@@ -697,7 +719,7 @@ void Interface_Connect(void)
          wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
          wprintw(top_window, " doppleganger "); waddch(top_window, ACS_HLINE);
 
-         if ( (!strcmp(Host_Source.ip, "")) || (!strcmp(Host_Dest.ip, "")) )
+         if ( number_of_hosts_in_lan == 1 && ((!strcmp(Host_Source.ip, "")) || (!strcmp(Host_Dest.ip, ""))) )
             wprintw(top_window, " illithid (Public ARP) ");
          else
             wprintw(top_window, " illithid (ARP Based) ");
@@ -709,7 +731,7 @@ void Interface_Connect(void)
          wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
          wnoutrefresh(top_window);
 
-         if ( (!strcmp(Host_Source.ip, "")) || (!strcmp(Host_Dest.ip, "")) )
+         if ( number_of_hosts_in_lan == 1 && ((!strcmp(Host_Source.ip, "")) || (!strcmp(Host_Dest.ip, ""))) )
             Interface_Sniff_Run(PUBLICARP);   // with HALF duplex ARP poisoning
          else
             Interface_Sniff_Run(ARPBASED);   // with FULL duplex ARP poisoning
@@ -739,8 +761,6 @@ void Interface_OldStyleSniff(short mode)
       Interface_PopUp("SOURCE and DEST must be different !!");
    else if ( !strcmp(Host_Source.mac, Host_Dest.mac) && strcmp(Host_Source.mac, ""))
       Interface_PopUp("SOURCE and DEST must be different !!");
-   else if ( mode && ((!strcmp(Host_Source.mac, "")) || (!strcmp(Host_Dest.mac, ""))) )
-      Interface_PopUp("Before sniffing select source AND destination !!");
    else
    {
       wmove(top_window,0,0); whline(top_window, ' ', W_TOPX2);
@@ -749,7 +769,12 @@ void Interface_OldStyleSniff(short mode)
       wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
 
       if (mode)
-         wprintw(top_window, "%15s", Host_Source.mac);
+      {
+         if (strcmp(Host_Source.mac, ""))
+            wprintw(top_window, "%15s", Host_Source.mac);
+         else
+            wprintw(top_window, "%15s","  ANY  ");
+      }
       else
       {
          if (strcmp(Host_Source.ip, ""))
@@ -764,7 +789,12 @@ void Interface_OldStyleSniff(short mode)
       wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
 
       if (mode)
-         wprintw(top_window, "%15s", Host_Dest.mac);
+      {
+         if (strcmp(Host_Dest.mac, ""))
+            wprintw(top_window, "%15s", Host_Dest.mac);
+         else
+            wprintw(top_window, "%15s","  ANY  ");
+      }
       else
       {
          if (strcmp(Host_Dest.ip, ""))
@@ -816,12 +846,18 @@ void Interface_CheckForPoisoner(void)
    SniffingHost *SniffList;
    int i;
    short found = 0;
-   char *mess = "checking for poisoners...";
+   char mess[26] = "checking for poisoners...";
 
 #ifdef DEBUG
    Debug_msg("Interface_CheckForPoisoner");
 #endif
 
+   if (number_of_hosts_in_lan == 1)
+   {
+      Interface_PopUp("You cannot check the lan in silent mode");
+      return;
+   }
+
    message_window = newwin(5, strlen(mess) + 4,0,0);
    mvwin(message_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(mess) + 4)/2 );
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
@@ -838,18 +874,14 @@ void Interface_CheckForPoisoner(void)
       if (SniffList[i].mode == 0) break;
       if (SniffList[i].mode == 1)
       {
-         char mesg[100];
          found = 1;
-         sprintf(mesg, "%s is replying for %s", Host_In_LAN[SniffList[i].Host_Index1].ip, Host_In_LAN[SniffList[i].Host_Index2].ip);
-         Interface_PopUp(mesg);
+         Interface_PopUp("%s is replying for %s", Host_In_LAN[SniffList[i].Host_Index1].ip, Host_In_LAN[SniffList[i].Host_Index2].ip);
       }
 
       if (SniffList[i].mode == 2)
       {
-         char mesg[100];
          found = 1;
-         sprintf(mesg, "MAC of %s and %s are identical !",Host_In_LAN[SniffList[i].Host_Index1].ip,Host_In_LAN[SniffList[i].Host_Index2].ip);
-         Interface_PopUp(mesg);
+         Interface_PopUp("MAC of %s and %s are identical !",Host_In_LAN[SniffList[i].Host_Index1].ip,Host_In_LAN[SniffList[i].Host_Index2].ip);
       }
    }
 
@@ -873,16 +905,15 @@ void Interface_FingerPrint(void)
    int dimY = 7;
    int dimX = 20;
    int i=0, j=0, k=0;
-   char mac_finger[W_MAINX2];
-   char *mess = "Fingerprinting ...";
-   char long_os_fingers[1000];
-   char temp[100];
+   char mess[20] = "Fingerprinting ...";
+   char *mac_finger;
+   char *long_os_fingers;
+   char temp[W_MAINX2];
 
 #ifdef DEBUG
    Debug_msg("Interface_FingerPrint -- [%s] [%s]", Host_In_LAN[*Pointer].ip, Host_In_LAN[*Pointer].mac );
 #endif
 
-
    message_window = newwin(5, strlen(mess) + 4,0,0);
    mvwin(message_window, W_MAINY1+(W_MAINY2-W_MAINY1)/2-2, W_MAINX2/2 - (strlen(mess) + 4)/2 );
    wbkgdset(message_window, COLOR_PAIR(TITLE_COLOR));
@@ -892,15 +923,17 @@ void Interface_FingerPrint(void)
    wnoutrefresh(message_window);
    doupdate();
 
-   strncpy(long_os_fingers, Fingerprint_OS(Host_In_LAN[*Pointer].ip), 1000 );
-   strncpy(mac_finger, Fingerprint_MAC(Host_In_LAN[*Pointer].mac), W_MAINX2-4-18 );
+   long_os_fingers = strdup(Fingerprint_OS(Host_In_LAN[*Pointer].ip) );
+   mac_finger = strdup(Fingerprint_MAC(Host_In_LAN[*Pointer].mac));
+
+   if (strlen(mac_finger)> W_MAINX2-4-18 ) mac_finger[W_MAINX2-4-18] = 0;
 
    delwin(message_window);
    touchwin(main_window);
    wnoutrefresh(main_window);
    doupdate();
 
-   for(i=0; i < strlen(long_os_fingers); i++)
+   for(i=0; i <= strlen(long_os_fingers); i++)
    {
       if (long_os_fingers[i] == '\n')
       {
@@ -912,8 +945,7 @@ void Interface_FingerPrint(void)
    }
 
    dimX = (strlen(mac_finger)>dimX) ? strlen(mac_finger) : dimX;
-
-   dimX = (dimX + 24 < W_MAINX2-1) ? dimX + 24 : W_MAINX2-1;
+   dimX = (dimX + 27 < W_MAINX2-1) ? dimX + 27 : W_MAINX2-1;
 
    finger_win = newwin(dimY, dimX, 0, 0);
    mvwin(finger_win, W_BOTTOMY2/2 - dimY/2, (W_MAINX2/2 - dimX/2) + 1);
@@ -932,11 +964,14 @@ void Interface_FingerPrint(void)
       strncpy(temp, strtok(long_os_fingers, "\n"), W_MAINX2-4-18);
 
    mvwprintw(finger_win, 2, 20, "%s", temp);
-   for(i=0; i<j-1 ;i++)
+   for(i=0; i<j-1; i++)
    {
-      memset(temp, 0, 100);
+      memset(temp, 0, W_MAINX2);
       strncpy(temp, strtok(NULL, "\n"), W_MAINX2-4-18);
       mvwprintw(finger_win, 3+i, 20, "%s", temp);
+      #ifdef DEBUG
+         Debug_msg("Interface_FingerPrint -- [%s]", temp );
+      #endif
    }
    mvwprintw(finger_win, dimY-3, 20, "%s", mac_finger);
 
@@ -950,10 +985,39 @@ void Interface_FingerPrint(void)
    wnoutrefresh(main_window);
    doupdate();
 
+   free(long_os_fingers);
+   free(mac_finger);
+
 }
 
 
 
+void Interface_EntryRemoval(void)
+{
+#ifdef DEBUG
+   Debug_msg("Interface_EntryRemoval -- %d", *Pointer);
+#endif
+
+   if (*Pointer == 0 || number_of_hosts_in_lan == 1)
+   {
+      Interface_PopUp("You cannot remove yourself from the list !!");
+      return;
+   }
+
+   if (*Pointer < number_of_hosts_in_lan-1)
+      memmove(&Host_In_LAN[*Pointer], &Host_In_LAN[*Pointer+1], (number_of_hosts_in_lan-1-*Pointer)*sizeof(HOST));
+   else
+      *Pointer = *Pointer - 1;
+
+   number_of_hosts_in_lan--;
+
+   Host_In_LAN = realloc(Host_In_LAN, number_of_hosts_in_lan*sizeof(HOST));
+   if (Host_In_LAN == NULL)
+      Error_msg("ec_interface:%d realloc() | ERRNO : %d | %s", __LINE__, errno, strerror(errno));
+
+}
+
+
 
 void Interface_Run(void)
 {
@@ -963,26 +1027,26 @@ void Interface_Run(void)
    Debug_msg("Interface_Run");
 #endif
 
-   pipe_with_illithid = Buffer_Create(2.0e5);      // creates the buffers
-   pipe_with_illithid_data = Buffer_Create(1.0e6); // 1Mb in shared memory !!!
-
-   if (Options.silent)
-   {
-      if (Options.arpsniff)
-         Interface_Connect();
-      if (Options.macsniff)
-         Interface_OldStyleSniff(1);
-      if (Options.sniff)
-         Interface_OldStyleSniff(0);
-      #ifdef PERMIT_PLUGINS
-         if (Options.plugin)
-            Interface_Plugins_Run();
-      #endif
-   }
+   if (Options.arpsniff)
+      Interface_Connect();
+   if (Options.macsniff)
+      Interface_OldStyleSniff(1);
+   if (Options.sniff)
+      Interface_OldStyleSniff(0);
+   #ifdef PERMIT_PLUGINS
+      if (Options.plugin)
+         Interface_Plugins_Run();
+   #endif
 
    if (Options.silent && !(Options.sniff || Options.macsniff || Options.arpsniff))
       Options.silent = 0;
 
+   if (Options.passive)
+   {
+      Interface_Passive_Run();
+      Interface_InitList();
+   }
+
    loop
    {
       KeyPress = wgetch(main_window);
@@ -1047,6 +1111,12 @@ void Interface_Run(void)
                   Interface_FingerPrint();
                   break;
 
+         case 'O':
+         case 'o':
+                  Interface_Passive_Run();
+                  Interface_InitList();
+                  break;
+
          case ' ':
                   werase(top_window);
                   wnoutrefresh(top_window);
@@ -1061,6 +1131,12 @@ void Interface_Run(void)
                   Interface_Redraw();
                   break;
 
+         case 'D':
+         case 'd':
+                  Interface_EntryRemoval();
+                  Interface_InitList();
+                  break;
+
 #ifdef PERMIT_PLUGINS
          case 'P':
          case 'p':
@@ -1075,18 +1151,20 @@ void Interface_Run(void)
                      static char *help[] = {
                         "[qQ][F10] - quit",
                         "[return]  - select the IP",
-                        "[space]   - wipe/deselect the IPs",
+                        "[space]   - deselect the IPs",
                         "[tab]     - switch between source and dest",
                         "[aA]      - ARP posoning based sniffing ",
                         "             . for sniffing on switched LAN",
                         "             . for man-in-the-middle technique",
                         "[sS]      - IP based sniffing",
                         "[mM]      - MAC based sniffing",
+                        "[dD]      - delete an entry from the list",
                         "[xX]      - Packet Forge",
 #ifdef PERMIT_PLUGINS
                         "[pP]      - run a plugin",
 #endif
                         "[fF]      - OS fingerprint",
+                        "[oO]      - passive host identification",
                         "[cC]      - check for other poisoner...",
                         "[rR]      - refresh the list",
                         "[hH]      - this help screen",
@@ -1146,9 +1224,15 @@ void Interface_CloseScreen(void)
    Debug_msg("Interface_CloseScreen");
 #endif
 
+   curs_set(1);
    clear();
    refresh();
    endwin();
+
+#ifdef HAVE_TERMIOS_H
+   tcsetattr(0, TCSANOW, &original_term_info);
+#endif
+
 }
 
 
@@ -1162,8 +1246,8 @@ void Interface_WExit(char *buffer)
 
    if (ScreenMode) Interface_CloseScreen();
 
-	printf("\033[01m\033[1m%s %s brought from the dark side of the net by ALoR and NaGA...\033[0m\n\n", PROGRAM, VERSION);
-	printf("\033[01m\033[1mmay the packets be with you...\033[0m\n\n");
+   printf("\033[01m\033[1m%s %s brought from the dark side of the net by ALoR and NaGA...\033[0m\n\n", PROGRAM, VERSION);
+   printf("\033[01m\033[1mmay the packets be with you...\033[0m\n\n");
    printf("\n");
    printf(buffer);
    printf("\n\n");
