@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_interface_factory.c,v 1.2 2001/08/30 10:05:19 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -23,7 +25,11 @@
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
 #include <string.h>
-#include <ncurses.h>
+#ifdef HAVE_NCURSES_H
+   #include <ncurses.h>
+#else
+   #include <curses.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 #ifdef HAVE_CTYPE_H
@@ -60,7 +66,7 @@
 
 // protos...
 
-int Interface_Factory_Run();
+int Interface_Factory_Run(void);
 int Interface_Factory_ETH(u_char *buf);
 int Interface_Factory_IP(u_char *buf, short *proto);
 int Interface_Factory_TCP(u_char *buf);
@@ -292,20 +298,20 @@ int Interface_Factory_TCP(u_char *buf)
    tcp_form[n++] = make_field(2, 14, 1, 5, FALSE);
    tcp_form[n++] = make_label(2, 22, "Dest port   :");
    tcp_form[n++] = make_field(2, 36, 1, 5, FALSE);
-   tcp_form[n++] = make_label(4, 0, "Seq : 0x");
-   tcp_form[n++] = make_field(4, 8, 1, 8, FALSE);
-   tcp_form[n++] = make_label(4, 22, "Ack : 0x");
-   tcp_form[n++] = make_field(4, 30, 1, 8, FALSE);
-   tcp_form[n++] = make_label(6, 0, "Flags    : ");
-   tcp_form[n++] = make_field(6, 11, 1, 5, FALSE);
-   tcp_form[n++] = make_label(6, 20, "(S/A/P/R/F)");
-   tcp_form[n++] = make_label(7, 0, "CheckSum : 0x");
-   tcp_form[n++] = make_field(7, 13, 1, 4, FALSE);
-   tcp_form[n++] = make_label(7, 20, "(leave blank for auto)");
-   tcp_form[n++] = make_label(9, 0, "Payload :");
-   tcp_form[n++] = make_field(10, 0, 8, 50, FALSE);
-   tcp_form[n++] = make_label(19, 0, "Or load Payload from file:");
-   tcp_form[n++] = make_field(20, 0, 1, 50, FALSE);
+   tcp_form[n++] = make_label(3, 0, "Sequence : 0x");
+   tcp_form[n++] = make_field(3, 13, 1, 8, FALSE);
+   tcp_form[n++] = make_label(3, 22, "Acknowledge : 0x");
+   tcp_form[n++] = make_field(3, 38, 1, 8, FALSE);
+   tcp_form[n++] = make_label(4, 0, "Flags    : ");
+   tcp_form[n++] = make_field(4, 11, 1, 5, FALSE);
+   tcp_form[n++] = make_label(4, 20, "(S/A/P/R/F)");
+   tcp_form[n++] = make_label(5, 0, "CheckSum : 0x");
+   tcp_form[n++] = make_field(5, 13, 1, 4, FALSE);
+   tcp_form[n++] = make_label(5, 20, "(leave blank for auto)");
+   tcp_form[n++] = make_label(7, 0, "Payload :");
+   tcp_form[n++] = make_field(8, 0, 8, 50, FALSE);
+   tcp_form[n++] = make_label(17, 0, "Or load Payload from file:");
+   tcp_form[n++] = make_field(18, 0, 1, 50, FALSE);
    tcp_form[n++] = (FIELD *)0;
 
 
@@ -1329,7 +1335,6 @@ int Interface_Factory_Run(void)
 
    u_char *buf;
    u_char *forged_pck;
-   char sent[50];
    int ret = 0;
    int MTU, sock;
    short proto;
@@ -1376,16 +1381,15 @@ int Interface_Factory_Run(void)
    Debug_msg("Interface_Factory_Run -- %d byte(s) sent", forged_pck-buf);
 #endif
 
-   sprintf(sent, "%d byte(s) forged and sent on the wire", forged_pck-buf);
    Interface_Redraw();
-   Interface_PopUp(sent);
+   Interface_PopUp("%d byte(s) forged and sent on the wire", forged_pck-buf);
 
 cancelled:
 #ifdef DEBUG
    Debug_msg("Interface_Factory_Run -- freeing buffer");
 #endif
    Inet_Forge_packet_destroy( buf );
-   close(sock);
+   Inet_CloseRawSock(sock);
 
    return 0;
 }
