@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_interface_form.c,v 1.1.1.1 2001/08/27 19:26:16 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -24,7 +26,11 @@
 #ifdef HAVE_FORM
 
 #include <string.h>
-#include <ncurses.h>
+#ifdef HAVE_NCURSES_H
+	#include <ncurses.h>
+#else
+	#include <curses.h>
+#endif
 #include <unistd.h>
 #ifdef HAVE_CTYPE_H
    #include <ctype.h>
@@ -51,7 +57,7 @@
 
 // protos...
 
-FIELD *make_label(int frow, int fcol, NCURSES_CONST char *label);
+FIELD *make_label(int frow, int fcol, char *label);
 FIELD *make_field(int frow, int fcol, int rows, int cols, bool secure);
 void display_form(FORM *f);
 void erase_form(FORM *f);
@@ -72,7 +78,7 @@ extern int W_BOTTOMY2;
 
 
 
-FIELD *make_label(int frow, int fcol, NCURSES_CONST char *label)
+FIELD *make_label(int frow, int fcol, char *label)
 {
    FIELD *f = new_field(1, strlen(label), frow, fcol, 0, 0);
 
@@ -103,7 +109,7 @@ void display_form(FORM *f)
 
    scale_form(f, &rows, &cols);
 
-   if ((w = newwin(rows+6, cols+4, W_BOTTOMY2/2 - rows/2 - 4, W_MAINX2/2 - cols/2)) != (WINDOW *)0)
+   if ((w = newwin(rows+6, cols+4, W_BOTTOMY2/2 - rows/2 - 3, W_MAINX2/2 - cols/2)) != (WINDOW *)0)
    {
       wbkgdset(w, COLOR_PAIR(HELP_COLOR));
       set_form_win(f, w);
