@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_interface_sniff.c,v 1.2 2001/08/30 10:05:19 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -23,7 +25,11 @@
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
 #include <string.h>
-#include <ncurses.h>
+#ifdef HAVE_NCURSES_H
+   #include <ncurses.h>
+#else
+   #include <curses.h>
+#endif
 #include <unistd.h>
 #include <signal.h>
 #include <sys/time.h>
@@ -31,6 +37,9 @@
 #include "include/ec_interface.h"
 #include "include/ec_interface_sniff_data.h"
 #include "include/ec_interface_factory.h"
+#include "include/ec_interface_inject.h"
+#include "include/ec_interface_passive.h"
+#include "include/ec_filterdrop.h"
 #include "include/ec_decodedata.h"
 #include "include/ec_illithid.h"
 #include "include/ec_doppleganger.h"
@@ -39,6 +48,7 @@
 #include "include/ec_inet_forge.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_buffer.h"
+#include "include/ec_error.h"
 #include "include/ec_logtofile.h"
 
 #ifdef DEBUG
@@ -84,6 +94,7 @@ int Conn_Base_Pointer = 0;
 int Conn_Pointer = 0;
 extern int Sel_Number;
 
+extern short inject;
 
 //---------------------------
 
@@ -92,11 +103,8 @@ void Interface_Sniff_InitList(void)
 {
    int j;
    int Conn_Top_Pointer;
-   char info[100];
+   char info[250];
 
-//#ifdef DEBUG
-// Debug_msg("Interface_Sniff_InitList");
-//#endif
 
    if (has_colors())
       wbkgdset(main_window, COLOR_PAIR(MAIN_COLOR));
@@ -145,9 +153,9 @@ void Interface_Sniff_InitList(void)
    mvwprintw(bottom_window, 1, 1, " %s", Conn_Between_Hosts[Conn_Pointer].pass);
    snprintf(info, W_MAINX2 - 28, "%s", Conn_Between_Hosts[Conn_Pointer].info);
    if (strlen(info)+1 == W_MAINX2 - 28)
-      mvwprintw(bottom_window, 1, 25, " %s->", info);
+      mvwprintw(bottom_window, 1, 20, " %s->", info);
    else
-      mvwprintw(bottom_window, 1, 25, " %s", info);
+      mvwprintw(bottom_window, 1, 20, " %s", info);
 
    wnoutrefresh(bottom_window);
    wnoutrefresh(main_window);
@@ -160,7 +168,7 @@ void Interface_Sniff_PointItem(char direction)
 {
 
    int Old_Conn_Pointer;
-   char info[100];
+   char info[250];
 
    if (number_of_connections == 0) return;   // no connection... no action... ;)
 
@@ -227,9 +235,9 @@ void Interface_Sniff_PointItem(char direction)
    mvwprintw(bottom_window, 1, 1, " %s", Conn_Between_Hosts[Conn_Pointer].pass);
    snprintf(info, W_MAINX2 - 28, "%s", Conn_Between_Hosts[Conn_Pointer].info);
    if (strlen(info)+1 == W_MAINX2 - 28)
-      mvwprintw(bottom_window, 1, 25, " %s->", info);
+      mvwprintw(bottom_window, 1, 20, " %s->", info);
    else
-      mvwprintw(bottom_window, 1, 25, " %s", info);
+      mvwprintw(bottom_window, 1, 20, " %s", info);
 
    wnoutrefresh(bottom_window);
    wnoutrefresh(main_window);
@@ -271,12 +279,7 @@ void Interface_Sniff_RefreshList(void)
 
 void Interface_Sniff_KillConn(void)
 {
-
-   u_char *buf;
-   char *MACD, MACS[6];
-   u_long IPS, IPD;
-   int sock;
-//   char answer;
+   KILL_DATA kill_data;
 
    if (Conn_Between_Hosts[Conn_Pointer].proto == 'U')
    {
@@ -284,65 +287,21 @@ void Interface_Sniff_KillConn(void)
       return;
    }
 
-
-//   answer = Interface_PopUp("Do U really want to kill this connection (y/n)?");
-//   if ((answer != 'y') && (answer != 'Y'))
-//      return;
-
-
 #ifdef DEBUG
-   Debug_msg("Interface_Sniff_KillConn -- %s:%d -> %s:%d  %lu %lu", Conn_Between_Hosts[Conn_Pointer].source_ip,
-                                                                    Conn_Between_Hosts[Conn_Pointer].source_port,
-                                                                    Conn_Between_Hosts[Conn_Pointer].dest_ip,
-                                                                    Conn_Between_Hosts[Conn_Pointer].dest_port,
-                                                                    Conn_Between_Hosts[Conn_Pointer].source_seq,
-                                                                    Conn_Between_Hosts[Conn_Pointer].dest_seq );
+   Debug_msg("Interface_Sniff_KillConn -- %s:%d -> %s:%d ", Conn_Between_Hosts[Conn_Pointer].source_ip,
+                                                            Conn_Between_Hosts[Conn_Pointer].source_port,
+                                                            Conn_Between_Hosts[Conn_Pointer].dest_ip,
+                                                            Conn_Between_Hosts[Conn_Pointer].dest_port );
 #endif
 
-   Inet_GetMACfromString(Inet_MyMACAddress(), MACS);
-   IPS = inet_addr(Conn_Between_Hosts[Conn_Pointer].source_ip);
-   IPD = inet_addr(Conn_Between_Hosts[Conn_Pointer].dest_ip);
-
-   MACD = Inet_MacFromIP(IPD);
-
-   buf = Inet_Forge_packet( ETH_HEADER + IP_HEADER + TCP_HEADER );
-
-   Inet_Forge_ethernet( buf, MACS, MACD, ETH_P_IP );
-
-   Inet_Forge_ip( buf + ETH_HEADER, IPS, IPD, TCP_HEADER, 0xe77e, 0, IPPROTO_TCP );    // to source
-
-   Inet_Forge_tcp( buf + ETH_HEADER + IP_HEADER, Conn_Between_Hosts[Conn_Pointer].source_port,
-                                                 Conn_Between_Hosts[Conn_Pointer].dest_port,
-                                                 Conn_Between_Hosts[Conn_Pointer].source_seq,
-                                                 0,
-                                                 TH_RST,
-                                                 0, 0 );
-
-   sock = Inet_OpenRawSock(Options.netiface);
-
-   MACD = Inet_MacFromIP(IPS);
-
-   Inet_SendRawPacket(sock, buf, ETH_HEADER + IP_HEADER + TCP_HEADER );
-
-   Inet_Forge_ethernet( buf, MACS, MACD, ETH_P_IP );
-
-   Inet_Forge_ip( buf + ETH_HEADER, IPD, IPS, TCP_HEADER, 0xe77e, 0, IPPROTO_TCP );    // to dest
-
-   Inet_Forge_tcp( buf + ETH_HEADER + IP_HEADER, Conn_Between_Hosts[Conn_Pointer].dest_port,
-                                                 Conn_Between_Hosts[Conn_Pointer].source_port,
-                                                 Conn_Between_Hosts[Conn_Pointer].dest_seq,
-                                                 0,
-                                                 TH_RST,
-                                                 0, 0 );
-
-   Inet_SendRawPacket(sock, buf, ETH_HEADER + IP_HEADER + TCP_HEADER );
-
-   Inet_Forge_packet_destroy( buf );
+   kill_data.source_ip = inet_addr(Conn_Between_Hosts[Conn_Pointer].source_ip);
+   kill_data.dest_ip = inet_addr(Conn_Between_Hosts[Conn_Pointer].dest_ip);
+   kill_data.source_port = htons(Conn_Between_Hosts[Conn_Pointer].source_port);
+   kill_data.dest_port = htons(Conn_Between_Hosts[Conn_Pointer].dest_port);
+   write(pipe_kill[1], &kill_data, sizeof(KILL_DATA));
 
    strcpy(Conn_Between_Hosts[Conn_Pointer].status, "KILLED");
 
-   close(sock);
-
 }
 
 
@@ -378,7 +337,7 @@ void Interface_Sniff_ActiveDissecor(short mode)
 
 void Interface_Sniff_Run(short mode)
 {
-   int KeyPress;
+   int KeyPress, i;
    int Illithid_pid = 0, Dopple_pid = 0;
    #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
       int Grell_pid = 0;
@@ -391,6 +350,8 @@ void Interface_Sniff_Run(short mode)
    Debug_msg("Interface_Sniff_Run -- mode %d", mode);
 #endif
 
+   if (pipe_with_illithid == -1) pipe_with_illithid = Buffer_Create(5.0e5);      // creates the buffers
+   if (pipe_with_illithid_data == -1) pipe_with_illithid_data = Buffer_Create(1.0e6); // 1Mb in shared memory !!!
 
    wbkgdset(main_window, COLOR_PAIR(MAIN_COLOR));
    werase(main_window);
@@ -405,6 +366,7 @@ void Interface_Sniff_Run(short mode)
       active_dissector = 0;
 
    Interface_Sniff_ActiveDissecor(mode);
+   Interface_Inject_FilterTopStatus();
 
    number_of_connections = 0;
    Interface_Sniff_RefreshList();
@@ -416,6 +378,19 @@ void Interface_Sniff_Run(short mode)
    {
       case ARPBASED:
       case PUBLICARP:
+                     Inet_DisableForwarding();
+
+                     for(i=0; i<number_of_hosts_in_lan; i++)
+                     {
+                        if ( !strcmp(Host_Source.ip, Host_In_LAN[i].ip) )
+                           strcpy(Host_Source.mac, Host_In_LAN[i].mac);
+                        if ( !strcmp(Host_Dest.ip, Host_In_LAN[i].ip) )
+                           strcpy(Host_Dest.mac, Host_In_LAN[i].mac);
+                     }
+
+                     if ( !strcmp(Host_Source.mac, "") && !strcmp(Host_Dest.mac, "") )
+                        Error_msg("Doppelganger needs at least one valid mac address !!\n\n(the ip was not found in the list)");
+
                      Dopple_pid = Doppleganger_Run(Options.netiface, Host_Source.ip, Host_Dest.ip, Host_Source.mac, Host_Dest.mac);
 
                      if ( mode == ARPBASED )
@@ -436,6 +411,7 @@ void Interface_Sniff_Run(short mode)
                      break;
    }
 
+   if (Options.passive) Interface_Passive_Run();
 
    memset(&TimeOut, 0, sizeof(TimeOut));  //  timeout = 0
    FD_ZERO(&msk_fd);
@@ -447,7 +423,6 @@ void Interface_Sniff_Run(short mode)
 
       select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);
 
-      //memset(&data_from_illithid, 0, sizeof(CONNECTION));
       datalen = Buffer_Get(pipe_with_illithid, &data_from_illithid, sizeof(CONNECTION));
 
       if (datalen > 0)
@@ -485,13 +460,11 @@ void Interface_Sniff_Run(short mode)
                      if (Conn_Between_Hosts[Conn_Pointer].source_ip == NULL) break;   // no connections
                      wmove(bottom_window, 0, 0);
                      mvwprintw(top_window, 2, 52, "                    ");
-                     if ( mode == PUBLICARP )
-                         kill(Dopple_pid,SIGTERM);    // kill publicARP doppleganger...
 
                      kill(Illithid_pid, SIGUSR2);
                      #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
                         if (Grell_pid)
-                           if ( kill(-1*Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
+                           if ( kill(-Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
                               kill(Grell_pid, SIGUSR2);
                      #endif
                      Interface_Sniff_Data_Run(Conn_Between_Hosts[Conn_Pointer].source_ip,
@@ -507,13 +480,25 @@ void Interface_Sniff_Run(short mode)
                      kill(Illithid_pid, SIGUSR2);
                      #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
                         if (Grell_pid)
-                           if ( kill(-1*Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
+                           if ( kill(-Grell_pid, SIGUSR2) == -1)   // send the signal to the Grell's group (-PID)
                               kill(Grell_pid, SIGUSR2);
                      #endif
-                     if ( mode == PUBLICARP )
-                         Dopple_pid = Doppleganger_Run(Options.netiface, Host_Source.ip, Host_Dest.ip, Host_Source.mac, Host_Dest.mac);
 
                      Interface_Sniff_ActiveDissecor(mode);
+                     if (filter_on_source && FilterDrop_CheckMode(Filter_Array_Source, mode))
+                     {
+                        Interface_PopUp("The Source filter chain can be used only in ARPBASED mode !!");
+                        filter_on_source = 0;
+                        write(pipe_filter[1], "S", 1);
+                        Interface_Inject_FilterStatus();
+                     }
+                     if (filter_on_dest && FilterDrop_CheckMode(Filter_Array_Dest, mode))
+                     {
+                        Interface_PopUp("The Dest filter chain can be used only in ARPBASED mode !!");
+                        filter_on_dest = 0;
+                        write(pipe_filter[1], "D", 1);
+                        Interface_Inject_FilterStatus();
+                     }
                      Interface_Sniff_InitList();
                      break;
 
@@ -529,14 +514,12 @@ void Interface_Sniff_Run(short mode)
                      {
                         char source[40];
                         char dest[40];
-                        char mesg[80];
                         if (Conn_Between_Hosts[Conn_Pointer].source_ip == NULL) break;   // no connections
                         snprintf(source, 40, " %s (%s) <-->", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].source_ip),
                                                               Conn_Between_Hosts[Conn_Pointer].source_ip);
                         snprintf(dest, 40, " %s (%s)", Inet_HostName(Conn_Between_Hosts[Conn_Pointer].dest_ip),
                                                        Conn_Between_Hosts[Conn_Pointer].dest_ip);
-                        snprintf(mesg, 80, "%s%s", source, dest);
-                        Interface_PopUp(mesg);
+                        Interface_PopUp("%s%s", source, dest);
                      }
                      break;
 
@@ -554,13 +537,7 @@ void Interface_Sniff_Run(short mode)
 
             case 'L':
             case 'l':
-                     {
-                        char *file;
-                        char mesg[80];
-                        file = LogToFile_DumpPass();
-                        snprintf(mesg, 80, "Passwords dumped to %s", file);
-                        Interface_PopUp(mesg);
-                     }
+                     Interface_PopUp("Passwords dumped to %s", LogToFile_DumpPass());
                      break;
 
 #ifdef PERMIT_PLUGINS
@@ -584,6 +561,18 @@ void Interface_Sniff_Run(short mode)
                      }
                      break;
 
+            case 'F':
+            case 'f':
+                     Interface_Inject_SetFilter(mode);
+                     Interface_Redraw();
+                     break;
+
+            case 'O':
+            case 'o':
+                     Interface_Passive_Run();
+                     Interface_Sniff_InitList();
+                     break;
+
             case KEY_CTRL_L:  // CTRL+L refresh the screen
                      Interface_Redraw();
                      break;
@@ -596,11 +585,13 @@ void Interface_Sniff_Run(short mode)
                            "[return]  - sniff the selected connection",
                            "[xX]      - Packet Forge",
                            "[aA]      - enable/disable ACTIVE password collectors",
+                           "[fF]      - set/edit filters chains",
                            "[lL]      - log all collected passwords to a file",
                            "[kK]      - kill the connection (be careful !)",
 #ifdef PERMIT_PLUGINS
                            "[pP]      - run a plugin",
 #endif
+                           "[oO]      - passive scanning of the LAN",
                            "[dD]      - resolve ip via DNS",
                            "[rR]      - refresh the list",
                            NULL};
@@ -611,6 +602,14 @@ void Interface_Sniff_Run(short mode)
 
             case 'Q':
             case 'q':
+                     if (inject == 1)
+                     {
+                        char answer;
+                        answer = Interface_PopUp("If U exit now some connections may be ReSeTted!! continue (y/n)?");
+                        Interface_Redraw();
+                        if ((answer != 'y') && (answer != 'Y'))
+                           break;
+                     }
                      kill(Illithid_pid, SIGTERM);
                      if (Dopple_pid) kill(Dopple_pid, SIGTERM);
                      #if defined (HAVE_OPENSSL) && defined (PERMIT_HTTPS)
@@ -623,6 +622,7 @@ void Interface_Sniff_Run(short mode)
                      werase(bottom_window);
                      wnoutrefresh(bottom_window);
                      doupdate();
+                     number_of_connections = -1;
                      return;
                      break;
 
