@@ -16,6 +16,8 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+    $Id: ec_interface_sniff_data.c,v 1.2 2001/09/06 20:07:42 alor Exp $
 */
 
 #include "include/ec_main.h"
@@ -23,7 +25,11 @@
 #ifdef HAVE_NCURSES  // don't compile if ncurses interface is not supported
 
 #include <string.h>
-#include <ncurses.h>
+#ifdef HAVE_NCURSES_H
+   #include <ncurses.h>
+#else
+   #include <curses.h>
+#endif
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -34,16 +40,16 @@
 #include "include/ec_interface_inject.h"
 #include "include/ec_decodedata.h"
 #include "include/ec_dissector.h"
-#include "include/ec_doppleganger.h"
 #include "include/ec_illithid.h"
 #include "include/ec_buffer.h"
 #include "include/ec_error.h"
+#include "include/ec_inet_structures.h"
 
 #ifdef DEBUG
    #include "include/ec_debug.h"
 #endif
 
-#define PAD_BUFFER 500        // buffered lines for sniffing
+#define PAD_BUFFER 1000        // buffered lines for sniffing
 
 
 #define BOTTOM_COLOR 1        // color schemes
@@ -74,9 +80,8 @@ void Interface_Sniff_Data_StopCont(void);
 void Interface_Sniff_Data_View(short mode);
 void Interface_Sniff_Data_Scroll(short direction);
 void Interface_Sniff_Data_DrawScroller(void);
-void Interface_Sniff_Data_Inject(char proto, char *app, char *ipsource, char *ipdest);
-void Interface_Sniff_Data_Filter(void);
-void Interface_Sniff_Data_ActiveFilter(short mode);
+void Interface_Sniff_Data_Inject(char proto, char *app, char *ipsource, short portsource, char *ipdest, short portdest);
+void Interface_Sniff_Data_Kill(char proto, char *ipsource, short portsource, char *ipdest, short portdest);
 
 // global variables
 
@@ -92,7 +97,7 @@ extern int Conn_Pointer;
 short log_source, log_dest;
 short view;
 short stop = 0;
-short inject = -1;
+short inject = 0;
 
 int inject_tod=0, inject_tos=0;
 
@@ -239,7 +244,7 @@ void Interface_Sniff_Data_LogToFile(char proto, char *ipsource, int portsource,
       {
          file_source = fopen(filename, "a");
          if (file_source == NULL)
-            Error_msg("ec_interface_sniff_data:%d fopen(%s) | ERRNO : %d | %s", filename, __LINE__, errno, strerror(errno));
+            ERROR_MSG("fopen()");
          log_source = 1;
          mvwprintw(data_source_win, W_MAINY2 - W_MAINY1 - 4, W_MAINX2 / 2 - 10, "LOGGED");
       }
@@ -247,7 +252,7 @@ void Interface_Sniff_Data_LogToFile(char proto, char *ipsource, int portsource,
       {
          file_dest = fopen(filename, "a");
          if (file_dest == NULL)
-            Error_msg("ec_interface_sniff_data:%d fopen(%s) | ERRNO : %d | %s", filename, __LINE__, errno, strerror(errno));
+            ERROR_MSG("fopen()");
          log_dest = 1;
          mvwprintw(data_dest_win, W_MAINY2 - W_MAINY1 - 4, W_MAINX2 / 2 - 10, "LOGGED");
       }
@@ -422,7 +427,7 @@ void Interface_Sniff_Data_SniffData(char *source, char *dest, int portsource, in
                            break;
          }
    }
-   if (inject == -1) inject = 0; // ok we have taken at least one packet
+
    doupdate();
 }
 
@@ -483,116 +488,62 @@ void Interface_Sniff_Data_Scroll(short direction)
 
 
 
-void Interface_Sniff_Data_Inject(char proto, char *app, char *ipsource, char *ipdest)
+void Interface_Sniff_Data_Inject(char proto, char *app, char *ipsource, short portsource, char *ipdest, short portdest)
 {
 
-   char inject_data[MAX_INJECT];
+   INJECTED_DATA inject_data;
    int data_len;
-   char answer;
 
 #ifdef DEBUG
    Debug_msg("Interface_Sniff_Data_Inject");
 #endif
 
-   if (inject == -1 && proto == 'T')
-   {
-      Interface_PopUp("Before injecting we need to sniff at least one packet...");
-      Interface_Sniff_Data_Redraw();
-      return;
-   }
-
-   if (inject == 0)
-   {
-      answer = Interface_PopUp("Do U really want to inject character in this connection (y/n)?");
-      Interface_Sniff_Data_Redraw();
-      if ((answer != 'y') && (answer != 'Y'))
-         return;
-   }
-
-   inject = 1;  // on exit the connection must be RSTted
-
-   memset(inject_data, 0, MAX_INJECT);
-
-   data_len = Interface_Inject_Run(inject_data, proto, app);
+   memset(&inject_data, 0, sizeof(INJECTED_DATA));
 
+   data_len = Interface_Inject_Run(inject_data.data, proto, app);
    Interface_Sniff_Data_Redraw();
 
-   if (win_pointer == data_dest_win)
+   inject_data.proto = proto;
+   inject_data.datalen = data_len;
+
+   if (data_len)
    {
-      write(pipe_inject_stod[1], inject_data, data_len);
-      wmove(bottom_window, 1, 30);
-      wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
-      wprintw(bottom_window, "%d ", inject_tos += data_len);
-      wbkgdset(bottom_window, COLOR_PAIR(SNIFF_COLOR));
-      wprintw(bottom_window, "chars injected to %s", ipsource);
-   }
-   else
-   {
-      write(pipe_inject_dtos[1], inject_data, data_len);
-      wmove(bottom_window, 0, 30);
-      wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
-      wprintw(bottom_window, "%d ", inject_tod += data_len);
-      wbkgdset(bottom_window, COLOR_PAIR(SNIFF_COLOR));
-      wprintw(bottom_window, "chars injected to %s", ipdest);
-   }
-
-   wnoutrefresh(bottom_window);
-
-   doupdate();
-
-}
-
-
-void Interface_Sniff_Data_Filter(void)
-{
-
-   DROP_FILTER filter;
-   DROP_FILTER null_filter;
-
-#ifdef DEBUG
-   Debug_msg("Interface_Sniff_Data_Filter %d", inject);
-#endif
-
-   memset(&filter, 0, sizeof(DROP_FILTER));
-   memset(&null_filter, 0, sizeof(DROP_FILTER));
-
-   if (Interface_Inject_Filter(&filter))
-   {
-      if (filter.slen == 0 && memcmp(&filter, &null_filter, sizeof(DROP_FILTER)) )
-      {
-         Interface_PopUp("Setting a 0 length search string is NOT a good idea... ;)");
-         Interface_Sniff_Data_Redraw();
-         return;
-      }
+      inject = 1;  // on exit the connection must be RSTted
 
       if (win_pointer == data_dest_win)
       {
          #ifdef DEBUG
-            Debug_msg("Interface_Sniff_Data_Filter -- set filter on source [%s]", filter.search);
+            Debug_msg("Interface_Sniff_Data_Inject -- INJECT -- %s", ipdest);
          #endif
-         write(pipe_filter_stod[1], &filter, sizeof(DROP_FILTER));
-         memcpy(&ARP_Filter_Source, &filter,  sizeof(DROP_FILTER));
+         inject_data.source_ip = inet_addr(ipsource);
+         inject_data.dest_ip = inet_addr(ipdest);
+         inject_data.source_port = htons(portsource);
+         inject_data.dest_port = htons(portdest);
+         write(pipe_inject[1], &inject_data, sizeof(INJECTED_DATA));
+         wmove(bottom_window, 1, 30);
+         wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
+         wprintw(bottom_window, "%d ", inject_tos += data_len);
+         wbkgdset(bottom_window, COLOR_PAIR(SNIFF_COLOR));
+         wprintw(bottom_window, "chars injected to %s", ipsource);
       }
       else
       {
          #ifdef DEBUG
-            Debug_msg("Interface_Sniff_Data_Filter -- set filter on dest [%s]", filter.search);
+            Debug_msg("Interface_Sniff_Data_Inject -- INJECT -- %s", ipsource);
          #endif
-         write(pipe_filter_dtos[1], &filter, sizeof(DROP_FILTER));
-         memcpy(&ARP_Filter_Dest, &filter,  sizeof(DROP_FILTER));
-      }
-
-      if ( !memcmp(&ARP_Filter_Source, &null_filter, sizeof(DROP_FILTER)) && !memcmp(&ARP_Filter_Dest, &null_filter, sizeof(DROP_FILTER)) )
-         Interface_Sniff_Data_ActiveFilter(0);
-      else
-      {
-         Interface_Sniff_Data_ActiveFilter(1);
-         if (filter.type != 'L') inject = 1;  // on exit the connection must be RSTted
+         inject_data.source_ip = inet_addr(ipdest);
+         inject_data.dest_ip = inet_addr(ipsource);
+         inject_data.source_port = htons(portdest);
+         inject_data.dest_port = htons(portsource);
+         write(pipe_inject[1], &inject_data, sizeof(INJECTED_DATA));
+         wmove(bottom_window, 0, 30);
+         wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
+         wprintw(bottom_window, "%d ", inject_tod += data_len);
+         wbkgdset(bottom_window, COLOR_PAIR(SNIFF_COLOR));
+         wprintw(bottom_window, "chars injected to %s", ipdest);
       }
    }
 
-   Interface_Sniff_Data_Redraw();
-
    wnoutrefresh(bottom_window);
 
    doupdate();
@@ -600,34 +551,37 @@ void Interface_Sniff_Data_Filter(void)
 }
 
 
-void Interface_Sniff_Data_ActiveFilter(short mode)
+
+
+void Interface_Sniff_Data_Kill(char proto, char *ipsource, short portsource, char *ipdest, short portdest)
 {
+   KILL_DATA kill_data;
 
-#ifdef DEBUG
-   Debug_msg("Interface_Sniff_Data_ActiveFilter %d", mode);
-#endif
-
-   wbkgdset(top_window, COLOR_PAIR(HELP_COLOR));
-   mvwprintw(top_window, 2, 55, "Active Filter: ");
-
-   switch(mode)
+   if (proto == 'U')
    {
-      case 1:  wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
-               mvwprintw(top_window, 2, 70, "ON ");
-               break;
-
-      case 0:  wbkgdset(top_window, COLOR_PAIR(BOTTOM_COLOR));
-               mvwprintw(top_window, 2, 70, "OFF");
-               break;
+      Interface_PopUp("Trying to kill an UDP connection ?!? Ehi Kiddie, go home !!");
+      return;
    }
 
-   wnoutrefresh(top_window);
-   doupdate();
+#ifdef DEBUG
+   Debug_msg("Interface_Sniff_Data_Kill -- %s:%d -> %s:%d ", ipsource, portsource, ipdest, portdest);
+#endif
+
+   kill_data.source_ip = inet_addr(ipsource);
+   kill_data.dest_ip = inet_addr(ipdest);
+   kill_data.source_port = htons(portsource);
+   kill_data.dest_port = htons(portdest);
+   write(pipe_kill[1], &kill_data, sizeof(KILL_DATA));
+
+   strcpy(Conn_Between_Hosts[Conn_Pointer].status, "KILLED");
+
+   Interface_PopUp("Connection KILLED !!");
 
 }
 
 
 
+
 void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int portdest, char *macsource, char *macdest, char proto, char *app, short mode)
 {
    int KeyPress;
@@ -635,26 +589,9 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
    int dimX = W_MAINX2 / 2 - 2;
    fd_set msk_fd;
    struct timeval TimeOut;
-   int Illithid_data_pid = 0, dopple_pid = 0;
 
    Buffer_Flush(pipe_with_illithid_data);
 
-   pipe(pipe_inject_stod);
-   pipe(pipe_inject_dtos);
-   pipe(pipe_filter_stod);
-   pipe(pipe_filter_dtos);
-
-   if ( mode <= PUBLICARP )   // PUBLICARP or ARPBASED
-   {
-      if ( mode == PUBLICARP )
-      {
-         dopple_pid = Doppleganger_Run(Options.netiface, ipsource, ipdest, macsource, macdest);
-         mode = ARPBASED;
-      }
-
-      Illithid_data_pid = Illithid_ARPBased_GetData(Options.netiface, proto, ipsource, portsource, ipdest, portdest, macsource, macdest);
-   }
-
 #ifdef DEBUG
    Debug_msg("Interface_Sniff_Data_Run -- %d -- %c [%s:%d] [%s:%d] [%s] [%s]", mode, proto, ipsource, portsource, ipdest, portdest, macsource, macdest );
 #endif
@@ -691,7 +628,6 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
 
    wbkgdset(main_window, COLOR_PAIR(MAIN_COLOR));
 
-   Interface_Sniff_Data_ActiveFilter(0);
    Interface_Sniff_Data_DrawScroller();
 
    werase(main_window);
@@ -786,13 +722,13 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
             case 'i':
                   if ( mode == ARPBASED )
                   {
-                     if (strstr(app, "ssh") || strstr(app, "SSH"))
+                     if (strstr(app, "ssh") || strstr(app, "SSH") || strstr(app, "HTTPS"))
                      {
                         Interface_PopUp("You CAN'T inject in a crypted connection !!");
                         Interface_Sniff_Data_Redraw();
                      }
                      else
-                        Interface_Sniff_Data_Inject(proto, app, ipsource, ipdest);
+                        Interface_Sniff_Data_Inject(proto, app, ipsource, portsource, ipdest, portdest);
                   }
                   else
                   {
@@ -803,21 +739,14 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
 
             case 'F':
             case 'f':
-                  if ( mode == ARPBASED )
-                  {
-                     if (strstr(app, "ssh") || strstr(app, "SSH"))
-                     {
-                        Interface_PopUp("You CAN'T filter a crypted connection !!");
-                        Interface_Sniff_Data_Redraw();
-                     }
-                     else
-                        Interface_Sniff_Data_Filter();
-                  }
-                  else
-                  {
-                     Interface_PopUp("Active Filtering is only supported in ARP Based mode !!");
-                     Interface_Sniff_Data_Redraw();
-                  }
+                  Interface_Inject_SetFilter(mode);
+                  Interface_Sniff_Data_Redraw();
+                  break;
+
+            case 'K':
+            case 'k':
+                  Interface_Sniff_Data_Kill(proto, ipsource, portsource, ipdest, portdest);
+                  Interface_Sniff_Data_Redraw();
                   break;
 
             case KEY_TAB:
@@ -837,7 +766,8 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
                         "[pgUp]    - scroll back (as well as UpArrow)",
                         "[pgDw]    - scroll forward (as well as DownArrow)",
                         "[iI]      - inject characters in the connection",
-                        "[fF]      - filter (search for, and replace or drop) packets",
+                        "[fF]      - set/edit filters chains",
+                        "[kK]      - kill the connection (be careful !)",
                         "[aA]      - ASCII view",
                         "[xX]      - HEX view",
                         "[sS]      - stop/cont the sniff (only visualization)",
@@ -851,30 +781,15 @@ void Interface_Sniff_Data_Run(char *ipsource, int portsource, char *ipdest, int
             case KEY_F(10):
             case 'Q':
             case 'q':
-                     if (inject == 1 && proto == 'T')
-                     {
-                        char answer;
-                        answer = Interface_PopUp("If U exit now the connection may be ReSeTted!! continue (y/n)?");
-                        Interface_Sniff_Data_Redraw();
-                        if ((answer != 'y') && (answer != 'Y'))
-                           break;
-                        else
-                           strcpy(Conn_Between_Hosts[Conn_Pointer].status, "KILLED");
-                     }
-                     if (Illithid_data_pid) kill(Illithid_data_pid, SIGTERM);
-                     if (dopple_pid) kill(dopple_pid, SIGTERM);
                      if (log_dest) fclose(file_dest);
                      if (log_source) fclose(file_source);
                      log_dest = log_source = 0;
                      stop = 0;
-                     inject = -1;
                      inject_tod=0; inject_tos=0;  // reset counter
                      delwin(data_source_win);
                      data_source_win = NULL;    // for winching (see ec_signal.c)
                      wbkgdset(bottom_window, COLOR_PAIR(BOTTOM_COLOR));
                      mvwhline(top_window, 2, 55, ' ', 20);
-                     memset(&ARP_Filter_Source, 0, sizeof(DROP_FILTER));
-                     memset(&ARP_Filter_Dest, 0, sizeof(DROP_FILTER));
                      delwin(data_dest_win);
                      delwin(data_source);
                      delwin(data_dest);
