@@ -17,18 +17,26 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_logtofile.c,v 1.21 2003/03/11 20:26:57 alor Exp $
+    $Id: ec_logtofile.c,v 1.4 2001/09/13 20:15:48 alor Exp $
 */
 
-#include "include/ec_main.h"
-
+#include <stdio.h>
+#include <string.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <time.h>
 
+#include "include/ec_main.h"
 #include "include/ec_inet.h"
+#include "include/ec_error.h"
 #include "include/ec_fingerprint.h"
 #include "include/ec_inet_structures.h"
 #include "include/ec_decodedata.h"
 
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
+
 // protos
 
 void LogToFile(SNIFFED_DATA *data);
@@ -37,83 +45,22 @@ void LogToFile_Collect(CONNECTION *data);
 void LogToFile_FilteredData(u_char * buf_ip);
 char *LogToFile_DumpPass(void);
 char *LogToFile_MakePassiveReport(char mode);
-int LogToFile_AlreadyLogged(CONNECTION *data, char *filename);
 
 // -------------------------------
 
-int LogToFile_AlreadyLogged(CONNECTION *data, char *filename)
-{
-   FILE *ftr;
-   char line[100];
-   int foo;
-   char *p, *q;
-   char source[16], dest[16], type[15], user[50], pass[50];
-
-   memset(line, 0, sizeof(line));
-
-   ftr = fopen(filename, "r");
-   if (ftr == NULL) return 0;
-
-   while ( fgets(line, 100, ftr) ) {
-
-      if ( ( p = strchr(line, '\n')) )
-         *p = 0;
-
-      if( sscanf(line, "%d:%d:%d", &foo, &foo, &foo) == 3 ) {  // new connection is found
-
-         strncpy(type, strrchr(line, ' ') + 1, 14 );
-         memset(source, 0, sizeof(source));
-         memset(dest, 0, sizeof(dest));
-
-         p = line + 10;
-         strncpy(source, strtok(p, ":"), 15);
-
-         q = strtok(NULL, ":");
-         strncpy(dest, strrchr(q, ' ') + 1, 15);
-
-         strcpy(user, ""); // inizialize a new user and pass
-         strcpy(pass, "");
-      }
-
-      if (!strncmp(line, "USER: ", 6)) {
-         strcpy(user, line);
-      }
-
-      if (!strncmp(line, "PASS: ", 6)) {
-         strcpy(pass, line);
-      }
-
-      if (!strcmp(user, data->user) && !strcmp(pass, data->pass) &&
-          !strcmp(type, data->type) && !strcmp(source, data->source_ip) &&
-          !strcmp(dest, data->dest_ip)) {
-         DEBUG_MSG("LogToFile_AlreadyLogged");
-         fclose(ftr);
-         return 1; // we have found the same user/pass IP
-      }
-
-   }
-
-   fclose(ftr);
-   return 0;
-
-}
-
-
 void LogToFile_Collect(CONNECTION *data)
 {
    FILE *fto;
-   static char filename[100] = "";
+   static char filename[50] = "";
    time_t tt = time(NULL);
    struct tm *dd = localtime(&tt);
 
-   DEBUG_MSG("LogToFile_Collect");
+#ifdef DEBUG
+   Debug_msg("LogToFile_Collect");
+#endif
 
    if (!strcmp(filename, ""))
-      snprintf(filename, sizeof(filename), "%04d%02d%02d-Collected-Passwords.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
-
-
-   if (LogToFile_AlreadyLogged(data, filename)) return;   // don't log the same user/pass IP twice
-
+      sprintf(filename, "%04d%02d%02d-Collected-Passwords.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
 
    fto = fopen(filename, "a");
    if (fto == NULL)
@@ -140,29 +87,23 @@ void LogToFile_Collect(CONNECTION *data)
 void LogToFile(SNIFFED_DATA *data)
 {
    FILE *fto;
-   char filename[100];
-#ifndef CYGWIN
+   char filename[50];
    time_t tt = time(NULL);
    struct tm *dd = localtime(&tt);
-#endif
-   
-#ifdef CYGWIN
-   snprintf(filename, sizeof(filename), "%c-%s[%d]-%s[%d].log",
-                        data->proto, data->source_ip, data->source_port, data->dest_ip, data->dest_port );
-#else
-   snprintf(filename, sizeof(filename), "%04d%02d%02d-%c-%s:%d-%s:%d.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
-                        data->proto, data->source_ip, data->source_port, data->dest_ip, data->dest_port );
-#endif
 
-   fto = fopen(filename, "ab");
+
+   sprintf(filename, "%04d%02d%02d-%c-%s:%d-%s:%d.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
+                        data->proto, data->source_ip, data->source_port, data->dest_ip, data->dest_port );
+
+   fto = fopen(filename, "a");
    if (fto == NULL)
       ERROR_MSG("fopen()");
 
 
    write(fileno(fto), data->data, data->datasize);
 
-//   if ( (data->flags & TH_RST) || (data->flags & TH_FIN) )
-//      fprintf(fto, "\n\n| CONNECTION CLOSED ON %02d:%02d:%02d |\n\n",  dd->tm_hour, dd->tm_min, dd->tm_sec);
+   if ( (data->flags & TH_RST) || (data->flags & TH_FIN) )
+      fprintf(fto, "\n\n| CONNECTION CLOSED ON %02d:%02d:%02d |\n\n",  dd->tm_hour, dd->tm_min, dd->tm_sec);
 
    fflush(fto);
    fclose(fto);
@@ -179,10 +120,9 @@ void LogToFile_FilteredData(u_char * buf_ip)
    short datalen = 0;
    FILE *fto;
    char logname[100];
-#ifndef CYGWIN
    time_t tt = time(NULL);
    struct tm *dd = localtime(&tt);
-#endif
+
 
    ip = (IP_header *) buf_ip;
 
@@ -191,35 +131,25 @@ void LogToFile_FilteredData(u_char * buf_ip)
       udp = (UDP_header *) ((int)ip + ip->h_len * 4);
       data = (char *)((int)udp + UDP_HEADER);
       datalen = ntohs(udp->len) - UDP_HEADER;
-#ifdef CYGWIN
-      snprintf(logname, sizeof(logname), "Filter-U-%s[%d]-", int_ntoa(ip->source_ip), ntohs(udp->source));
-      snprintf(logname, sizeof(logname), "%s%s[%d].log", logname, int_ntoa(ip->dest_ip), ntohs(udp->dest) ); // damned static in inet_ntoa...
-#else
-      snprintf(logname, sizeof(logname), "%04d%02d%02d-Filtered-U-%s:%d-", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
+      sprintf(logname, "%04d%02d%02d-Filtered-U-%s:%d-", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
                         int_ntoa(ip->source_ip),
                         ntohs(udp->source));
-      snprintf(logname, sizeof(logname), "%s%s:%d.log", logname, int_ntoa(ip->dest_ip), ntohs(udp->dest) ); // damned static in inet_ntoa...
-#endif
+      sprintf(logname, "%s%s:%d.log", logname, int_ntoa(ip->dest_ip), ntohs(udp->dest) ); // damned static in inet_ntoa...
    }
    else if (ip->proto == IPPROTO_TCP)
    {
       tcp = (TCP_header *) ((int)ip + ip->h_len * 4);
       data = (char *)((int)tcp + tcp->doff * 4);
       datalen = (int)ip + ntohs(ip->t_len) - (int)data;
-#ifdef CYGWIN
-      snprintf(logname, sizeof(logname), "Filter-T-%s[%d]-", int_ntoa(ip->source_ip), ntohs(tcp->source));
-      snprintf(logname, sizeof(logname), "%s%s[%d].log", logname, int_ntoa(ip->dest_ip), ntohs(tcp->dest) ); // damned static in inet_ntoa...
-#else
-      snprintf(logname, sizeof(logname), "%04d%02d%02d-Filtered-T-%s:%d-", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
+      sprintf(logname, "%04d%02d%02d-Filtered-T-%s:%d-", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday,
                         int_ntoa(ip->source_ip),
                         ntohs(tcp->source));
-      snprintf(logname, sizeof(logname), "%s%s:%d.log", logname, int_ntoa(ip->dest_ip), ntohs(tcp->dest) ); // damned static in inet_ntoa...
-#endif
+      sprintf(logname, "%s%s:%d.log", logname, int_ntoa(ip->dest_ip), ntohs(tcp->dest) ); // damned static in inet_ntoa...
    }
 
    if (data && datalen)
    {
-      fto = fopen(logname, "ab");
+      fto = fopen(logname, "a");
       if (fto == NULL)
          ERROR_MSG("fopen()");
       write(fileno(fto), data, datalen);
@@ -237,13 +167,12 @@ char *LogToFile_DumpPass(void)
    int i;
    time_t tt = time(NULL);
    struct tm *dd = localtime(&tt);
+
 #ifdef DEBUG
-   int count = 0;
+   Debug_msg("LogToFile_DumpPass");
 #endif
 
-   DEBUG_MSG("LogToFile_DumpPass");
-
-   snprintf(logname, sizeof(logname), "%04d%02d%02d-Dumped_Password.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
+   sprintf(logname, "%04d%02d%02d-%02d:%02d:-Dumped_Password.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday, dd->tm_hour, dd->tm_min);
 
    fto = fopen(logname, "a");
    if (fto == NULL)
@@ -264,32 +193,9 @@ char *LogToFile_DumpPass(void)
          if (strlen(Conn_Between_Hosts[i].info))
             fprintf(fto, "\n%s\n", Conn_Between_Hosts[i].info);
          fflush(fto);
-#ifdef DEBUG
-          count++;
-#endif
-      } else if (strlen(Conn_Between_Hosts[i].info)) {
-         fprintf(fto, "\n\n%s:%d -> %s:%d\t\t%s\n\n", Conn_Between_Hosts[i].source_ip,
-                                                      Conn_Between_Hosts[i].source_port,
-                                                      Conn_Between_Hosts[i].dest_ip,
-                                                      Conn_Between_Hosts[i].dest_port,
-                                                      Conn_Between_Hosts[i].type
-                                                      );
-         fprintf(fto, "USER : \n");
-         fprintf(fto, "PASS : \n");
-         if (strlen(Conn_Between_Hosts[i].info))
-            fprintf(fto, "\n%s\n", Conn_Between_Hosts[i].info);
-         fflush(fto);
-#ifdef DEBUG
-          count++;
-#endif
       }
    }
-   
-#ifdef DEBUG
-   /* Mac os X will complain about the count var not declared... */
-   DEBUG_MSG("LogToFile_DumpPass -- %d passwords dumped", count);
-#endif
-   
+
    fclose(fto);
    return logname;
 }
@@ -307,12 +213,14 @@ char *LogToFile_MakePassiveReport(char mode)
    u_long MyIP, NetMask;
    struct open_ports *current;
 
-   DEBUG_MSG("LogToFile_MakePassiveReport");
+#ifdef DEBUG
+   Debug_msg("LogToFile_MakePassiveReport");
+#endif
 
    if (mode == 'l')
-      snprintf(logname, sizeof(logname), "%04d%02d%02d-Passive_Local_Report.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
+      sprintf(logname, "%04d%02d%02d-%02d:%02d-Passive_Local_Report.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday, dd->tm_hour, dd->tm_min);
    else if (mode == 'L')
-      snprintf(logname, sizeof(logname), "%04d%02d%02d-Passive_Full_Report.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday);
+      sprintf(logname, "%04d%02d%02d-%02d:%02d-Passive_Full_Report.log", dd->tm_year+1900, dd->tm_mon+1, dd->tm_mday, dd->tm_hour, dd->tm_min);
 
    fto = fopen(logname, "a");
    if (fto == NULL)
@@ -407,6 +315,3 @@ char *LogToFile_MakePassiveReport(char mode)
 
 
 /* EOF */
-
-// vim:ts=3:expandtab
-
