@@ -17,16 +17,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_parser.c,v 1.39 2003/07/04 20:41:42 alor Exp $
+    $Id: ec_parser.c,v 1.6 2001/09/10 16:41:33 alor Exp $
 */
 
 
-#include <ec_main.h>
-
-#ifdef HAVE_GTK
-   #include <ec_gtk_splash.h>
-#endif
+#include "include/ec_main.h"
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <ctype.h>
 #ifdef HAVE_GETOPT_H
    #include <getopt.h>
@@ -34,41 +34,35 @@
    #include "missing/getopt.h"
 #endif
 
-#include <ec_inet.h>
-#include <ec_simple.h>
-#include <ec_dissector.h>
-#include <ec_inet_structures.h>
-#include <ec_filterdrop.h>
+#include "include/ec_error.h"
+#include "include/ec_inet.h"
+#include "include/ec_simple.h"
+#include "include/ec_dissector.h"
+#include "include/ec_inet_structures.h"
+#include "include/ec_filterdrop.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 extern char *Execute_Plugin;
 
-char *loading_plugs;    // list of plugin to be loaded.  it has this form: "|dummy|foo|bar|..."
-
-struct list {
-   int n;
-   int cur;
-   u_char values[0xff];
-};
+char *list_to_parse;    // host list expanded from wildcards
 
 // protos...
 
 extern void Main_Usage(void);
 void Parser_ParseConfFile(char *filename);
 void Parser_Dissectors(char *toparse);
-void Parser_Plugins(char *toparse);
-char Parser_Activated_Plugin(char *name);
 void Parser_ParseParameters(char *first, char *second, char *third, char *fourth);
 int Parser_ParseOptions(int counter, char **values);
-char *Parser_PrintFilter(DROP_FILTER *ptr, int i);
+char * Parser_PrintFilter(DROP_FILTER *ptr, int i);
 void Parser_LoadFilters(char *filename);
 int Parser_HostList(char *to_parse);
 char * Parser_AddStr(char *list, char *string);
-int Parser_Expand_List(char *to_parse);
-void Parser_Expand_Token(char *s, struct list *buf);
+void Parser_Expand(char *to_parse);
 int match_pattern(const char *s, const char *pattern);
 void Parser_Filters(char *line, DROP_FILTER *filter);
-char *Parser_StrSpacetoUnder(char *h_name);
-void Parser_StatusBar(int max);
 
 //-----------------------------------
 
@@ -81,7 +75,6 @@ int Parser_ParseOptions(int counter, char **values)
       { "help", no_argument, NULL, 'h' },
       { "version", no_argument, NULL, 'v' },
       { "simple", no_argument, NULL, 'N' },
-      { "gtk", no_argument, NULL, 'G' },
       { "list", no_argument, NULL, 'l' },
       { "arpsniff", no_argument, NULL, 'a' },
       { "sniff", no_argument, NULL, 's' },
@@ -90,19 +83,18 @@ int Parser_ParseOptions(int counter, char **values)
       { "netmask", required_argument, NULL, 'n' },
       { "check", no_argument, NULL, 'c' },
       { "plugin", required_argument, NULL, 'p' },
-      { "hexview", no_argument, NULL, '1' },
-      { "textview", no_argument, NULL, '2' },
-      { "ebcdicview", no_argument, NULL, '3' },
+      { "hexview", no_argument, NULL, 'x' },
       { "silent", no_argument, NULL, 'z' },
+      { "udp", no_argument, NULL, 'u' },
       { "fingerprint", no_argument, NULL, 'f' },
-      { "linktype", no_argument, NULL, 'g' },
+      { "linktype", no_argument, NULL, 't' },
       { "collect", no_argument, NULL, 'C' },
       { "broadping", no_argument, NULL, 'b' },
       { "logtofile", no_argument, NULL, 'L' },
       { "quiet", no_argument, NULL, 'q' },
       { "etterconf", required_argument, NULL, 'e' },
       { "dontresolve", no_argument, NULL, 'd' },
-      { "newcert", no_argument, NULL, 'w' },
+      { "newcert", no_argument, NULL, 'k' },
       { "filter", required_argument, NULL, 'F' },
       { "hosts", required_argument, NULL, 'H' },
       { "yes", no_argument, NULL, 'y' },
@@ -110,24 +102,21 @@ int Parser_ParseOptions(int counter, char **values)
       { "reverse", no_argument, NULL, 'R' },
       { "spoof", required_argument, NULL, 'S' },
       { "stormdelay", required_argument, NULL, 'Z' },
-      { "bufferlen", required_argument, NULL, 'B' },
       { "passive", no_argument, NULL, 'O' },
-      { "loadhosts", required_argument, NULL, 'j' },
-      { "savehosts", no_argument, NULL, 'k' },
-      { "proto", required_argument, NULL, 't' },
-      { "readpcapfile", required_argument, NULL, 'T' },
-      { "writepcapfile", required_argument, NULL, 'Y' },
-      { "onlypoison", no_argument, NULL, 'J' },
-      { "forceip", no_argument, NULL, 'X' },
-      { "refresh", required_argument, NULL, 'r' },
       { 0 , 0 , 0 , 0}
    };
 
-   DEBUG_MSG("Parser_ParseOptions -- [%d] [%s]", counter, *values);
+#ifdef DEBUG
+   Debug_msg("Parser_ParseOptions -- [%d] [%s]", counter, *values);
+#endif
 
    optind = 0;
 
-   while ((c = getopt_long (counter, values, "hvyNGlasmci:p:123zfgCbn:Lqe:dwF:H:D:RS:Z:B:Oj:kt:T:Y:JXr:",long_options, (int *)0)) != EOF) {
+#ifdef PERMIT_PLUGINS
+   while ((c = getopt_long (counter, values, "hvyNlasmci:p:xzuftCbn:Lqe:dkF:H:D:RS:Z:O",long_options, (int *)0)) != EOF) {
+#else
+   while ((c = getopt_long (counter, values, "hvyNlasmci:xzuftCbn:Lqe:dkF:H:D:RS:Z:O",long_options, (int *)0)) != EOF) {    // no plugin
+#endif
 
       switch (c) {
 
@@ -135,14 +124,13 @@ int Parser_ParseOptions(int counter, char **values)
             Main_Usage();
          break;
 
-         case 'v':   Options.version = 1;       break;
+         case 'v':   Options.version = 1;
+                     Options.normal = 1;        break;
 
          case 'y':   Options.yes = 1;           break;
 
          case 'N':   Options.normal = 1;        break;
 
-         case 'G':   Options.gtk = 1;           break;
-
          case 'l':   Options.list = 1;          break;
 
          case 'a':   Options.arpsniff = 1;      break;
@@ -153,11 +141,7 @@ int Parser_ParseOptions(int counter, char **values)
 
          case 'c':   Options.check = 1;         break;
 
-         case '1':   Options.hexview = 1;       break;
-                     
-         case '2':   Options.textview = 1;      break;
-                     
-         case '3':   Options.ebcdicview = 1;    break;
+         case 'x':   Options.hexview = 1;       break;
 
          case 'z':   Options.silent = 1;        break;
 
@@ -165,7 +149,7 @@ int Parser_ParseOptions(int counter, char **values)
 
          case 'f':   Options.finger = 1;        break;
 
-         case 'g':   Options.link = 1;          break;
+         case 't':   Options.link = 1;          break;
 
          case 'C':   Options.collect = 1;       break;
 
@@ -182,61 +166,19 @@ int Parser_ParseOptions(int counter, char **values)
          case 'O':   Options.passive = 1;
                      Options.silent = 1;        break;
 
+#ifdef PERMIT_PLUGINS
          case 'p':
-            #ifdef PERMIT_PLUGINS
-                     Options.plugin = 1;
-                     Execute_Plugin = strdup(optarg);
-
-            #else
-                     Error_msg("PLUGIN SUPPORT not supported on this OS.");
-            #endif
+            Options.plugin = 1;
+            Execute_Plugin = strdup(optarg);
          break;
+#endif
 
          case 'i':
-            strlcpy(Options.netiface, optarg, sizeof(Options.netiface));
+            strncpy(Options.netiface, optarg, 10);
          break;
 
-         case 'j':
-            Options.hostsfromfile = 1;
-            Options.hostfile = strdup(optarg);
-         break;
-
-         case 'k':
-            Options.hoststofile = 1;
-         break;
-
-         case 'T':
-            Options.silent = 1;
-            Options.dumpfile = strdup(optarg);
-         break;
-
-         case 'Y':
-            Options.writedump = 1;
-            Options.dumpfile = strdup(optarg);
-         break;
-
-         case 't':
-            if (!strncasecmp(optarg, "tcp", 3)) Options.tcp = 1;
-            else if (!strncasecmp(optarg, "udp", 3)) Options.udp = 1;
-            else if (!strncasecmp(optarg, "all", 3)) {
-               Options.tcp = 1;
-               Options.udp = 1;
-            } else
-               Error_msg("[%s]: protocol unrecognized", optarg);
-
-         break;
-
-         case 'J':
-            Options.normal = 1;
-            Options.onlypoison = 1;
-         break;
-         
-         case 'X':  
-            Options.force_ip = 1;
-         break;
-           
          case 'n':
-            strlcpy(Options.netmask, optarg, sizeof(Options.netmask));
+            strncpy(Options.netmask, optarg, 16);
          break;
 
          case 'D':
@@ -246,17 +188,7 @@ int Parser_ParseOptions(int counter, char **values)
 
          case 'Z':
             Options.storm_delay = atoi(optarg);
-            if (Options.storm_delay == 0) Options.storm_delay = 1;      // at least one usec...
-         break;
-
-         case 'r':
-            Options.refresh_delay = atoi(optarg);
-            if (Options.refresh_delay == 0) Options.refresh_delay = 1;   // at least one sec...
-         break;
-
-	 case 'B':
-            Options.buflen = atoi(optarg);
-            if (Options.buflen > MAX_BUFLEN) Options.buflen = MAX_BUFLEN;     
+            if (Options.storm_delay == 0) Options.storm_delay = 1;      // at least one second...
          break;
 
          case 'S':
@@ -270,7 +202,8 @@ int Parser_ParseOptions(int counter, char **values)
                return 1;
          break;
 
-         case 'w':
+         case 'k':
+            Options.normal = 1;
             Simple_CreateCertFile();
          break;
 
@@ -307,26 +240,18 @@ void Parser_ParseParameters(char *first, char *second, char *third, char *fourth
 {
 
 #ifdef DEBUG
-   if (first)  DEBUG_MSG("Parser_ParseParameters -- 1 [%s]", first); else goto exit_debug;
-   if (second) DEBUG_MSG("Parser_ParseParameters -- 2 [%s]", second); else goto exit_debug;
-   if (third)  DEBUG_MSG("Parser_ParseParameters -- 3 [%s]", third); else goto exit_debug;
-   if (fourth) DEBUG_MSG("Parser_ParseParameters -- 4 [%s]", fourth); else goto exit_debug;
+   if (first)  Debug_msg("Parser_ParseParameters -- 1 [%s]", first); else goto exit_debug;
+   if (second) Debug_msg("Parser_ParseParameters -- 2 [%s]", second); else goto exit_debug;
+   if (third)  Debug_msg("Parser_ParseParameters -- 3 [%s]", third); else goto exit_debug;
+   if (fourth) Debug_msg("Parser_ParseParameters -- 4 [%s]", fourth); else goto exit_debug;
 exit_debug:
 #endif
 
 #define R(a,b,c) (a & b) | ((a ^ b) & c)     // returns true if more than one was selected
 
-   if (!Options.normal &&
-                   (Options.reverse || Options.collect || Options.list ||
-                    Options.finger || Options.logtofile || Options.quiet ||
-                    Options.check))
-      Error_msg("Perhaps you have forgotten the -N option...\n");
+   if (Options.quiet && !Options.normal && !Options.logtofile )
+      Error_msg("Demonization is only useful with -NL or -NLC !!\n\n");
 
-   if (!Options.normal) Options.reverse = 0;
-
-   if (Options.quiet && !Options.normal ) 
-      Error_msg("Demonization is only useful with -N *cannot daemonize the interface* !!\n\n");
-   
    if ( R(Options.arpsniff, Options.sniff, Options.macsniff) )
       Error_msg("Please select only one sniffing method !!\n\n");
 
@@ -339,14 +264,6 @@ exit_debug:
    if (Options.normal && Options.passive && (Options.sniff || Options.arpsniff || Options.macsniff))
       Error_msg("Passive scanning can't be combined with a sniffing method !!\n\n");
 
-   if (Options.dumpfile && Options.arpsniff && !Options.writedump)
-      Error_msg("You can't ARP sniff from a file !!\n\n");
-
-   if (Options.onlypoison && (Options.passive || Options.filter || 
-                              Options.sniff || Options.arpsniff || Options.macsniff ||
-                              Options.logtofile))
-      Error_msg("This options can't be used in conjunction with -J !!\n\n");
-   
    if (Options.sniff || Options.macsniff)
       Options.silent = 1;
 
@@ -371,7 +288,7 @@ exit_debug:
             Error_msg("SOURCE and DEST MAC address must be different !!");
       }
 
-      if (Options.arpsniff || Options.onlypoison)
+      if (Options.arpsniff)
       {
          int i=0;
          char check[6];
@@ -449,7 +366,7 @@ exit_debug:
    }
    else // !silent
    {
-      if ((Options.arpsniff || Options.onlypoison) && !first)
+      if (Options.arpsniff && !first)
          Error_msg("Please specify  source and destination IP for ARP Based (full-duplex)\n"
                    "or only one IP for PUBLIC ARP (half-duplex)");
 
@@ -475,10 +392,12 @@ exit_debug:
 
 
 
-   DEBUG_MSG("Parser_ParseParameters - name - [%s][%s]", Host_Dest.name, Host_Source.name);
-   DEBUG_MSG("Parser_ParseParameters -  IP  - [%s][%s]", Host_Dest.ip, Host_Source.ip);
-   DEBUG_MSG("Parser_ParseParameters - port - [%d][%d]", Host_Dest.port, Host_Source.port);
-   DEBUG_MSG("Parser_ParseParameters -  MAC - [%s][%s]", Host_Dest.mac, Host_Source.mac);
+#ifdef DEBUG
+   Debug_msg("Parser_ParseParameters - name - [%s][%s]", Host_Dest.name, Host_Source.name);
+   Debug_msg("Parser_ParseParameters -  IP  - [%s][%s]", Host_Dest.ip, Host_Source.ip);
+   Debug_msg("Parser_ParseParameters - port - [%d][%d]", Host_Dest.port, Host_Source.port);
+   Debug_msg("Parser_ParseParameters -  MAC - [%s][%s]", Host_Dest.mac, Host_Source.mac);
+#endif
 
 }
 
@@ -486,12 +405,10 @@ exit_debug:
 
 char * Parser_AddStr(char *list, char *string)
 {
-   int len = strlen(list)+strlen(string)+1;
-
-   list = realloc(list, len);
+   list = realloc(list, strlen(list)+strlen(string)+1);
    if (list == NULL)
       ERROR_MSG("realloc()");
-   strlcat(list, string, len);
+   strcat(list, string);
 
    return list;
 }
@@ -509,11 +426,10 @@ void Parser_ParseConfFile(char *filename)
    int pargc = 0;
    char *pargv[256];
    char dissectors = 0;
-#ifdef PERMIT_PLUGINS
-   char plugins = 0;
-#endif
 
-   DEBUG_MSG("Parser_ParseConfFile - %s", filename);
+#ifdef DEBUG
+   Debug_msg("Parser_ParseConfFile - %s", filename);
+#endif
 
    memset(&pargv, 0, 256*sizeof(int));
 
@@ -538,7 +454,9 @@ void Parser_ParseConfFile(char *filename)
       if (!strlen(line))   // skip 0 length line
          continue;
 
-      DEBUG_MSG("Parser_ParseConfFile - [%s]", line);
+      #ifdef DEBUG
+         Debug_msg("Parser_ParseConfFile - [%s]", line);
+      #endif
 
       if (!strncasecmp(line, "OPTIONS: ", 9))
       {
@@ -635,7 +553,10 @@ void Parser_ParseConfFile(char *filename)
 
          option = strdup(line+6);
          if ( inet_aton(option, (struct in_addr *)&illithid_gwip) == 0)
+         {
+            Options.normal = 1;  // prevent Error_msg to close the screen
             Error_msg("Incorrect GWIP (%s) in the conf file !!", option);
+         }
          free(option);
       }
 
@@ -650,19 +571,6 @@ void Parser_ParseConfFile(char *filename)
          fprintf (stdout, "Setting dissectors handlers...\n");
          dissectors = 1;
       }
-#ifdef PERMIT_PLUGINS
-      if (!strncasecmp(line, "</hooking plugins>", 18))
-         plugins = 0;
-
-      if (plugins)
-         Parser_Plugins(line);
-
-      if (!strncasecmp(line, "<hooking plugins>", 17))
-      {
-         fprintf (stdout, "Plugins to be loaded...\n");
-         plugins = 1;
-      }
-#endif
 
    } while (!feof(etterconf));
 
@@ -670,7 +578,9 @@ void Parser_ParseConfFile(char *filename)
    if (!strcmp(to_be_parsed, "etter.conf ")) // no options in the file....
       return;
 
-   DEBUG_MSG("Parser_ParseConfFile - [%s]", to_be_parsed);
+#ifdef DEBUG
+      Debug_msg("Parser_ParseConfFile - [%s]", to_be_parsed);
+#endif
 
    ptr = strtok(to_be_parsed, " ");
    pargv[pargc++] = strdup(ptr);
@@ -682,9 +592,9 @@ void Parser_ParseConfFile(char *filename)
 {
    int i;
    for(i=0; i<pargc; i++)
-      DEBUG_MSG("Parser_ParseConfFile - [%d] %s", i, pargv[i]);
+      Debug_msg("Parser_ParseConfFile - [%d] %s", i, pargv[i]);
 
-   DEBUG_MSG("Parser_ParseConfFile - pargc [%d]", pargc);
+   Debug_msg("Parser_ParseConfFile - pargc [%d]", pargc);
 }
 #endif
 
@@ -694,11 +604,9 @@ void Parser_ParseConfFile(char *filename)
 }
 
 
-
-
 void Parser_Dissectors(char *toparse)
 {
-   char name[30]; // doubled this from 15
+   char name[15];
    char arguments[25];
    char *parseport;
    char *proto;
@@ -710,9 +618,9 @@ void Parser_Dissectors(char *toparse)
    memset(name, 0, sizeof(name));
    memset(arguments, 0, sizeof(arguments));
 
-   strlcpy(name, strtok(toparse, "="), sizeof(name));
+   strncpy(name, strtok(toparse, "="), 15);
 
-   strlcpy(arguments, strtok(NULL, "="), sizeof(arguments));
+   strncpy(arguments, strtok(NULL, "="), 24);
 
    if (!strncmp(arguments, "OFF", 3))
    {
@@ -761,46 +669,6 @@ void Parser_Dissectors(char *toparse)
 
 
 
-void Parser_Plugins(char *toparse)
-{
-   char name[20], l_name[22];
-   char args[4];
-
-   if (!strchr(toparse, '='))    // malformed line
-      return;
-
-   if (!loading_plugs) loading_plugs = (char *)calloc(1,1);
-
-   memset(name, 0, sizeof(name));
-   strlcpy(name, strtok(toparse, "="), 20);
-   strlcpy(args, strtok(NULL, "="), 4);
-
-   if (!strncmp(args, "ON", 2))
-   {
-      snprintf(l_name, sizeof(l_name), "|%s|", name);
-      loading_plugs = (char *)realloc(loading_plugs, strlen(loading_plugs)+strlen(l_name)+2);
-      strcat(loading_plugs, l_name);
-      fprintf(stdout, "%s\n", name);
-   }
-}
-
-
-
-
-char Parser_Activated_Plugin(char *name)
-{
-   char l_name[22];
-   snprintf(l_name, sizeof(l_name), "|%s|", name);
-
-   if (!loading_plugs) return 0;
-
-   if (strstr(loading_plugs, l_name)) return 1;
-
-   return 0;
-}
-
-
-
 void Parser_Filters(char *line, DROP_FILTER *filter)
 {
    int i, j;
@@ -814,9 +682,9 @@ void Parser_Filters(char *line, DROP_FILTER *filter)
       q = strstr(p, "</search>");
       i = ((int)q-(int)p) - strlen("<search>");
       if (i==0) return;
-      snprintf(tmp, sizeof(tmp), "<search>%%%dc</search>", i);
+      sprintf(tmp, "<search>%%%dc</search>", i);
       sscanf(p, tmp, filter->display_search);
-      filter->wildcard = FilterDrop_ParseWildcard(tmp_search, filter->display_search, sizeof(tmp_search));
+      filter->wildcard = FilterDrop_ParseWildcard(tmp_search, filter->display_search);
       filter->slen = FilterDrop_strescape(filter->search, tmp_search);
       return;
    }
@@ -826,7 +694,7 @@ void Parser_Filters(char *line, DROP_FILTER *filter)
       q = strstr(p, "</replace>");
       i = ((int)q-(int)p) - strlen("<replace>");
       if (i==0) return;
-      snprintf(tmp,sizeof(tmp) ,"<replace>%%%dc</replace>", i);
+      sprintf(tmp, "<replace>%%%dc</replace>", i);
       sscanf(p, tmp, filter->display_replace);
       filter->rlen = FilterDrop_strescape(filter->replace, filter->display_replace);
       return;
@@ -880,7 +748,9 @@ void Parser_LoadFilters(char *filename)
    if (Filter_File)
    {
       etterfilter = fopen(Filter_File, "r");
-      DEBUG_MSG("Parser_LoadFilters - [%s]", Filter_File);
+      #ifdef DEBUG
+         Debug_msg("Parser_LoadFilters - [%s]", Filter_File);
+      #endif
    }
    else if (!strcmp(filename, ""))
    {
@@ -888,25 +758,29 @@ void Parser_LoadFilters(char *filename)
       etterfilter = fopen(line, "r");
       if (etterfilter == NULL)
       {
-         strlcpy(line, DATA_PATH, sizeof(line));
-         strlcat(line, "/etter.filter", sizeof(line));
+         strcpy(line, INSTALL_PATH);
+         strcat(line, "etter.filter");
          etterfilter = fopen(line, "r");
       }
       Filter_File = strdup(line);
-      DEBUG_MSG("Parser_LoadFilters - [%s]", line);
+      #ifdef DEBUG
+         Debug_msg("Parser_LoadFilters - [%s]", line);
+      #endif
    }
    else
    {
       etterfilter = fopen(filename, "r");
       Filter_File = strdup(filename);
-      DEBUG_MSG("Parser_LoadFilters - [%s]", filename);
+      #ifdef DEBUG
+         Debug_msg("Parser_LoadFilters - [%s]", filename);
+      #endif
    }
 
    if (etterfilter == NULL)
-         Error_msg("CAN'T find a filter file in ./ or in %s", DATA_PATH);
+         Error_msg("CAN'T find a filter file in ./ or in %s", INSTALL_PATH);
 
-   Filter_Source = 0;
-   Filter_Dest = 0;
+   Filter_Source=0;
+   Filter_Dest=0;
 
    if (Filter_Array_Source) free(Filter_Array_Source);
    if (Filter_Array_Dest) free(Filter_Array_Dest);
@@ -967,10 +841,10 @@ void Parser_LoadFilters(char *filename)
 {
    short i;
    for (i=0; i<Filter_Source; i++)
-      DEBUG_MSG("\tSOURCE: %s", Parser_PrintFilter(Filter_Array_Source, i));
+      Debug_msg("\tSOURCE: %s", Parser_PrintFilter(Filter_Array_Source, i));
 
    for (i=0; i<Filter_Dest; i++)
-      DEBUG_MSG("\tDEST  : %s", Parser_PrintFilter(Filter_Array_Dest, i));
+      Debug_msg("\tDEST  : %s", Parser_PrintFilter(Filter_Array_Dest, i));
 }
 #endif
 
@@ -982,7 +856,7 @@ char * Parser_PrintFilter(DROP_FILTER *ptr, int i)
    static char tmp[100];
    int j;
 
-   j = snprintf(tmp, sizeof(tmp), "%2d | %5d:%-5d %c [%-10.10s] %c ", i,  ptr[i].source, ptr[i].dest, ptr[i].proto, ptr[i].display_search, ptr[i].type);
+   j = sprintf(tmp, "%2d | %5d:%-5d %c [%-10.10s] %c ", i,  ptr[i].source, ptr[i].dest, ptr[i].proto, ptr[i].display_search, ptr[i].type);
    if (ptr[i].type == 'R')
       j += sprintf(tmp+j, "[%-10.10s] ", ptr[i].display_replace);
    else
@@ -1001,150 +875,117 @@ char * Parser_PrintFilter(DROP_FILTER *ptr, int i)
 
 
 
-void Parser_Expand_Token(char *s, struct list *buf)
+void Parser_Expand(char *to_parse)
 {
-/*
- * This function parses the input in the form [1-3,17,5-11]
- * and fill the structure with expanded numbers.
- */
+   static int j=0;
+   int i=0, found=0;
+   char *q;
+   char new_parse[25];
+   char ip[25];
+   char *pattern;
 
-   char *str = strdup(s);
-   char *p, *q, r;
-   char *end;
-   u_int a = 0, b = 0;
-          
-   p = str;
-   end = p + strlen(p);
-   
-   while (p < end) {
-      q = p;
-      
-      while ( isdigit((int)*q) && q++ < end);
-      
-      r = *q;   
-      *q = 0;
-      a = atoi(p);
-      if (a > 255) 
-         Error_msg("Invalid IP range !!");
-      
-      if ( r == '-') {
-         p = ++q;
-         while ( isdigit((int)*q) && q++ < end);
-         *q = 0;
-         if (*p == '\0') 
-            Error_msg("Invalid IP range !!");
-         b = atoi(p);
-         if (b > 255 || b < a) 
-            Error_msg("Invalid IP range !!");
-      } else {
-         b = a; 
-      } 
-      
-      for(; a <= b; a++) {
-         buf->n++;
-         buf->values[buf->n - 1] = (u_char) a;
-      }
-      
-      if (q == end) break;
-      else  p = q + 1;      
-   }
-  
-   free(str);
-}
+   memset(new_parse, 0, 25);
+   memset(ip, 0, 25);
+   pattern = strdup(to_parse);
 
-
-
-int Parser_Expand_List(char *to_parse)
-{
-   struct list ADDR[4];
-   char *addr[4];
-   char parsed_ip[16];
-   char *p, *q;
-   int i = 0, j;
-   int permut = 1;
-   static int k = 0;
- 
-   memset(&ADDR, 0, sizeof(ADDR));
-   
-   p = to_parse;
-   
-   while ( (q = strtok(p, ".")) ) {
-      addr[i++] = strdup(q);
-      if (p != NULL) p = NULL;
-      if (i > 4) break;
-   }
-  
-   if (i != 4) 
-      Error_msg("Invalid IP format !!");
-   
-   DEBUG_MSG("Parser_Expand_List -- [%s] [%s] [%s] [%s]", addr[0], addr[1], addr[2], addr[3]);
-  
-   for (i = 0; i < 4; i++) {
-      p = addr[i];
-      Parser_Expand_Token(p, &ADDR[i]);
-   }
-
-   for (i = 0; i < 4; i++)     /* count the free permutations */
-      permut *= ADDR[i].n; 
-   
-   for (i = 0; i < permut; i++) {
-     
-      sprintf(parsed_ip, "%d.%d.%d.%d",  ADDR[0].values[ADDR[0].cur],
-                                         ADDR[1].values[ADDR[1].cur],
-                                         ADDR[2].values[ADDR[2].cur],
-                                         ADDR[3].values[ADDR[3].cur]);
-      
-   DEBUG_MSG("\t\t%s", parsed_ip);
-      
-      Host_List[k++] = strdup(parsed_ip);
-      Host_List = realloc(Host_List, (k+2)*sizeof(char *));
-      if (Host_List == NULL)
-         ERROR_MSG("realloc()");
-      
-      
-      ADDR[3].cur++;                /* give the impulse to the last octet */
-      
-      for (j = 2; j >= 0; j--) {    /* adjust the other as in a digital counter */
-         if ( ADDR[j+1].cur >= ADDR[j+1].n  ) {
-            ADDR[j].cur++;
-            ADDR[j+1].cur = 0;
+   if (strstr(pattern, "*") || strstr(pattern, "?"))
+   {
+      for( q=strtok(pattern, "."); q!=NULL; q=strtok(NULL, "."))
+      {
+         i++;
+         if (!found && (strstr(q, "*") || strstr(q, "?")) )
+         {
+            strlcat(new_parse, "%d", 25);
+            if (i<4) strlcat(new_parse, ".", 25);
+            found=1;
          }
+         else
+         {
+            strlcat(new_parse, q, 25);
+            if (i<4) strlcat(new_parse, ".", 25);
+         }
+
       }
-      
-      Parser_StatusBar(permut);
-      
    }
-   
-   for (i = 0; i < 4; i++)
-      free(addr[i]);
-              
-   return permut;
+   else
+      strcpy(new_parse, to_parse);
+
+   free(pattern);
+
+   if (!found) // no more wildcards
+   {
+      if (j++%10 == 0)  // the progress bar...
+      {
+         printf(".");
+         fflush(stdout);
+      }
+      list_to_parse = realloc(list_to_parse, strlen(list_to_parse)+strlen(to_parse)+2);
+      if (list_to_parse == NULL)
+         ERROR_MSG("realloc()");
+      strcat(list_to_parse, to_parse);
+      strcat(list_to_parse, ",");
+      return;
+   }
+
+   for(i=0; i<256; i++)
+   {
+      sprintf(ip, new_parse, i);
+      if (match_pattern(ip, to_parse))
+         Parser_Expand(ip);
+   }
 }
 
 
+
 int Parser_HostList(char *to_parse)
 {
    char *ip;
-   int i = 0;
+   u_long dummy;
+   int i=0;
 
-   fprintf(stdout, "Expanding the -H option hosts list...\n\n");
-   
-   DEBUG_MSG("Parser_HostList - [%s]", to_parse);
+#ifdef DEBUG
+   Debug_msg("Parser_HostList - [%s]", to_parse);
+#endif
 
-   if (strstr(to_parse, " ") || strstr(to_parse, "%") )
-      Error_msg("\nThe host list has an invalid format...");
+   fprintf(stdout, "Expanding wildcarded hosts...");
 
-   Host_List = calloc(2, sizeof(char *)); /* the list is modified by Parser_Expand_List */
+   if (strstr(to_parse, " "))
+   {
+      fprintf(stdout, "\n");
+      Options.normal = 1;
+      Error_msg("The host list can't contain blank spaces...");
+   }
+
+   list_to_parse = calloc(1,1);
+
+   for(ip=strsep(&to_parse, ","); ip != NULL; ip=strsep(&to_parse, ","))
+      Parser_Expand(ip);
+
+#ifdef DEBUG
+   Debug_msg("Parser_HostList - [%s]", list_to_parse);
+#endif
+
+   fprintf(stdout, "\nLoading IP addresses from list...\n");
+
+   Host_List = calloc(2, sizeof(char *));
    if (Host_List == NULL)
       ERROR_MSG("calloc()");
-   
-   for(ip=strsep(&to_parse, ";"); ip != NULL; ip=strsep(&to_parse, ";")) 
-      i += Parser_Expand_List(ip);
 
-   fprintf(stdout, "\n");
+   for(ip=strtok(list_to_parse, ","); ip != NULL; ip=strtok(NULL, ","))
+   {
+      if (inet_aton(ip, (struct in_addr *)&dummy) != 0)
+      {
+         Host_List[i++] = strdup(ip);
+         Host_List = realloc(Host_List, (i+2)*sizeof(char *));
+         if (Host_List == NULL)
+            ERROR_MSG("realloc()");
+      }
+      else
+         fprintf(stdout, "WARNING: %s is an invalid IP address\n", ip);
+   }
+
+   free(list_to_parse);
 
-   DEBUG_MSG("Parser_HostList - expanded hosts [%d]", i);
-   
    return i;
 }
 
@@ -1153,22 +994,27 @@ int Parser_HostList(char *to_parse)
 /* Pattern matching code from OpenSSH. */
 int match_pattern(const char *s, const char *pattern)
 {
-   for (;;) {
+   for (;;)
+   {
       if (!*pattern) return (!*s);
 
-      if (*pattern == '*') {
+      if (*pattern == '*')
+      {
          pattern++;
 
          if (!*pattern) return (1);
 
-         if (*pattern != '?' && *pattern != '*') {
-            for (; *s; s++) {
+         if (*pattern != '?' && *pattern != '*')
+         {
+            for (; *s; s++)
+            {
                if (*s == *pattern && match_pattern(s + 1, pattern + 1))
                   return (1);
             }
             return (0);
          }
-         for (; *s; s++) {
+         for (; *s; s++)
+         {
             if (match_pattern(s, pattern))
                return (1);
          }
@@ -1186,89 +1032,5 @@ int match_pattern(const char *s, const char *pattern)
 }
 
 
-char *Parser_StrSpacetoUnder(char *h_name)
-{
-   int i;
-   static char toggle_name[200];
-
-   strcpy(toggle_name, h_name);
-   i = strlen(toggle_name);
-
-   for (i--;i>=0;i--)
-      if (toggle_name[i]==' ')
-         toggle_name[i]='_';
-
-   return toggle_name;
-}
-
-
-
-void Parser_StatusBar(int max)
-{
-   static int progress = 1;
-
-   if (!Options.onscreen && !Options.gtk) {
-      float percent;
-      int i;
-
-      percent = (float)(progress)*100/(max);
-
-   /* we use stderr to avoid scrambling of logfile generated by: ettercap -Nl > logfile */
-
-      switch(progress % 4) {
-         case 0:
-               fprintf(stderr, "\r| |");
-               break;
-         case 1:
-               fprintf(stderr, "\r/ |");
-               break;
-         case 2:
-               fprintf(stderr, "\r- |");
-               break;
-         case 3:
-               fprintf(stderr, "\r\\ |");
-               break;
-      }
-
-      for (i=0; i < percent/2; i++)
-         fprintf(stderr, "=");
-
-      fprintf(stderr, ">");
-
-      for(; i < 50; i++)
-         fprintf(stderr, " ");
-
-      fprintf(stderr, "| %6.2f %%", percent );
-
-      fflush(stderr);
-
-      if (progress == max) {
-         progress = 1;
-         fprintf(stderr, "\r* |==================================================>| 100.00 %%\n");
-         return;
-      }
-
-      progress++;
-
-   } 
-#ifdef HAVE_GTK
-   else if(Options.gtk) {
-      gdouble percent;
-
-      percent = (gdouble)(progress)/(max);
-      Ginterface_Splash_Progress_Change(percent);
-      progress++;
-   }
-#endif
-}
-
-
-
-
-
-
 /* EOF */
 
-
-// vim:ts=3:expandtab
-
