@@ -17,77 +17,122 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_signal.c,v 1.12 2002/07/20 09:41:26 alor Exp $
+    $Id: ec_signal.c,v 1.1.1.1 2001/08/27 19:23:43 alor Exp $
 */
 
 #include "include/ec_main.h"
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <signal.h>
+#include <sys/time.h>
+#include <sys/resource.h>
+#include <unistd.h>
+#if defined (LINUX) && !defined (__USE_GNU)
+   #define __USE_GNU    // for strsignal()
+#endif
+#include <string.h>
 
-#ifndef CYGWIN
-   #include <sys/resource.h>
+
+#ifdef HAVE_NCURSES
+   #ifdef HAVE_NCURSES_H
+      #include <ncurses.h>
+   #else
+      #include <curses.h>
+   #endif
+   #include "include/ec_interface.h"
+   #include "include/ec_interface_sniff_data.h"
+   #include "include/ec_interface_plugins.h"
+
+   extern WINDOW *data_source_win;
+   extern WINDOW *plugin_window;
+   extern short ScreenMode;
+
+   #ifdef HAVE_TERMIOS_H
+      #include <termios.h>
+      extern struct termios original_term_info;
+   #endif
 #endif
 
-#include "include/ec_interface.h"
-#include "include/ec_thread.h"
+#include "include/ec_error.h"
+
+#ifdef DEBUG
+   #include "include/ec_debug.h"
+#endif
 
 // protos...
 
 void Signal_SigBuster(void);
+RETSIGTYPE Signal_ILL(int sig);
 RETSIGTYPE Signal_SEGV(int sig);
 RETSIGTYPE Signal_TERM(int sig);
 RETSIGTYPE Signal_WINCH(int sig);
-
-#ifdef HAVE_TERMIOS_H
-   extern void reset_term(void);
-#endif
-
+RETSIGTYPE Signal_USR1(int sig);
+RETSIGTYPE Signal_USR2(int sig);
 
 //-----------------------
 
 void Signal_SigBuster(void)
 {
-   DEBUG_MSG("Signal_Buster");
+#ifdef DEBUG
+   Debug_msg("Signal_Buster");
+#endif
 
-   signal(SIGBUS,   Signal_SEGV);
+   signal(SIGILL,  Signal_ILL);     // this is received by ettercap if one of its sons get a SIGSEGV
+   signal(SIGBUS,  Signal_SEGV);
    signal(SIGSEGV,  Signal_SEGV);
-   signal(SIGINT,   Signal_TERM);
    signal(SIGHUP,   Signal_TERM);
    signal(SIGTERM,  Signal_TERM);
    signal(SIGWINCH, Signal_WINCH);
    signal(SIGCHLD,  SIG_IGN);       // if I kill a forked process it doesn't become a zombie...
    signal(SIGPIPE,  Signal_TERM);   // if ettercap SIGSEGV, illithid receives this sig... (pipe_with_illithid)
-#ifdef SOLARIS
-   signal(SIGALRM, SIG_IGN);  // a fix for some solaris wreid behaviour
+
+   signal(SIGUSR1, Signal_USR1);
+   signal(SIGUSR2, Signal_USR2);
+}
+
+
+RETSIGTYPE Signal_ILL(int sig)
+{
+#ifdef DEBUG
+   Debug_msg("[%s] One of my sons got an error...", program_argv0 );
 #endif
+#ifdef HAVE_NCURSES
+   if (ScreenMode)    // restore the cursor
+      curs_set(1);
+
+   #ifdef HAVE_TERMIOS_H
+      tcsetattr(0, TCSANOW, &original_term_info);
+   #endif
+#endif
+
+   exit(666);
 }
 
 
 RETSIGTYPE Signal_SEGV(int sig)
 {
 #ifdef DEBUG
-
-#ifndef CYGWIN
    struct rlimit corelimit = {RLIM_INFINITY, RLIM_INFINITY};
+   if (sig == SIGBUS)
+      Debug_msg("[%s] Bus Error...", program_argv0 );
+   else if (sig == SIGSEGV)
+      Debug_msg("[%s] Segmentation Fault...", program_argv0 );
 #endif
 
-   if (sig == SIGBUS)
-      DEBUG_MSG("[%s] Bus Error...", ECThread_getname(pthread_self()) );
-   else if (sig == SIGSEGV)
-      DEBUG_MSG("[%s] Segmentation Fault...", ECThread_getname(pthread_self()) );
-#endif
+   if (!strstr(program_argv0, PROGRAM)) kill(getppid(), SIGILL);    // kill the main process
 
 #ifdef HAVE_NCURSES
-   if (Options.onscreen)    // close the ncurses screen
+   if (ScreenMode)    // close the ncurses screen
       Interface_CloseScreen();
 #endif
 
 
    fprintf (stderr, "\n\033[01m\033[1m Ooops !! Somewhere in the stack a pointer got crazy...\n\n");
    if (sig == SIGBUS)
-      fprintf (stderr, " [%s] Bus Error...\033[0m\n\n\n", ECThread_getname(pthread_self()));
+      fprintf (stderr, " [%s] Bus Error...\033[0m\n\n\n", program_argv0);
    else if (sig == SIGSEGV)
-      fprintf (stderr, " [%s] Segmentation Fault...\033[0m\n\n", ECThread_getname(pthread_self()));
+      fprintf (stderr, " [%s] Segmentation Fault...\033[0m\n\n", program_argv0);
 
    fprintf (stderr, "===========================================================================\n");
    fprintf (stderr, " To report this error follow these steps:\n\n");
@@ -98,7 +143,8 @@ RETSIGTYPE Signal_SEGV(int sig)
 #else
    fprintf (stderr, "  1) and 2) already done...\n\n");
 #endif
-   if (sig == SIGBUS) {
+   if (sig == SIGBUS)
+   {
       fprintf (stderr, "  3) make a report : \"tar zcvf error.tar.gz ettercap_debug.log\"\n\n");
       fprintf (stderr, "  4) mail us the error.tar.gz\n\n\n\n");
       exit(666);
@@ -107,10 +153,10 @@ RETSIGTYPE Signal_SEGV(int sig)
    fprintf (stderr, "  3) make a report : \"tar zcvf error.tar.gz ettercap_debug.log \"\n\n");
    fprintf (stderr, "  4) get the gdb backtrace :\n"
                     "  \t - \"gdb ettercap core\"\n"
-                    "  \t - at the gdb prompt \"bt\"\n"
-                    "  \t - at the gdb prompt \"quit\" and return to the shell\n"
-                    "  \t - copy and paste this output.\n\n");
-   fprintf (stderr, "  5) mail us the output of gdb and the error.tar.gz\n");
+                    " \t - at the gdb prompt \"bt\"\n"
+                    " \t - at the gdb prompt \"quit\" and return to the shell\n"
+                    " \t - copy and paste this output.\n\n");
+   fprintf (stderr, " 5) mail us the output of gdb and the error.tar.gz\n");
    fprintf (stderr, "============================================================================\n");
 
 #ifndef DEBUG
@@ -119,10 +165,7 @@ RETSIGTYPE Signal_SEGV(int sig)
    fprintf(stderr, "\n\033[01m\033[1m Overriding any 'ulimit -c 0'...\n"
                    " Setting core size to RLIM_INFINITY...\n\n"
                    " Core dumping... (use the 'core' file for gdb analysis)\033[0m\n\n");
-#ifndef CYGWIN
    setrlimit(RLIMIT_CORE, &corelimit);
-#endif
-
    signal(sig, SIG_DFL);
    raise(sig);
 #endif
@@ -132,34 +175,28 @@ RETSIGTYPE Signal_SEGV(int sig)
 
 RETSIGTYPE Signal_TERM(int sig)
 {
-#ifdef HAVE_STRSIGNAL
-   DEBUG_MSG("[%s] Signal handler... (caught SIGNAL: %d) | %s", ECThread_getname(pthread_self()), sig, strsignal(sig));
-#else
-   DEBUG_MSG("[%s] Signal handler... (caught SIGNAL: %d)", ECThread_getname(pthread_self()), sig);
+#ifdef DEBUG
+   #ifdef HAVE_STRSIGNAL
+      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d) | %s", program_argv0, sig, strsignal(sig));
+   #else
+      Debug_msg("[%s] Signal handler... (caught SIGNAL: %d)", program_argv0, sig);
+   #endif
 #endif
 
+   if (!strstr(program_argv0, PROGRAM)) exit(0);    // is not the main process...
 
 #ifdef HAVE_NCURSES
-   if (Options.onscreen)
+   if (ScreenMode)
       Interface_CloseScreen();
 #endif
-#ifdef HAVE_TERMIOS_H
-   if(!Options.quiet && !Options.onscreen)
-      reset_term();
-#endif  
 
-   if (sig == SIGINT) {
-      fprintf(stderr, "\n\nUser requested a CTRL+C... (deprecated, next time use 'q')\n\n");
-   } else {
-      #ifdef HAVE_STRSIGNAL
-         fprintf(stderr, "\n\n Ettercap Signal handler... (caught SIGNAL: %d) | %s\n\n", sig, strsignal(sig));
-      #else
-         fprintf(stderr, "\n\n Ettercap Signal handler... (caught SIGNAL: %d)\n\n", sig);
-      #endif
-   }
-
-   signal(sig, SIG_IGN);
+   #ifdef HAVE_STRSIGNAL
+      printf("\n Signal handler... (caught SIGNAL: %d) | %s\n\n", sig, strsignal(sig));
+   #else
+      printf("\n Signal handler... (caught SIGNAL: %d)\n\n", sig);
+   #endif
 
+   kill(-1*getpid(), SIGTERM);  // kill all my forked processes
    exit(1);
 
 }
@@ -170,19 +207,29 @@ RETSIGTYPE Signal_WINCH(int sig)
 {
 
 #ifdef HAVE_NCURSES
-   if (Options.onscreen) {
-      if (data_source_win != NULL) {
-         DEBUG_MSG("Winching sniff data windows...");
+   if (ScreenMode)
+   {
+      if (data_source_win != NULL)
+      {
+         #ifdef DEBUG
+            Debug_msg("Winching sniff data windows...");
+         #endif
          Interface_Sniff_Data_Winch();
       }
 #ifdef PERMIT_PLUGINS
-      else if (plugin_window != NULL) {
-         DEBUG_MSG("Winching plugin windows...");
+      else if (plugin_window != NULL)
+      {
+         #ifdef DEBUG
+            Debug_msg("Winching plugin windows...");
+         #endif
          Interface_Plugins_Winch();
       }
 #endif
-      else {
-         DEBUG_MSG("Winching windows...");
+      else
+      {
+         #ifdef DEBUG
+            Debug_msg("Winching windows...");
+         #endif
          Interface_Winch();
       }
    }
@@ -191,7 +238,25 @@ RETSIGTYPE Signal_WINCH(int sig)
 }
 
 
+
+RETSIGTYPE Signal_USR1(int sig)
+{
+   active_dissector = (active_dissector) ? 0 : 1;     // activate/deactivate active dissector (arp based)
+   #ifdef DEBUG
+      Debug_msg("\tactive_dissector %d", active_dissector);
+   #endif
+   signal(SIGUSR1, Signal_USR1);
+}
+
+
+
+RETSIGTYPE Signal_USR2(int sig)
+{
+   Connection_Mode = (Connection_Mode) ? 0 : 1;
+   #ifdef DEBUG
+      Debug_msg("\tConnection_Mode %d", Connection_Mode);
+   #endif
+   signal(SIGUSR2, Signal_USR2);
+}
+
 /* EOF */
-
-// vim:ts=3:expandtab
-
