@@ -14,6 +14,10 @@ extern int Inet_CheckSwitch(void);
 
 extern int Inet_Fake_Host(void);
 
+extern int Inet_OpenSocket(char *host, short port);
+extern int Inet_Http_Send(int s, char *payload);
+extern int Inet_Http_Receive(int s, char *payload);
+
 // Following are architecture dependent !! implementations are in ./src/`uname`/ec_inet_`uname`.c
 
 extern int Inet_FindIFace(char *iface);
@@ -23,6 +27,7 @@ extern int Inet_GetIfaceInfo(char *iface, int *MTU, char *MyMAC, unsigned long *
 extern int Inet_SetPromisc(char *iface);
 
 extern int Inet_OpenRawSock(char *iface);
+extern void Inet_CloseRawSock(int sock);
 extern int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type);
 extern int Inet_SendRawPacket(int sock, char *buffer, int len);
 extern int Inet_SetARPEntry(unsigned long IP, char MAC[6]);
